@@ -22,27 +22,31 @@ vertices, edges or facets, or a user-provided tuple defintion"""
 # Modified by Anders Logg, 2010.
 #
 # First added:  2010-11-16
-# Last changed: 2010-11-17
+# Last changed: 2012-11-12
 
 from dolfin import *
 
 # Create mesh
-mesh = UnitCube(24, 24, 24)
+mesh = UnitCubeMesh(24, 24, 24)
 
 # Compute vertex-based coloring
-colors = mesh.color("vertex")
+mesh.color("vertex");
+colors = MeshColoring.cell_colors(mesh, "vertex")
 plot(colors, title="Vertex-based cell coloring", interactive=True)
 
 # Compute edge-based coloring
-colors = mesh.color("edge")
+mesh.color("edge");
+colors = MeshColoring.cell_colors(mesh, "edge")
 plot(colors, title="Edge-based cell coloring", interactive=True)
 
 # Compute facet-based coloring
-colors = mesh.color("facet")
+mesh.color("facet");
+colors = MeshColoring.cell_colors(mesh, "facet")
 plot(colors, title="Facet-based cell coloring", interactive=True)
 
 # Compute facet-based coloring with distance 2
 dim = mesh.topology().dim()
 coloring_type = (dim, dim - 1, dim, dim - 1, dim)
-colors = mesh.color(coloring_type)
+mesh.color(coloring_type);
+colors = MeshColoring.cell_colors(mesh, coloring_type)
 plot(colors, title="Facet-based cell coloring with distance 2", interactive=True)
