@@ -94,7 +94,7 @@ int main()
   };
 
   // Read mesh and create function space
-  Mesh mesh("gear.xml.gz");
+  Mesh mesh("../gear.xml.gz");
   Elasticity::Form_a::TestSpace V(mesh);
 
   // Create right-hand side
@@ -111,7 +111,7 @@ int main()
   DirichletBC bcr(V, r, right);
 
   // Collect boundary conditions
-  std::vector<const BoundaryCondition*> bcs;
+  std::vector<const DirichletBC*> bcs;
   bcs.push_back(&bcl);
   bcs.push_back(&bcr);
 
@@ -168,9 +168,10 @@ int main()
   file_stress << stress;
 
   // Save colored mesh paritions in VTK format if running in parallel
-  if (dolfin::MPI::num_processes() > 1)
+  if (dolfin::MPI::size(mesh.mpi_comm()) > 1)
   {
-    CellFunction<std::size_t> partitions(mesh, dolfin::MPI::process_number());
+    CellFunction<std::size_t>
+      partitions(mesh, dolfin::MPI::rank(mesh.mpi_comm()));
     File file("partitions.pvd");
     file << partitions;
   }
