@@ -74,7 +74,7 @@ mu    = E / (2.0*(1.0 + nu))
 lmbda = E*nu / ((1.0 + nu)*(1.0 - 2.0*nu))
 
 def sigma(v):
-    return 2.0*mu*sym(grad(v)) + lmbda*tr(sym(grad(v)))*Identity(v.cell().d)
+    return 2.0*mu*sym(grad(v)) + lmbda*tr(sym(grad(v)))*Identity(v.geometric_dimension())
 
 a = inner(sigma(u), grad(v))*dx
 L = inner(f, v)*dx
@@ -98,8 +98,9 @@ solve(a == L, u, bcs, solver_parameters={"symmetric": True})
 File("elasticity.pvd", "compressed") << u
 
 # Save colored mesh partitions in VTK format if running in parallel
-if MPI.num_processes() > 1:
-    File("partitions.pvd") << CellFunction("uint", mesh, MPI.process_number())
+if MPI.size(mesh.mpi_comm()) > 1:
+    File("partitions.pvd") << CellFunction("size_t", mesh, \
+                                           MPI.rank(mesh.mpi_comm()))
 
 # Project and write stress field to post-processing file
 W = TensorFunctionSpace(mesh, "Discontinuous Lagrange", 0)
