@@ -35,11 +35,11 @@ forces).
 from dolfin import *
 
 # Read the mesh from file
-mesh =  Mesh("../mesh.xml.gz")
+mesh =  Mesh("../dolfin_fine.xml.gz")
 
 # Create FunctionSpace for pressure field
 Vp = FunctionSpace(mesh, "CG", 1)
-p = Function(Vp, "../pressure.xml.gz")
+p = Function(Vp, "../dolfin_fine_pressure.xml.gz")
 
 # Define sub domain for the dolphin
 class Fish(SubDomain):
@@ -47,15 +47,18 @@ class Fish(SubDomain):
         return x[0] > DOLFIN_EPS and x[0] < (1.0 - DOLFIN_EPS) and \
                x[1] > DOLFIN_EPS and x[1] < (1.0 - DOLFIN_EPS)
 
+markers = FacetFunctionSizet(mesh, 1)
+Fish().mark(markers, 1);
+
 # Define functionals for drag and lift
+ds = ds[markers]
 n = FacetNormal(mesh)
-D = -p*n[0]*ds
-L = p*n[1]*ds
+D = -p*n[0]*ds(1)
+L = p*n[1]*ds(1)
 
 # Assemble functionals over sub domain
-fish = Fish()
-drag = assemble(D, mesh=mesh, exterior_facet_domains=fish)
-lift = assemble(L, mesh=mesh, exterior_facet_domains=fish)
+drag = assemble(D)
+lift = assemble(L)
 
-print "Lift: %f" %lift
-print "Drag: %f" %drag
+print "Lift: %f" % lift
+print "Drag: %f" % drag
