@@ -32,7 +32,9 @@ print 'Maximal radius ratio:', qmax
 
 # Show histogram using matplotlib
 hist = MeshQuality.radius_ratio_matplotlib_histogram(mesh)
-hist = hist.replace('pylab.show()', 'pylab.show(block=False)')
+hist = hist.replace('import pylab', 'import matplotlib\n    matplotlib.use(\'Agg\')\n    import pylab')
+hist = hist.replace('pylab.show()', 'pylab.savefig("mesh-quality.pdf")')
+print hist
 exec(hist)
 
 # Show mesh
