@@ -48,7 +48,7 @@ class DirichletBoundary(SubDomain):
 bc = DirichletBC(P1, u0, DirichletBoundary())
 
 # Goal functional
-J = (u - u0)*(u - u0)*dx
+J = (u - u0)*(u - u0)*dx(mesh)
 
 # Forward problem
 problem = (inner(grad(v1), p*grad(w1))*dx, v1*f*dx)
@@ -84,7 +84,7 @@ for i in range(100):
     px -= dp
 
     # Print value of functional
-    jval = assemble(J, mesh = mesh)
+    jval = assemble(J)
     print "J = ", jval
     print u.vector().max()
 
