@@ -16,7 +16,7 @@
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2009-11-11
-# Last changed: 2011-02-25
+# Last changed: 2012-11-12
 
 from dolfin import *
 from numpy import ones
@@ -25,18 +25,18 @@ from numpy import ones
 series = TimeSeries("primal")
 
 # Create a mesh and a vector
-mesh = UnitSquare(2, 2)
+mesh = UnitSquareMesh(2, 2)
 x = Vector()
 
 # Add a bunch of meshes and vectors to the series
 t = 0.0
 while t < 1.0:
 
-    # Refine mesh and resize vector
+    # Refine mesh
     mesh = refine(mesh);
-    x.resize(mesh.num_vertices());
 
     # Set some vector values
+    x = Vector(mesh.mpi_comm(), mesh.num_vertices());
     x[:] = ones(x.size())
 
     # Append to series
@@ -47,6 +47,7 @@ while t < 1.0:
 
 # Retrieve mesh and vector at some point in time
 series.retrieve(mesh, 0.29)
+x = Vector()
 series.retrieve(x, 0.31, False)
 
 # Plot mesh
