@@ -20,9 +20,6 @@
 // Modified by Ola Skavhaug 2008-2009
 // Modified by Niclas Jansson 2009
 // Modified by Joachim B Haga 2012
-//
-// First added:  2007-11-30
-// Last changed: 2012-11-17
 
 #include <numeric>
 #include <dolfin/log/dolfin_log.h>
@@ -47,100 +44,120 @@ MPI_Info& dolfin::MPIInfo::operator*()
   return info;
 }
 //-----------------------------------------------------------------------------
-
-
-//-----------------------------------------------------------------------------
-dolfin::MPICommunicator::MPICommunicator()
-{
-  MPI_Comm_dup(MPI_COMM_WORLD, &communicator);
-}
-//-----------------------------------------------------------------------------
-dolfin::MPICommunicator::~MPICommunicator()
-{
-  MPI_Comm_free(&communicator);
-}
-//-----------------------------------------------------------------------------
-MPI_Comm& dolfin::MPICommunicator::operator*()
-{
-  return communicator;
-}
-//-----------------------------------------------------------------------------
-//-----------------------------------------------------------------------------
-void dolfin::MPINonblocking::wait_all()
-{
-  if (!reqs.empty())
-  {
-    boost::mpi::wait_all(reqs.begin(), reqs.end());
-    reqs.clear();
-  }
-}
+#endif
 //-----------------------------------------------------------------------------
 //-----------------------------------------------------------------------------
 unsigned int dolfin::MPI::process_number()
 {
+  deprecation("MPI::process_number",
+              "1.4", "1.5",
+              "MPI::process_number() has been replaced by MPI::rank(MPI_Comm).");
+
+#ifdef HAS_MPI
   SubSystemsManager::init_mpi();
   int rank;
   MPI_Comm_rank(MPI_COMM_WORLD, &rank);
   return rank;
+#else
+  return 0;
+#endif
 }
 //-----------------------------------------------------------------------------
 unsigned int dolfin::MPI::num_processes()
 {
+  deprecation("MPI::num_processes",
+              "1.4", "1.5",
+              "MPI::num_processes() has been replaced by MPI::size(MPI_Comm).");
+
+#ifdef HAS_MPI
   SubSystemsManager::init_mpi();
   int size;
   MPI_Comm_size(MPI_COMM_WORLD, &size);
   return size;
+#else
+  return 1;
+#endif
 }
 //-----------------------------------------------------------------------------
-bool dolfin::MPI::is_broadcaster()
+unsigned int dolfin::MPI::rank(const MPI_Comm comm)
+{
+#ifdef HAS_MPI
+  SubSystemsManager::init_mpi();
+  int rank;
+  MPI_Comm_rank(comm, &rank);
+  return rank;
+#else
+  return 0;
+#endif
+}
+//-----------------------------------------------------------------------------
+unsigned int dolfin::MPI::size(const MPI_Comm comm)
+{
+#ifdef HAS_MPI
+  SubSystemsManager::init_mpi();
+  int size;
+  MPI_Comm_size(comm, &size);
+  return size;
+#else
+  return 1;
+#endif
+}
+//-----------------------------------------------------------------------------
+bool dolfin::MPI::is_broadcaster(const MPI_Comm comm)
 {
   // Always broadcast from processor number 0
-  return num_processes() > 1 && process_number() == 0;
+  return size(comm) > 1 && rank(comm) == 0;
 }
 //-----------------------------------------------------------------------------
-bool dolfin::MPI::is_receiver()
+bool dolfin::MPI::is_receiver(const MPI_Comm comm)
 {
   // Always receive on processors with numbers > 0
-  return num_processes() > 1 && process_number() > 0;
+  return size(comm) > 1 && rank(comm) > 0;
 }
 //-----------------------------------------------------------------------------
-void dolfin::MPI::barrier()
+void dolfin::MPI::barrier(const MPI_Comm comm)
 {
-  MPICommunicator comm;
-  MPI_Barrier(*comm);
+#ifdef HAS_MPI
+  MPI_Barrier(comm);
+#endif
 }
 //-----------------------------------------------------------------------------
-std::size_t dolfin::MPI::global_offset(std::size_t range, bool exclusive)
+std::size_t dolfin::MPI::global_offset(const MPI_Comm comm,
+                                       std::size_t range, bool exclusive)
 {
-  MPICommunicator mpi_comm;
-  boost::mpi::communicator comm(*mpi_comm, boost::mpi::comm_duplicate);
-
+#ifdef HAS_MPI
   // Compute inclusive or exclusive partial reduction
-  std::size_t offset = boost::mpi::scan(comm, range, std::plus<std::size_t>());
+  std::size_t offset = 0;
+  MPI_Scan(&range, &offset, 1, mpi_type<std::size_t>(), MPI_SUM, comm);
   if (exclusive)
     offset -= range;
-
   return offset;
+#else
+  return 0;
+#endif
 }
 //-----------------------------------------------------------------------------
-std::pair<std::size_t, std::size_t> dolfin::MPI::local_range(std::size_t N)
+std::pair<std::size_t, std::size_t>
+dolfin::MPI::local_range(const MPI_Comm comm, std::size_t N)
 {
-  return local_range(process_number(), N);
+  return local_range(comm, rank(comm), N);
 }
 //-----------------------------------------------------------------------------
-std::pair<std::size_t, std::size_t> dolfin::MPI::local_range(unsigned int process,
-                                                             std::size_t N)
+std::pair<std::size_t, std::size_t>
+dolfin::MPI::local_range(const MPI_Comm comm, unsigned int process,
+                         std::size_t N)
 {
-  return local_range(process, N, num_processes());
+  return compute_local_range(process, N, size(comm));
 }
 //-----------------------------------------------------------------------------
-std::pair<std::size_t, std::size_t> dolfin::MPI::local_range(unsigned int process,
-                                                             std::size_t N,
-                                                             unsigned int num_processes)
+std::pair<std::size_t, std::size_t>
+dolfin::MPI::compute_local_range(unsigned int process,
+                                 std::size_t N,
+                                 unsigned int size)
 {
   // Compute number of items per process and remainder
-  const std::size_t n = N / num_processes;
-  const std::size_t r = N % num_processes;
+  const std::size_t n = N / size;
+  const std::size_t r = N % size;
 
   // Compute local range
   std::pair<std::size_t, std::size_t> range;
@@ -158,16 +175,17 @@ std::pair<std::size_t, std::size_t> dolfin::MPI::local_range(unsigned int proces
   return range;
 }
 //-----------------------------------------------------------------------------
-unsigned int dolfin::MPI::index_owner(std::size_t index, std::size_t N)
+unsigned int dolfin::MPI::index_owner(const MPI_Comm comm,
+                                      std::size_t index, std::size_t N)
 {
   dolfin_assert(index < N);
 
   // Get number of processes
-  const unsigned int _num_processes = num_processes();
+  const unsigned int _size = size(comm);
 
   // Compute number of items per process and remainder
-  const std::size_t n = N / _num_processes;
-  const std::size_t r = N % _num_processes;
+  const std::size_t n = N / _size;
+  const std::size_t r = N % _size;
 
   // First r processes own n + 1 indices
   if (index < r * (n + 1))
@@ -177,84 +195,3 @@ unsigned int dolfin::MPI::index_owner(std::size_t index, std::size_t N)
   return r + (index - r * (n + 1)) / n;
 }
 //-----------------------------------------------------------------------------
-//-----------------------------------------------------------------------------
-#else
-//-----------------------------------------------------------------------------
-//-----------------------------------------------------------------------------
-void dolfin::MPINonblocking::wait_all()
-{
-  dolfin_error("MPI.h",
-               "call MPINonblocking::wait_all",
-               "DOLFIN has been configured without MPI support");
-}
-//-----------------------------------------------------------------------------
-//-----------------------------------------------------------------------------
-unsigned int dolfin::MPI::process_number()
-{
-  return 0;
-}
-//-----------------------------------------------------------------------------
-unsigned int dolfin::MPI::num_processes()
-{
-  return 1;
-}
-//-----------------------------------------------------------------------------
-bool dolfin::MPI::is_broadcaster()
-{
-  return false;
-}
-//-----------------------------------------------------------------------------
-bool dolfin::MPI::is_receiver()
-{
-  return false;
-}
-//-----------------------------------------------------------------------------
-void dolfin::MPI::barrier()
-{
-  dolfin_error("MPI.cpp",
-               "call MPI::barrier",
-               "Your DOLFIN installation has been built without MPI support");
-}
-//-----------------------------------------------------------------------------
-std::size_t dolfin::MPI::global_offset(std::size_t range, bool exclusive)
-{
-  return 0;
-}
-//-----------------------------------------------------------------------------
-std::pair<std::size_t, std::size_t> dolfin::MPI::local_range(std::size_t N)
-{
-  return std::make_pair(0, N);
-}
-//-----------------------------------------------------------------------------
-std::pair<std::size_t, std::size_t> dolfin::MPI::local_range(unsigned int process,
-                                                             std::size_t N)
-{
-  if (process != 0 || num_processes() > 1)
-  {
-    dolfin_error("MPI.cpp",
-                 "access local range for process",
-                 "DOLFIN has not been configured with MPI support");
-  }
-  return std::make_pair(0, N);
-}
-//-----------------------------------------------------------------------------
-std::pair<std::size_t, std::size_t>
-  dolfin::MPI::local_range(unsigned int process, std::size_t N,
-                           unsigned int num_processes)
-{
-  if (process != 0 || num_processes > 1)
-  {
-    dolfin_error("MPI.cpp",
-                 "access local range for process",
-                 "DOLFIN has not been configured with MPI support");
-  }
-  return std::make_pair(0, N);
-}
-//-----------------------------------------------------------------------------
-unsigned int dolfin::MPI::index_owner(std::size_t i, std::size_t N)
-{
-  dolfin_assert(i < N);
-  return 0;
-}
-//-----------------------------------------------------------------------------
-#endif
