@@ -4,8 +4,8 @@
 // DOLFIN common
 
 #include <dolfin/common/init.h>
+#include <dolfin/common/defines.h>
 #include <dolfin/common/types.h>
-#include <dolfin/common/real.h>
 #include <dolfin/common/constants.h>
 #include <dolfin/common/timing.h>
 #include <dolfin/common/Array.h>
