@@ -23,7 +23,8 @@
 // First added:  2007-01-17
 // Last changed: 2013-09-19
 
-#include <boost/scoped_ptr.hpp>
+#include <memory>
+
 #include <dolfin/common/Timer.h>
 #include <dolfin/function/FunctionSpace.h>
 #include <dolfin/function/GenericFunction.h>
@@ -50,19 +51,17 @@ void AssemblerBase::init_global_tensor(GenericTensor& A, const Form& a)
 {
   dolfin_assert(a.ufc_form());
 
-  check_parameters();
-
   // Get dof maps
   std::vector<const GenericDofMap*> dofmaps;
   for (std::size_t i = 0; i < a.rank(); ++i)
     dofmaps.push_back(a.function_space(i)->dofmap().get());
 
-  if (reset_sparsity)
+  if (A.empty())
   {
     Timer t0("Build sparsity");
 
     // Create layout for initialising tensor
-    boost::shared_ptr<TensorLayout> tensor_layout;
+    std::shared_ptr<TensorLayout> tensor_layout;
     tensor_layout = A.factory().create_layout(a.rank());
     dolfin_assert(tensor_layout);
 
@@ -87,7 +86,8 @@ void AssemblerBase::init_global_tensor(GenericTensor& A, const Form& a)
     }
 
     // Initialise tensor layout
-    tensor_layout->init(global_dimensions, block_size, local_range);
+    tensor_layout->init(a.mesh().mpi_comm(), global_dimensions, block_size,
+                        local_range);
 
     // Build sparsity pattern if required
     if (tensor_layout->sparsity_pattern())
@@ -114,7 +114,7 @@ void AssemblerBase::init_global_tensor(GenericTensor& A, const Form& a)
     if (A.rank() == 2 && keep_diagonal)
     {
       // Down cast to GenericMatrix
-      GenericMatrix& _A = A.down_cast<GenericMatrix>();
+      GenericMatrix& _matA = A.down_cast<GenericMatrix>();
 
       // Loop over rows and insert 0.0 on the diagonal
       const double block = 0.0;
@@ -123,7 +123,7 @@ void AssemblerBase::init_global_tensor(GenericTensor& A, const Form& a)
       for (std::size_t i = row_range.first; i < range; i++)
       {
         dolfin::la_index _i = i;
-        _A.set(&block, 1, &_i, 1, &_i);
+        _matA.set(&block, 1, &_i, 1, &_i);
       }
       A.apply("flush");
     }
@@ -150,23 +150,6 @@ void AssemblerBase::init_global_tensor(GenericTensor& A, const Form& a)
     A.zero();
 }
 //-----------------------------------------------------------------------------
-void AssemblerBase::check_parameters() const
-{
-  if (reset_sparsity && add_values)
-  {
-    dolfin_error("AssemblerBase.cpp",
-                 "check parameters",
-                 "Can not add values when the sparsity pattern is reset");
-  }
-
-  if (!reset_sparsity && keep_diagonal)
-  {
-    dolfin_error("AssemblerBase.cpp",
-                 "check parameters",
-                 "Not resetting tensor and keeping diagonal entries are incompatible");
-  }
-}
-//-----------------------------------------------------------------------------
 void AssemblerBase::check(const Form& a)
 {
   dolfin_assert(a.ufc_form());
@@ -176,7 +159,7 @@ void AssemblerBase::check(const Form& a)
 
   // Extract mesh and coefficients
   const Mesh& mesh = a.mesh();
-  const std::vector<boost::shared_ptr<const GenericFunction> >
+  const std::vector<std::shared_ptr<const GenericFunction> >
     coefficients = a.coefficients();
 
   // Check that we get the correct number of coefficients
@@ -199,8 +182,8 @@ void AssemblerBase::check(const Form& a)
                    i, a.coefficient_name(i).c_str());
     }
 
-    // auto_ptr deletes its object when it exits its scope
-    boost::scoped_ptr<ufc::finite_element> fe(a.ufc_form()->create_finite_element(i + a.rank()));
+    // unique_ptr deletes its object when it exits its scope
+    std::unique_ptr<ufc::finite_element> fe(a.ufc_form()->create_finite_element(i + a.rank()));
 
     // Checks out-commented since they only work for Functions, not Expressions
     const std::size_t r = coefficients[i]->value_rank();
@@ -230,7 +213,7 @@ You might have forgotten to specify the value dimension correctly in an Expressi
   // Check that the cell dimension matches the mesh dimension
   if (a.rank() + a.ufc_form()->num_coefficients() > 0)
   {
-    boost::scoped_ptr<ufc::finite_element> element(a.ufc_form()->create_finite_element(0));
+    std::unique_ptr<ufc::finite_element> element(a.ufc_form()->create_finite_element(0));
     dolfin_assert(element);
     if (mesh.type().cell_type() == CellType::interval && element->cell_shape() != ufc::interval)
     {
