@@ -22,7 +22,7 @@
 // Modified by Mikael Mortensen, 2013
 //
 // First added:  2007-04-10
-// Last changed: 2013-06-19
+// Last changed: 2014-01-23
 
 #include <map>
 #include <utility>
@@ -78,9 +78,9 @@ DirichletBC::DirichletBC(const FunctionSpace& V,
   parameters = default_parameters();
 }
 //-----------------------------------------------------------------------------
-DirichletBC::DirichletBC(boost::shared_ptr<const FunctionSpace> V,
-                         boost::shared_ptr<const GenericFunction> g,
-                         boost::shared_ptr<const SubDomain> sub_domain,
+DirichletBC::DirichletBC(std::shared_ptr<const FunctionSpace> V,
+                         std::shared_ptr<const GenericFunction> g,
+                         std::shared_ptr<const SubDomain> sub_domain,
                          std::string method,
                          bool check_midpoint)
   : Hierarchical<DirichletBC>(*this),
@@ -108,9 +108,9 @@ DirichletBC::DirichletBC(const FunctionSpace& V, const GenericFunction& g,
   parameters = default_parameters();
 }
 //-----------------------------------------------------------------------------
-DirichletBC::DirichletBC(boost::shared_ptr<const FunctionSpace> V,
-                         boost::shared_ptr<const GenericFunction> g,
-                         boost::shared_ptr<const MeshFunction<std::size_t> > sub_domains,
+DirichletBC::DirichletBC(std::shared_ptr<const FunctionSpace> V,
+                         std::shared_ptr<const GenericFunction> g,
+                         std::shared_ptr<const MeshFunction<std::size_t> > sub_domains,
                          std::size_t sub_domain,
                          std::string method)
   : Hierarchical<DirichletBC>(*this), _function_space(V), _g(g),
@@ -134,8 +134,8 @@ DirichletBC::DirichletBC(const FunctionSpace& V, const GenericFunction& g,
   parameters = default_parameters();
 }
 //-----------------------------------------------------------------------------
-DirichletBC::DirichletBC(boost::shared_ptr<const FunctionSpace> V,
-                         boost::shared_ptr<const GenericFunction> g,
+DirichletBC::DirichletBC(std::shared_ptr<const FunctionSpace> V,
+                         std::shared_ptr<const GenericFunction> g,
                          std::size_t sub_domain, std::string method)
   : Hierarchical<DirichletBC>(*this),
     _function_space(V),
@@ -148,8 +148,8 @@ DirichletBC::DirichletBC(boost::shared_ptr<const FunctionSpace> V,
   parameters = default_parameters();
 }
 //-----------------------------------------------------------------------------
-DirichletBC::DirichletBC(boost::shared_ptr<const FunctionSpace> V,
-                         boost::shared_ptr<const GenericFunction> g,
+DirichletBC::DirichletBC(std::shared_ptr<const FunctionSpace> V,
+                         std::shared_ptr<const GenericFunction> g,
                          const std::vector<std::size_t>& markers,
                          std::string method)
   : Hierarchical<DirichletBC>(*this),
@@ -222,21 +222,23 @@ void DirichletBC::gather(Map& boundary_values) const
 {
   Timer timer("DirichletBC gather");
 
-  typedef std::vector<std::pair<std::size_t, double> > bv_vec_type;
-  typedef std::map<std::size_t, bv_vec_type> map_type;
-
   typedef boost::unordered_map<std::size_t, std::vector<unsigned int> >
     shared_dof_type;
   typedef shared_dof_type::const_iterator shared_dof_iterator;
   typedef std::vector<unsigned int>::const_iterator proc_iterator;
 
+  dolfin_assert(_function_space->mesh());
+  MPI_Comm mpi_comm = _function_space->mesh()->mpi_comm();
+  std::size_t comm_size = MPI::size(mpi_comm);
+
+  // Get dofmap
   dolfin_assert(_function_space->dofmap());
   const GenericDofMap& dofmap = *_function_space->dofmap();
   const shared_dof_type& shared_dofs = dofmap.shared_dofs();
 
   // Create list of boundary values to send to each processor
-
-  map_type proc_map;
+  std::vector<std::vector<std::size_t> > proc_map0(comm_size);
+  std::vector<std::vector<double> > proc_map1(comm_size);
   for (Map::const_iterator bv = boundary_values.begin();
        bv != boundary_values.end(); ++bv)
   {
@@ -249,21 +251,29 @@ void DirichletBC::gather(Map& boundary_values) const
       for (proc_iterator proc = shared_dof->second.begin();
            proc != shared_dof->second.end(); ++proc)
       {
-        proc_map[*proc].push_back(*bv);
+        proc_map0[*proc].push_back(bv->first);
+        proc_map1[*proc].push_back(bv->second);
       }
     }
   }
 
   // Distribute the lists between neighbours
-  map_type received_bvs;
-  MPI::distribute(dofmap.neighbours(), proc_map, received_bvs);
+  std::vector<std::vector<std::size_t> > received_bvc0;
+  std::vector<std::vector<double> > received_bvc1;
+  MPI::all_to_all(mpi_comm, proc_map0, received_bvc0);
+  MPI::all_to_all(mpi_comm, proc_map1, received_bvc1);
 
   // Add the received boundary values to the local boundary values
-
-  for (map_type::const_iterator it = received_bvs.begin();
-       it != received_bvs.end(); ++it)
+  for (std::size_t p = 0; p < comm_size; ++p)
   {
-    boundary_values.insert(it->second.begin(), it->second.end());
+    dolfin_assert(received_bvc0[p].size() == received_bvc1[p].size());
+    std::vector<std::pair<std::size_t, double> > _vec(received_bvc0[p].size());
+    for (std::size_t i = 0; i < _vec.size(); ++i)
+    {
+      _vec[i].first  = received_bvc0[p][i];
+      _vec[i].second = received_bvc1[p][i];
+    }
+    boundary_values.insert(_vec.begin(), _vec.end());
   }
 }
 //-----------------------------------------------------------------------------
@@ -385,12 +395,12 @@ const std::vector<std::size_t>& DirichletBC::markers() const
   return _facets;
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<const GenericFunction> DirichletBC::value() const
+std::shared_ptr<const GenericFunction> DirichletBC::value() const
 {
   return _g;
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<const SubDomain> DirichletBC::user_sub_domain() const
+std::shared_ptr<const SubDomain> DirichletBC::user_sub_domain() const
 {
   return _user_sub_domain;
 }
@@ -474,14 +484,14 @@ void DirichletBC::homogenize()
   const std::size_t value_rank = _g->value_rank();
   if (!value_rank)
   {
-    boost::shared_ptr<Constant> zero(new Constant(0.0));
+    std::shared_ptr<Constant> zero(new Constant(0.0));
     set_value(zero);
   }
   else if (value_rank == 1)
   {
     const std::size_t value_dim = _g->value_dimension(0);
     std::vector<double> values(value_dim, 0.0);
-    boost::shared_ptr<Constant> zero(new Constant(values));
+    std::shared_ptr<Constant> zero(new Constant(values));
     set_value(zero);
   }
   else
@@ -490,12 +500,12 @@ void DirichletBC::homogenize()
     for (std::size_t i = 0; i < value_rank; i++)
       value_shape.push_back(_g->value_dimension(i));
     std::vector<double> values(_g->value_size(), 0.0);
-    boost::shared_ptr<Constant> zero(new Constant(value_shape, values));
+    std::shared_ptr<Constant> zero(new Constant(value_shape, values));
     set_value(zero);
   }
 }
 //-----------------------------------------------------------------------------
-void DirichletBC::set_value(boost::shared_ptr<const GenericFunction> g)
+void DirichletBC::set_value(std::shared_ptr<const GenericFunction> g)
 {
   _g = g;
 }
@@ -635,11 +645,11 @@ void DirichletBC::check() const
   }
 }
 //-----------------------------------------------------------------------------
-void DirichletBC::init_facets() const
+void DirichletBC::init_facets(const MPI_Comm mpi_comm) const
 {
   Timer timer("DirichletBC init facets");
 
-  if (MPI::max(_facets.size()) > 0)
+  if (MPI::max(mpi_comm, _facets.size()) > 0)
     return;
 
   if (_user_sub_domain)
@@ -650,7 +660,7 @@ void DirichletBC::init_facets() const
     init_from_mesh(_user_sub_domain_marker);
 }
 //-----------------------------------------------------------------------------
-void DirichletBC::init_from_sub_domain(boost::shared_ptr<const SubDomain>
+void DirichletBC::init_from_sub_domain(std::shared_ptr<const SubDomain>
                                        sub_domain) const
 {
   dolfin_assert(_facets.size() == 0);
@@ -758,21 +768,23 @@ void DirichletBC::compute_bc_topological(Map& boundary_values,
   dolfin_assert(_function_space);
   dolfin_assert(_g);
 
+  // Get mesh and dofmap
+  dolfin_assert(_function_space->mesh());
+  const Mesh& mesh = *_function_space->mesh();
+
   // Extract the list of facets where the BC should be applied
-  init_facets();
+  init_facets(mesh.mpi_comm());
 
   // Special case
   if (_facets.empty())
   {
-    if (MPI::num_processes() == 1)
+    if (MPI::size(mesh.mpi_comm()) == 1)
       warning("Found no facets matching domain for boundary condition.");
     return;
   }
 
-  // Get mesh and dofmap
-  dolfin_assert(_function_space->mesh());
+  // Get dofmap
   dolfin_assert(_function_space->dofmap());
-  const Mesh& mesh = *_function_space->mesh();
   const GenericDofMap& dofmap = *_function_space->dofmap();
 
   // Topological dimension
@@ -783,7 +795,7 @@ void DirichletBC::compute_bc_topological(Map& boundary_values,
   mesh.init(D - 1, D);
 
   // Get restriction if any
-  boost::shared_ptr<const Restriction> restriction
+  std::shared_ptr<const Restriction> restriction
     = _function_space->dofmap()->restriction();
 
   // Create UFC cell
@@ -859,21 +871,23 @@ void DirichletBC::compute_bc_geometric(Map& boundary_values,
   dolfin_assert(_function_space->element());
   dolfin_assert(_g);
 
+  // Get mesh
+  dolfin_assert(_function_space->mesh());
+  const Mesh& mesh = *_function_space->mesh();
+
   // Extract the list of facets where the BC *might* be applied
-  init_facets();
+  init_facets(mesh.mpi_comm());
 
   // Special case
   if (_facets.empty())
   {
-    if (MPI::num_processes() == 1)
+    if (MPI::size(mesh.mpi_comm()) == 1)
       warning("Found no facets matching domain for boundary condition.");
     return;
   }
 
-  // Get mesh and dofmap
-  dolfin_assert(_function_space->mesh());
+  // Get dofmap
   dolfin_assert(_function_space->dofmap());
-  const Mesh& mesh = *_function_space->mesh();
   const GenericDofMap& dofmap = *_function_space->dofmap();
 
   // Initialize facets, needed for geometric search
@@ -995,59 +1009,106 @@ void DirichletBC::compute_bc_pointwise(Map& boundary_values,
                                  : dofmap.ownership_range());
 
   // Iterate over cells
-  Progress p("Computing Dirichlet boundary values, pointwise search",
-             mesh.num_cells());
   std::vector<double> vertex_coordinates;
-  for (CellIterator cell(mesh); !cell.end(); ++cell)
+  if (MPI::max(mesh.mpi_comm(), _cells_to_localdofs.size()) == 0)
   {
-    // Update UFC cell
-    cell->get_vertex_coordinates(vertex_coordinates);
-    cell->get_cell_data(ufc_cell);
-
-    // Tabulate coordinates of dofs on cell
-    dofmap.tabulate_coordinates(data.coordinates, vertex_coordinates,
-                                *cell);
-
-    // Tabulate dofs on cell
-    const std::vector<dolfin::la_index>& cell_dofs
-      = dofmap.cell_dofs(cell->index());
-
-    // Interpolate function only once and only on cells where necessary
-    bool already_interpolated = false;
-
-    // Loop all dofs on cell
-    for (std::size_t i = 0; i < dofmap.cell_dimension(cell->index()); ++i)
+    // First time around all cells must be iterated over.
+    // Create map from cells attached to boundary to local dofs.
+    Progress p("Computing Dirichlet boundary values, pointwise search",
+               mesh.num_cells());
+    for (CellIterator cell(mesh); !cell.end(); ++cell)
     {
-      const std::size_t global_dof = cell_dofs[i];
+      // Update UFC cell
+      cell->get_vertex_coordinates(vertex_coordinates);
+      cell->get_cell_data(ufc_cell);
 
-      // Skip already checked dofs
-      if (already_visited.in_range(global_dof)
-          && !already_visited.insert(global_dof))
+      // Tabulate coordinates of dofs on cell
+      dofmap.tabulate_coordinates(data.coordinates, vertex_coordinates,
+                                  *cell);
+
+      // Tabulate dofs on cell
+      const std::vector<dolfin::la_index>& cell_dofs
+        = dofmap.cell_dofs(cell->index());
+
+      // Interpolate function only once and only on cells where necessary
+      bool already_interpolated = false;
+
+      std::vector<std::size_t> dofs;
+
+      // Loop all dofs on cell
+      for (std::size_t i = 0; i < dofmap.cell_dimension(cell->index()); ++i)
       {
-        continue;
+        const std::size_t global_dof = cell_dofs[i];
+
+        // Skip already checked dofs
+        if (already_visited.in_range(global_dof)
+            && !already_visited.insert(global_dof))
+        {
+          continue;
+        }
+
+        // Check if the coordinates are part of the sub domain (calls
+        // user-defined 'inside' function)
+        Array<double> x(gdim, &data.coordinates[i][0]);
+        if (!_user_sub_domain->inside(x, false))
+          continue;
+
+        if (!already_interpolated)
+        {
+          already_interpolated = true;
+
+          // Restrict coefficient to cell
+          _g->restrict(data.w.data(), *_function_space->element(), *cell,
+                      vertex_coordinates.data(), ufc_cell);
+
+          // Put cell index in storage for next time function is called
+          _cells_to_localdofs.insert(std::make_pair(cell->index(), dofs));
+        }
+
+        // Add local dof to map
+        _cells_to_localdofs[cell->index()].push_back(i);
+
+        // Set boundary value
+        const double value = data.w[i];
+        boundary_values[global_dof] = value;
       }
-
-      // Check if the coordinates are part of the sub domain (calls
-      // user-defined 'inside' function)
-      Array<double> x(gdim, &data.coordinates[i][0]);
-      if (!_user_sub_domain->inside(x, false))
-        continue;
-
-      if (!already_interpolated)
-      {
-        already_interpolated = true;
-
-        // Restrict coefficient to cell
-        _g->restrict(data.w.data(), *_function_space->element(), *cell,
-                     vertex_coordinates.data(), ufc_cell);
-      }
-
-      // Set boundary value
-      const double value = data.w[i];
-      boundary_values[global_dof] = value;
+      p++;
     }
+  }
+  else
+  {
+    // Loop over cells that contain dofs on boundary.
+    std::map<std::size_t, std::vector<std::size_t> >::const_iterator it;
+    for (it = _cells_to_localdofs.begin(); it != _cells_to_localdofs.end(); ++it)
+    {
+      const Cell cell(mesh, it->first);
+      // Update UFC cell
+      cell.get_vertex_coordinates(vertex_coordinates);
+      cell.get_cell_data(ufc_cell);
 
-    p++;
+      // Tabulate coordinates of dofs on cell
+      dofmap.tabulate_coordinates(data.coordinates, vertex_coordinates,
+                                  cell);
+
+      // Restrict coefficient to cell
+      _g->restrict(data.w.data(), *_function_space->element(), cell,
+                    vertex_coordinates.data(), ufc_cell);
+
+      // Tabulate dofs on cell
+      const std::vector<dolfin::la_index>& cell_dofs
+        = dofmap.cell_dofs(cell.index());
+
+      // Loop dofs on boundary of cell
+      for (std::size_t i = 0; i < it->second.size(); ++i)
+      {
+        const std::size_t local_dof  = it->second[i];
+        const std::size_t global_dof = cell_dofs[local_dof];
+
+        // Set boundary value
+        const double value = data.w[local_dof];
+        boundary_values[global_dof] = value;
+      }
+    }
   }
 }
 //-----------------------------------------------------------------------------
