@@ -33,7 +33,7 @@
 #include <utility>
 #include <vector>
 #include <boost/multi_array.hpp>
-#include <boost/shared_ptr.hpp>
+#include <memory>
 #include <boost/unordered_map.hpp>
 #include <ufc.h>
 
@@ -63,7 +63,7 @@ namespace dolfin
     ///         The ufc::dofmap.
     ///     mesh (_Mesh_)
     ///         The mesh.
-    DofMap(boost::shared_ptr<const ufc::dofmap> ufc_dofmap,
+    DofMap(std::shared_ptr<const ufc::dofmap> ufc_dofmap,
            const Mesh& mesh);
 
     /// Create a periodic dof map on mesh (mesh is not stored)
@@ -75,9 +75,9 @@ namespace dolfin
     ///         The mesh.
     ///     conatrained_boundary (_SubDomain_)
     ///         The subdomain marking the constrained (tied) boudaries.
-    DofMap(boost::shared_ptr<const ufc::dofmap> ufc_dofmap,
+    DofMap(std::shared_ptr<const ufc::dofmap> ufc_dofmap,
            const Mesh& mesh,
-           boost::shared_ptr<const SubDomain> constrained_domain);
+           std::shared_ptr<const SubDomain> constrained_domain);
 
     /// Create restricted dof map on mesh
     ///
@@ -86,8 +86,8 @@ namespace dolfin
     ///         The ufc::dofmap.
     ///     restriction (_Restriction_)
     ///         The restriction.
-    DofMap(boost::shared_ptr<const ufc::dofmap> ufc_dofmap,
-           boost::shared_ptr<const Restriction> restriction);
+    DofMap(std::shared_ptr<const ufc::dofmap> ufc_dofmap,
+           std::shared_ptr<const Restriction> restriction);
 
   private:
 
@@ -186,9 +186,9 @@ namespace dolfin
     /// pointer is returned.
     ///
     /// *Returns*
-    ///     boost::shared_ptr<const Restriction>
+    ///     std::shared_ptr<const Restriction>
     //          The restriction.
-    boost::shared_ptr<const Restriction> restriction() const;
+    std::shared_ptr<const Restriction> restriction() const;
 
     /// Return the ownership range (dofs in this range are owned by
     /// this process)
@@ -288,42 +288,12 @@ namespace dolfin
     ///         The dof coordinates (x0, y0, x1, y1, . . .)
     std::vector<double> tabulate_all_coordinates(const Mesh& mesh) const;
 
-    /// Return a map between vertices and dofs
-    /// (dof_ind = dof_to_vertex_map[vert_ind*dofs_per_vertex + local_dof],
-    /// where local_dof = 0, ..., dofs_per_vertex)
-    /// Ghost dofs are included - then dof_ind gets negative value
-    /// or value greater than process-local number of dofs.
-    ///
-    /// *Arguments*
-    ///     mesh (_Mesh_)
-    ///         The mesh to create the map between
-    ///
-    /// *Returns*
-    ///     std::vector<dolfin::la_index>
-    ///         The dof to vertex map
-    std::vector<dolfin::la_index> dof_to_vertex_map(const Mesh& mesh) const;
-
-    /// Return a map between vertices and dofs
-    /// (vert_ind*dofs_per_vertex + local_dof = vertex_to_dof_map[dof_ind],
-    /// where local_dof = 0, ..., dofs_per_vertex)
-    /// Ghost dofs are not included. This map is
-    /// an inversion of dof_to_vertex_map.
-    ///
-    /// *Arguments*
-    ///     mesh (_Mesh_)
-    ///         The mesh to create the map between
-    ///
-    /// *Returns*
-    ///     std::vector<std::size_t>
-    ///         The vertex to dof map
-    std::vector<std::size_t> vertex_to_dof_map(const Mesh& mesh) const;
-
     /// Create a copy of the dof map
     ///
     /// *Returns*
     ///     DofMap
     ///         The Dofmap copy.
-    boost::shared_ptr<GenericDofMap> copy() const;
+    std::shared_ptr<GenericDofMap> copy() const;
 
     /// Create a copy of the dof map on a new mesh
     ///
@@ -334,7 +304,7 @@ namespace dolfin
     /// *Returns*
     ///     DofMap
     ///         The new Dofmap copy.
-    boost::shared_ptr<GenericDofMap> create(const Mesh& new_mesh) const;
+    std::shared_ptr<GenericDofMap> create(const Mesh& new_mesh) const;
 
 
     /// Extract subdofmap component
@@ -348,7 +318,7 @@ namespace dolfin
     /// *Returns*
     ///     DofMap
     ///         The subdofmap component.
-    boost::shared_ptr<GenericDofMap>
+    std::shared_ptr<GenericDofMap>
         extract_sub_dofmap(const std::vector<std::size_t>& component,
                            const Mesh& mesh) const;
 
@@ -363,7 +333,7 @@ namespace dolfin
     /// *Returns*
     ///     DofMap
     ///         The collapsed dofmap.
-    boost::shared_ptr<GenericDofMap>
+    std::shared_ptr<GenericDofMap>
           collapse(boost::unordered_map<std::size_t, std::size_t>&
                    collapsed_map, const Mesh& mesh) const;
 
@@ -437,7 +407,7 @@ namespace dolfin
     std::vector<std::vector<dolfin::la_index> > _dofmap;
 
     // UFC dof map
-    boost::shared_ptr<const ufc::dofmap> _ufc_dofmap;
+    std::shared_ptr<const ufc::dofmap> _ufc_dofmap;
 
     // Number global mesh entities. This is usually the same as what
     // is reported by the mesh, but will differ for dofmaps constrained,
@@ -448,7 +418,7 @@ namespace dolfin
     boost::unordered_map<std::size_t, std::size_t> ufc_map_to_dofmap;
 
     // Restriction, pointer zero if not restricted
-    boost::shared_ptr<const Restriction> _restriction;
+    std::shared_ptr<const Restriction> _restriction;
 
     // Flag to determine if the DofMap is a view
     bool _is_view;
@@ -476,7 +446,7 @@ namespace dolfin
     std::set<std::size_t> _neighbours;
 
     // Map from slave to master mesh entities
-    boost::shared_ptr<std::map<unsigned int, std::map<unsigned int,
+    std::shared_ptr<std::map<unsigned int, std::map<unsigned int,
       std::pair<unsigned int, unsigned int> > > > slave_master_mesh_entities;
   };
 }
