@@ -48,10 +48,12 @@
 using namespace dolfin;
 
 //-----------------------------------------------------------------------------
-void DofMapBuilder::build(DofMap& dofmap, const Mesh& mesh,
-  boost::shared_ptr<const std::map<unsigned int, std::map<unsigned int,
-    std::pair<unsigned int, unsigned int> > > > slave_master_entities,
-  boost::shared_ptr<const Restriction> restriction)
+void DofMapBuilder::build(
+  DofMap& dofmap,
+  const Mesh& mesh,
+  std::shared_ptr<const std::map<unsigned int, std::map<unsigned int,
+  std::pair<unsigned int, unsigned int> > > > slave_master_entities,
+  std::shared_ptr<const Restriction> restriction)
 {
   // Start timer for dofmap initialization
   Timer t0("Init dofmap");
@@ -72,7 +74,11 @@ void DofMapBuilder::build(DofMap& dofmap, const Mesh& mesh,
                    slave_master_entities, restriction);
 
   // Check if dofmap is distributed
-  const bool distributed = MPI::num_processes() > 1;
+  const bool distributed = MPI::size(mesh.mpi_comm()) > 1;
+
+  // Build set of dofs that are not associated with a mesh entity
+  // (global dofs)
+  const std::set<std::size_t>  global_dofs = compute_global_dofs(dofmap);
 
   // Determine and set dof block size
   dolfin_assert(dofmap._ufc_dofmap);
@@ -84,14 +90,14 @@ void DofMapBuilder::build(DofMap& dofmap, const Mesh& mesh,
   if (distributed)
   {
     reorder_distributed(dofmap, mesh, restriction, restricted_dofs_inverse,
-                        block_size);
+                        block_size, global_dofs);
   }
   else
   {
     // Optionally re-order local dofmap for spatial locality
     const bool reorder = dolfin::parameters["reorder_dofs_serial"];
     if (reorder)
-      reorder_local(dofmap, mesh, block_size);
+      reorder_local(dofmap, mesh, block_size, global_dofs);
 
     // Set local dof ownbership range
     dofmap._ownership_range = std::make_pair(0, dofmap.global_dimension());
@@ -128,7 +134,7 @@ void DofMapBuilder::build_sub_map(DofMap& sub_dofmap,
 
   // Build UFC-based dof map for sub-dofmap
   map restricted_dofs_inverse;
-  boost::shared_ptr<const Restriction> restriction;
+  std::shared_ptr<const Restriction> restriction;
   build_ufc_dofmap(sub_dofmap, restricted_dofs_inverse, mesh,
                    parent_dofmap.slave_master_mesh_entities, restriction);
 
@@ -181,17 +187,21 @@ void DofMapBuilder::build_sub_map(DofMap& sub_dofmap,
       }
     }
   }
-
-  //sub_dofmap._ownership_range = std::make_pair(0, 0);
 }
 //-----------------------------------------------------------------------------
-std::size_t DofMapBuilder::build_constrained_vertex_indices(const Mesh& mesh,
- const std::map<unsigned int, std::pair<unsigned int, unsigned int> >& slave_to_master_vertices,
- std::vector<std::size_t>& modified_global_indices)
+std::size_t DofMapBuilder::build_constrained_vertex_indices(
+  const Mesh& mesh,
+  const std::map<unsigned int,
+  std::pair<unsigned int, unsigned int> >& slave_to_master_vertices,
+  std::vector<std::size_t>& modified_global_indices)
 {
+  // MPI communicator
+  const MPI_Comm mpi_comm = mesh.mpi_comm();
+
   // Get vertex sharing information (local index, [(sharing process p,
   // local index on p)])
-  const boost::unordered_map<unsigned int, std::vector<std::pair<unsigned int, unsigned int> > >
+  const boost::unordered_map<unsigned int, std::
+                             vector<std::pair<unsigned int, unsigned int> > >
     shared_vertices = DistributedMeshTools::compute_shared_entities(mesh, 0);
 
    // Mark shared vertices
@@ -217,11 +227,11 @@ std::size_t DofMapBuilder::build_constrained_vertex_indices(const Mesh& mesh,
   }
 
   // MPI process number
-  const std::size_t proc_num = MPI::process_number();
+  const std::size_t proc_num = MPI::rank(mesh.mpi_comm());
 
   // Communication data structures
   std::vector<std::vector<std::size_t> >
-    new_shared_vertex_indices(MPI::num_processes());
+    new_shared_vertex_indices(MPI::size(mesh.mpi_comm()));
 
   // Compute modified global vertex indices
   std::size_t new_index = 0;
@@ -278,7 +288,7 @@ std::size_t DofMapBuilder::build_constrained_vertex_indices(const Mesh& mesh,
   }
 
   // Send number of owned entities to compute offeset
-  std::size_t offset = MPI::global_offset(new_index, true);
+  std::size_t offset = MPI::global_offset(mpi_comm, new_index, true);
 
   // Add process offset to modified indices
   for (std::size_t i = 0; i < modified_global_indices.size(); ++i)
@@ -291,7 +301,8 @@ std::size_t DofMapBuilder::build_constrained_vertex_indices(const Mesh& mesh,
 
   // Send/receive new indices for shared vertices
   std::vector<std::vector<std::size_t> > received_vertex_data;
-  MPI::all_to_all(new_shared_vertex_indices, received_vertex_data);
+  MPI::all_to_all(mesh.mpi_comm(), new_shared_vertex_indices,
+                  received_vertex_data);
 
   // Set index for shared vertices that have been numbered by another
   // process
@@ -311,9 +322,9 @@ std::size_t DofMapBuilder::build_constrained_vertex_indices(const Mesh& mesh,
 
   // Request master vertex index from master owner
   std::vector<std::vector<std::size_t> >
-    master_send_buffer(MPI::num_processes());
+    master_send_buffer(MPI::size(mpi_comm));
   std::vector<std::vector<std::size_t> >
-    local_slave_index(MPI::num_processes());
+    local_slave_index(MPI::size(mpi_comm));
   std::map<unsigned int,
            std::pair<unsigned int, unsigned int> >::const_iterator master;
   for (master = slave_to_master_vertices.begin();
@@ -330,11 +341,12 @@ std::size_t DofMapBuilder::build_constrained_vertex_indices(const Mesh& mesh,
 
   // Send/receive master local indices for slave vertices
   std::vector<std::vector<std::size_t> > received_slave_vertex_indices;
-  MPI::all_to_all(master_send_buffer, received_slave_vertex_indices);
+  MPI::all_to_all(mpi_comm, master_send_buffer,
+                  received_slave_vertex_indices);
 
   // Send back new master vertex index
   std::vector<std::vector<std::size_t> >
-    master_vertex_indices(MPI::num_processes());
+    master_vertex_indices(MPI::size(mpi_comm));
   for (std::size_t p = 0; p < received_slave_vertex_indices.size(); ++p)
   {
     const std::vector<std::size_t>& local_master_indices
@@ -349,7 +361,8 @@ std::size_t DofMapBuilder::build_constrained_vertex_indices(const Mesh& mesh,
 
   // Send/receive new global master indices for slave vertices
   std::vector<std::vector<std::size_t> > received_new_slave_vertex_indices;
-  MPI::all_to_all(master_vertex_indices, received_new_slave_vertex_indices);
+  MPI::all_to_all(mpi_comm, master_vertex_indices,
+                  received_new_slave_vertex_indices);
 
   // Set index for slave vertices
   for (std::size_t p = 0; p < received_new_slave_vertex_indices.size(); ++p)
@@ -369,11 +382,12 @@ std::size_t DofMapBuilder::build_constrained_vertex_indices(const Mesh& mesh,
 
   // Send new indices to process that share a vertex but were not
   // responsible for re-numbering
-  return MPI::sum(new_index);
+  return MPI::sum(mpi_comm, new_index);
 }
 //-----------------------------------------------------------------------------
 void DofMapBuilder::reorder_local(DofMap& dofmap, const Mesh& mesh,
-                                  std::size_t block_size)
+                                  std::size_t block_size,
+				  const std::set<std::size_t>& global_dofs)
 {
   // Global dimension
   const std::size_t N = dofmap.global_dimension();
@@ -384,6 +398,7 @@ void DofMapBuilder::reorder_local(DofMap& dofmap, const Mesh& mesh,
   Graph graph(num_nodes);
 
   // Build local graph for blocks
+  std::vector<dolfin::la_index> dofs_tmp;
   for (CellIterator cell(mesh); !cell.end(); ++cell)
   {
     const std::vector<dolfin::la_index>& dofs0
@@ -394,11 +409,30 @@ void DofMapBuilder::reorder_local(DofMap& dofmap, const Mesh& mesh,
     dolfin_assert(dofs0.size() % block_size == 0);
     const std::size_t nodes_per_cell = dofs0.size()/block_size;
 
-    std::vector<dolfin::la_index>::const_iterator node0, node1;
+    // Build vector of graph 'edges'
+    dofs_tmp.resize(nodes_per_cell);
+    std::size_t dof_counter = 0;
     for (std::size_t i = 0; i < nodes_per_cell; ++i)
-      for (std::size_t j = 0; j < nodes_per_cell; ++j)
-        if (dofs0[i] != dofs1[j])
-          graph[dofs0[i] % num_nodes].insert(dofs1[j] % num_nodes);
+    {
+      if (global_dofs.find(dofs0[i]) == global_dofs.end())
+      {
+        if ((dofs0[i] % num_nodes) != (dofs1[i] % num_nodes))
+        {
+          dofs_tmp[i] = dofs1[i] % num_nodes;
+          ++dof_counter;
+        }
+      }
+    }
+
+    // Add graph 'edges'
+    for (std::size_t i = 0; i < nodes_per_cell; ++i)
+    {
+      if (global_dofs.find(dofs1[i]) == global_dofs.end())
+      {
+        graph[dofs0[i] % num_nodes].insert(dofs_tmp.begin(),
+                                           dofs_tmp.begin() + dof_counter);
+      }
+    }
   }
 
   // Reorder block graph
@@ -449,12 +483,13 @@ void DofMapBuilder::reorder_local(DofMap& dofmap, const Mesh& mesh,
   }
 }
 //-----------------------------------------------------------------------------
-void DofMapBuilder::build_ufc_dofmap(DofMap& dofmap,
-    DofMapBuilder::map& restricted_dofs_inverse,
-    const Mesh& mesh,
-    boost::shared_ptr<const std::map<unsigned int, std::map<unsigned int,
-      std::pair<unsigned int, unsigned int> > > > slave_master_entities,
-    boost::shared_ptr<const Restriction> restriction)
+void DofMapBuilder::build_ufc_dofmap(
+  DofMap& dofmap,
+  DofMapBuilder::map& restricted_dofs_inverse,
+  const Mesh& mesh,
+  std::shared_ptr<const std::map<unsigned int, std::map<unsigned int,
+  std::pair<unsigned int, unsigned int> > > > slave_master_entities,
+  std::shared_ptr<const Restriction> restriction)
 {
   // Start timer for dofmap initialization
   Timer t0("Init dofmap from UFC dofmap");
@@ -620,20 +655,18 @@ void DofMapBuilder::build_ufc_dofmap(DofMap& dofmap,
 //-----------------------------------------------------------------------------
 void DofMapBuilder::reorder_distributed(DofMap& dofmap,
                             const Mesh& mesh,
-                            boost::shared_ptr<const Restriction> restriction,
+                            std::shared_ptr<const Restriction> restriction,
                             const map& restricted_dofs_inverse,
-                            std::size_t block_size)
+                            std::size_t block_size,
+                            const std::set<std::size_t>& global_dofs)
 {
-  // Build set of dofs that are not associated with a mesh entity
-  // (global dofs)
-  set global_dofs = compute_global_dofs(dofmap);
 
   // Allocate data structure to hold node ownership (multiple dofs can
   // live at one node)
   boost::array<DofMapBuilder::set, 3> node_ownership;
 
-  // Allocate map data structure from a shared node to the processes that
-  // share it
+  // Allocate map data structure from a shared node to the processes
+  // that share it
   DofMapBuilder::vec_map shared_node_processes;
 
   // Computed owned and shared nodes (and owned and un-owned)
@@ -641,22 +674,25 @@ void DofMapBuilder::reorder_distributed(DofMap& dofmap,
                          global_dofs, mesh, restriction,
                          restricted_dofs_inverse, block_size);
 
-  // Renumber owned dofs and receive new numbering for unowned shared dofs
-  parallel_renumber(node_ownership, shared_node_processes, dofmap, mesh,
-                    restriction, restricted_dofs_inverse, block_size);
+  // Renumber owned dofs and receive new numbering for unowned shared
+  // dofs
+  parallel_renumber(node_ownership, shared_node_processes, dofmap, global_dofs,
+		    mesh, restriction, restricted_dofs_inverse, block_size);
 }
 //-----------------------------------------------------------------------------
 void DofMapBuilder::compute_node_ownership(boost::array<set, 3>& node_ownership,
                               vec_map& shared_node_processes,
                               DofMap& dofmap,
-                              const DofMapBuilder::set& global_dofs,
+                              const std::set<std::size_t>& global_dofs,
                               const Mesh& mesh,
-                              boost::shared_ptr<const Restriction> restriction,
+                              std::shared_ptr<const Restriction> restriction,
                               const map& restricted_nodes_inverse,
                               std::size_t block_size)
 {
   log(TRACE, "Determining dof ownership for parallel dof map");
 
+  const MPI_Comm mpi_comm = mesh.mpi_comm();
+
   const std::size_t N = dofmap.global_dimension();
   dolfin_assert(N % block_size == 0);
   const std::size_t num_nodes = N/block_size;
@@ -685,19 +721,22 @@ void DofMapBuilder::compute_node_ownership(boost::array<set, 3>& node_ownership,
   BoundaryMesh boundary(mesh, "local");
 
   // Create a random number generator for ownership 'voting'
-  boost::mt19937 engine(MPI::process_number());
+  boost::mt19937 engine(MPI::rank(mpi_comm));
   boost::uniform_int<> distribution(0, 100000000);
   boost::variate_generator<boost::mt19937&, boost::uniform_int<> >
     rng(engine, distribution);
 
-  // Build set of dofs on process boundary (first assuming that all are
-  // owned by this process)
+  // Build set of dofs on process boundary (first assuming that all
+  // are owned by this process)
   const MeshFunction<std::size_t>& cell_map
     = boundary.entity_map(boundary.topology().dim());
   if (!cell_map.empty())
   {
     for (CellIterator _f(boundary); !_f.end(); ++_f)
     {
+      // Create a vote per facet
+      const std::size_t facet_vote = rng();
+
       // Get boundary facet
       Facet f(mesh, cell_map[*_f]);
 
@@ -722,7 +761,8 @@ void DofMapBuilder::compute_node_ownership(boost::array<set, 3>& node_ownership,
         // Get facet node
         size_t facet_node = cell_dofs[facet_dofs[i]] % num_nodes;
 
-        // Map back to original (and common) numbering for restricted space
+        // Map back to original (and common) numbering for restricted
+        // space
         if (restriction)
         {
           const map_iterator it = restricted_nodes_inverse.find(facet_node);
@@ -734,7 +774,7 @@ void DofMapBuilder::compute_node_ownership(boost::array<set, 3>& node_ownership,
         if (shared_owned_nodes.find(facet_node) == shared_owned_nodes.end())
         {
           shared_owned_nodes.insert(facet_node);
-          node_vote[facet_node] = rng();
+          node_vote[facet_node] = facet_vote;
 
           send_buffer.push_back(facet_node);
           send_buffer.push_back(node_vote[facet_node]);
@@ -749,16 +789,15 @@ void DofMapBuilder::compute_node_ownership(boost::array<set, 3>& node_ownership,
   //        will avoid interleaving communication and computation.
 
   // Decide ownership of shared nodes
-  const std::size_t num_prococesses = MPI::num_processes();
-  const std::size_t process_number = MPI::process_number();
+  const std::size_t num_prococesses = MPI::size(mpi_comm);
+  const std::size_t process_number = MPI::rank(mpi_comm);
   std::vector<std::size_t> recv_buffer;
   for (std::size_t k = 1; k < num_prococesses; ++k)
   {
     const std::size_t src
       = (process_number - k + num_prococesses) % num_prococesses;
     const std::size_t dest = (process_number + k) % num_prococesses;
-    MPI::send_recv(send_buffer, dest, recv_buffer, src);
-
+    MPI::send_recv(mpi_comm, send_buffer, dest, recv_buffer, src);
     for (std::size_t i = 0; i < recv_buffer.size(); i += 2)
     {
       const std::size_t received_node = recv_buffer[i];
@@ -793,12 +832,12 @@ void DofMapBuilder::compute_node_ownership(boost::array<set, 3>& node_ownership,
     }
   }
 
-  // Add/remove global dofs to/from relevant sets (process 0 owns
+  // Add/remove global dofs to/from relevant sets (last process owns
   // global dofs)
-  if (process_number == 0)
+  if (process_number == num_prococesses - 1)
   {
     shared_owned_nodes.insert(global_dofs.begin(), global_dofs.end());
-    for (set::const_iterator dof = global_dofs.begin();
+    for (std::set<std::size_t>::const_iterator dof = global_dofs.begin();
          dof != global_dofs.end(); ++dof)
     {
       set::const_iterator _dof = shared_unowned_nodes.find(*dof);
@@ -809,7 +848,7 @@ void DofMapBuilder::compute_node_ownership(boost::array<set, 3>& node_ownership,
   else
   {
     shared_unowned_nodes.insert(global_dofs.begin(), global_dofs.end());
-    for (set::const_iterator dof = global_dofs.begin();
+    for (std::set<std::size_t>::const_iterator dof = global_dofs.begin();
          dof != global_dofs.end(); ++dof)
     {
       set::const_iterator _dof = shared_owned_nodes.find(*dof);
@@ -828,7 +867,8 @@ void DofMapBuilder::compute_node_ownership(boost::array<set, 3>& node_ownership,
       // Get cell node
       size_t cell_node = cell_dofs[i] % num_nodes;
 
-      // Map back to original (and common) numbering for restricted space
+      // Map back to original (and common) numbering for restricted
+      // space
       if (restriction)
       {
         const map_iterator it = restricted_nodes_inverse.find(cell_node);
@@ -848,29 +888,35 @@ void DofMapBuilder::compute_node_ownership(boost::array<set, 3>& node_ownership,
     // Global dimension for restricted space needs to be computed here
     // since it is not know by the UFC dof map.
     const std::size_t _owned_dim = owned_nodes.size();
-    const std::size_t _global_dimension = block_size*MPI::sum(_owned_dim);
+    const std::size_t _global_dimension = block_size*MPI::sum(mpi_comm,
+                                                              _owned_dim);
     dofmap._global_dimension = _global_dimension;
   }
   else
   {
     const std::size_t _owned_dim = owned_nodes.size();
-    dolfin_assert(block_size*MPI::sum(_owned_dim) == dofmap.global_dimension());
+    dolfin_assert(block_size*MPI::sum(mpi_comm, _owned_dim)
+                  == dofmap.global_dimension());
   }
 
   log(TRACE, "Finished determining dof ownership for parallel dof map");
 }
 //-----------------------------------------------------------------------------
-void
-DofMapBuilder::parallel_renumber(const boost::array<set, 3>& node_ownership,
-                                 const vec_map& shared_node_processes,
-                                 DofMap& dofmap,
-                                 const Mesh& mesh,
-                                 boost::shared_ptr<const Restriction> restriction,
-                                 const map& restricted_nodes_inverse,
-                                 std::size_t block_size)
+void DofMapBuilder::parallel_renumber(
+  const boost::array<set, 3>& node_ownership,
+  const vec_map& shared_node_processes,
+  DofMap& dofmap,
+  const std::set<std::size_t>& global_dofs,
+  const Mesh& mesh,
+  std::shared_ptr<const Restriction> restriction,
+  const map& restricted_nodes_inverse,
+  std::size_t block_size)
 {
   log(TRACE, "Renumber dofs for parallel dof map");
 
+  // MPI communicator
+  const MPI_Comm mpi_comm = mesh.mpi_comm();
+
   const std::size_t N = dofmap.global_dimension();
   dolfin_assert(N % block_size == 0);
   const std::size_t num_nodes = N/block_size;
@@ -894,8 +940,8 @@ DofMapBuilder::parallel_renumber(const boost::array<set, 3>& node_ownership,
   dolfin_assert(old_dofmap.size() == mesh.num_cells());
 
   // Compute offset for owned and non-shared nodes
-  const std::size_t process_offset = MPI::global_offset(owned_nodes.size(),
-                                                        true);
+  const std::size_t process_offset
+    = MPI::global_offset(mpi_comm, owned_nodes.size(), true);
 
   // Clear some data
   dofmap._off_process_owner.clear();
@@ -929,6 +975,9 @@ DofMapBuilder::parallel_renumber(const boost::array<set, 3>& node_ownership,
       std::vector<dolfin::la_index>::const_iterator dof0, dof1;
       for (std::size_t i = 0; i < nodes_per_cell; ++i)
       {
+	if (global_dofs.find(dofs0[i]) != global_dofs.end())
+	  continue;
+
         const std::size_t n0_old = dofs0[i] % num_nodes;
 
         // Get new node index from contiguous map
@@ -940,6 +989,9 @@ DofMapBuilder::parallel_renumber(const boost::array<set, 3>& node_ownership,
           dolfin_assert(n0_local < graph.size());
           for (std::size_t j = 0; j < nodes_per_cell; ++j)
           {
+	    if (global_dofs.find(dofs0[j]) != global_dofs.end())
+	      continue;
+
             const std::size_t n1_old = dofs1[j] % num_nodes;
             boost::unordered_map<std::size_t, std::size_t>::const_iterator
               n1 = my_old_to_new_node_index.find(n1_old);
@@ -963,7 +1015,7 @@ DofMapBuilder::parallel_renumber(const boost::array<set, 3>& node_ownership,
   if (ordering_library == "Boost")
     node_remap = BoostGraphOrdering::compute_cuthill_mckee(graph, true);
   else if (ordering_library == "SCOTCH")
-   node_remap = SCOTCH::compute_gps(graph);
+    node_remap = SCOTCH::compute_gps(graph);
   else if (ordering_library == "random")
   {
     // NOTE: Randomised dof ordering should only be used for
@@ -977,7 +1029,8 @@ DofMapBuilder::parallel_renumber(const boost::array<set, 3>& node_ownership,
   {
     dolfin_error("DofMapBuilder.cpp",
                  "reorder degrees of freedom",
-                 "The requested ordering library '%s' is unknown", ordering_library.c_str());
+                 "The requested ordering library '%s' is unknown",
+                 ordering_library.c_str());
   }
 
   // Map from old to new index for dofs
@@ -988,7 +1041,7 @@ DofMapBuilder::parallel_renumber(const boost::array<set, 3>& node_ownership,
   std::size_t counter = 0;
   std::vector<std::size_t> send_buffer;
   for (set_iterator owned_node = owned_nodes.begin();
-          owned_node != owned_nodes.end(); ++owned_node, counter++)
+       owned_node != owned_nodes.end(); ++owned_node, counter++)
   {
     // Set new node number
     old_to_new_node_index[*owned_node] = process_offset + node_remap[counter];
@@ -1003,7 +1056,8 @@ DofMapBuilder::parallel_renumber(const boost::array<set, 3>& node_ownership,
       dofmap.ufc_map_to_dofmap[ufc_dof_index] = new_dof_index;
     }
 
-    // If this node is shared and owned, buffer old and new index for sending
+    // If this node is shared and owned, buffer old and new index for
+    // sending
     if (shared_owned_nodes.find(*owned_node) != shared_owned_nodes.end())
     {
       send_buffer.push_back(*owned_node);
@@ -1017,15 +1071,15 @@ DofMapBuilder::parallel_renumber(const boost::array<set, 3>& node_ownership,
   //        will avoid interleaving communication and computation.
 
   // Exchange new node numbers for nodes that are shared
-  const std::size_t num_processes = MPI::num_processes();
-  const std::size_t process_number = MPI::process_number();
+  const std::size_t num_processes = MPI::size(mpi_comm);
+  const std::size_t process_number = MPI::rank(mpi_comm);
   std::vector<std::size_t> recv_buffer;
   for (std::size_t k = 1; k < num_processes; ++k)
   {
     const std::size_t src
       = (process_number - k + num_processes) % num_processes;
     const std::size_t dest = (process_number + k) % num_processes;
-    MPI::send_recv(send_buffer, dest, recv_buffer, src);
+    MPI::send_recv(mpi_comm, send_buffer, dest, recv_buffer, src);
 
     // Add dofs renumbered by another process to the old-to-new map
     for (std::size_t i = 0; i < recv_buffer.size(); i += 2)
@@ -1058,10 +1112,10 @@ DofMapBuilder::parallel_renumber(const boost::array<set, 3>& node_ownership,
 
   // FIXME: Should dofmap._shared_dofs be cleared?
 
-  // Insert the shared-dof-to-process mapping into the dofmap, renumbering
-  // as necessary
+  // Insert the shared-dof-to-process mapping into the dofmap,
+  // renumbering as necessary
   for (vec_map::const_iterator it = shared_node_processes.begin();
-            it != shared_node_processes.end(); ++it)
+       it != shared_node_processes.end(); ++it)
   {
     // Check for shared node in old_to_new_node_index map
     boost::unordered_map<std::size_t, std::size_t>::const_iterator
@@ -1119,19 +1173,19 @@ DofMapBuilder::parallel_renumber(const boost::array<set, 3>& node_ownership,
   log(TRACE, "Finished renumbering dofs for parallel dof map");
 }
 //-----------------------------------------------------------------------------
-DofMapBuilder::set DofMapBuilder::compute_global_dofs(const DofMap& dofmap)
+std::set<std::size_t> DofMapBuilder::compute_global_dofs(const DofMap& dofmap)
 {
   // Compute global dof indices
   std::size_t offset = 0;
-  set global_dof_indices;
+  std::set<std::size_t> global_dof_indices;
   compute_global_dofs(global_dof_indices, offset, dofmap._ufc_dofmap, dofmap);
 
   return global_dof_indices;
 }
 //-----------------------------------------------------------------------------
-void DofMapBuilder::compute_global_dofs(DofMapBuilder::set& global_dofs,
+void DofMapBuilder::compute_global_dofs(std::set<std::size_t>& global_dofs,
                           std::size_t& offset,
-                          boost::shared_ptr<const ufc::dofmap> ufc_dofmap,
+                          std::shared_ptr<const ufc::dofmap> ufc_dofmap,
                           const DofMap& dofmap)
 {
   dolfin_assert(ufc_dofmap);
@@ -1160,13 +1214,13 @@ void DofMapBuilder::compute_global_dofs(DofMapBuilder::set& global_dofs,
       }
 
       // Create dummy cell argument to tabulate single global dof
-      boost::scoped_ptr<ufc::cell> ufc_cell(new ufc::cell);
+      std::unique_ptr<ufc::cell> ufc_cell(new ufc::cell);
       std::size_t dof = 0;
       ufc_dofmap->tabulate_dofs(&dof, dofmap.num_global_mesh_entities,
                                 *ufc_cell);
 
       // Insert global dof index
-      std::pair<DofMapBuilder::set::iterator, bool> ret
+      std::pair<std::set<std::size_t>::iterator, bool> ret
         = global_dofs.insert(dof + offset);
       if (!ret.second)
       {
@@ -1182,7 +1236,7 @@ void DofMapBuilder::compute_global_dofs(DofMapBuilder::set& global_dofs,
     for (std::size_t i = 0; i < ufc_dofmap->num_sub_dofmaps(); ++i)
     {
       // Extract sub-dofmap and intialise
-      boost::shared_ptr<ufc::dofmap>
+      std::shared_ptr<ufc::dofmap>
         sub_dofmap(ufc_dofmap->create_sub_dofmap(i));
 
       compute_global_dofs(global_dofs, offset, sub_dofmap, dofmap);
@@ -1194,11 +1248,11 @@ void DofMapBuilder::compute_global_dofs(DofMapBuilder::set& global_dofs,
   }
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<ufc::dofmap>
-    DofMapBuilder::extract_ufc_sub_dofmap(const ufc::dofmap& ufc_dofmap,
-                      std::size_t& offset,
-                      const std::vector<std::size_t>& component,
-                      const std::vector<std::size_t>& num_global_mesh_entities)
+std::shared_ptr<ufc::dofmap> DofMapBuilder::extract_ufc_sub_dofmap(
+  const ufc::dofmap& ufc_dofmap,
+  std::size_t& offset,
+  const std::vector<std::size_t>& component,
+  const std::vector<std::size_t>& num_global_mesh_entities)
 {
   // Check if there are any sub systems
   if (ufc_dofmap.num_sub_dofmaps() == 0)
@@ -1229,7 +1283,7 @@ boost::shared_ptr<ufc::dofmap>
   for (std::size_t i = 0; i < component[0]; i++)
   {
     // Extract sub dofmap
-    boost::scoped_ptr<ufc::dofmap>
+    std::unique_ptr<ufc::dofmap>
       ufc_tmp_dofmap(ufc_dofmap.create_sub_dofmap(i));
     dolfin_assert(ufc_tmp_dofmap);
 
@@ -1241,7 +1295,7 @@ boost::shared_ptr<ufc::dofmap>
   }
 
   // Create UFC sub-system
-  boost::shared_ptr<ufc::dofmap>
+  std::shared_ptr<ufc::dofmap>
     sub_dofmap(ufc_dofmap.create_sub_dofmap(component[0]));
   dolfin_assert(sub_dofmap);
 
@@ -1255,7 +1309,7 @@ boost::shared_ptr<ufc::dofmap>
     for (std::size_t i = 1; i < component.size(); ++i)
       sub_component.push_back(component[i]);
 
-    boost::shared_ptr<ufc::dofmap> sub_sub_dofmap
+    std::shared_ptr<ufc::dofmap> sub_sub_dofmap
         = extract_ufc_sub_dofmap(*sub_dofmap, offset, sub_component,
                                  num_global_mesh_entities);
 
@@ -1269,7 +1323,7 @@ std::size_t DofMapBuilder::compute_blocksize(const ufc::dofmap& ufc_dofmap)
   if (ufc_dofmap.num_sub_dofmaps() > 1)
   {
     // Create UFC first sub-dofmap
-    boost::scoped_ptr<ufc::dofmap>
+    std::unique_ptr<ufc::dofmap>
       ufc_sub_dofmap0(ufc_dofmap.create_sub_dofmap(0));
     dolfin_assert(ufc_sub_dofmap0);
 
@@ -1286,7 +1340,7 @@ std::size_t DofMapBuilder::compute_blocksize(const ufc::dofmap& ufc_dofmap)
       // same number of dofs per entity
       for (std::size_t i = 1; i < ufc_dofmap.num_sub_dofmaps(); ++i)
       {
-        boost::scoped_ptr<ufc::dofmap>
+        std::unique_ptr<ufc::dofmap>
           ufc_sub_dofmap(ufc_dofmap.create_sub_dofmap(i));
         dolfin_assert(ufc_sub_dofmap);
         for (std::size_t d = 0; d <= ufc_dofmap.topological_dimension(); ++d)
