@@ -55,7 +55,10 @@ using namespace dolfin;
 //----------------------------------------------------------------------------
 void OpenMpAssembler::assemble(GenericTensor& A, const Form& a)
 {
-  if (MPI::num_processes() > 1)
+  // Get mesh
+  const Mesh& mesh = a.mesh();
+
+  if (MPI::size(mesh.mpi_comm()) > 1)
   {
     dolfin_error("OpenMPAssembler.cpp",
                  "perform multithreaded assembly using OpenMP assembler",
@@ -87,12 +90,6 @@ void OpenMpAssembler::assemble(GenericTensor& A, const Form& a)
   // Create data structure for local assembly data
   UFC ufc(a);
 
-  // Update off-process coefficients
-  const std::vector<boost::shared_ptr<const GenericFunction> >
-    coefficients = a.coefficients();
-  for (std::size_t i = 0; i < coefficients.size(); ++i)
-    coefficients[i]->update();
-
   // Initialize global tensor
   init_global_tensor(A, a);
 
@@ -212,7 +209,8 @@ void OpenMpAssembler::assemble_cells(GenericTensor& A, const Form& a,
       // Update to current cell
       cell.get_cell_data(ufc_cell);
       cell.get_vertex_coordinates(vertex_coordinates);
-      ufc.update(cell, vertex_coordinates, ufc_cell);
+      ufc.update(cell, vertex_coordinates, ufc_cell,
+                 integral->enabled_coefficients());
 
       // Get local-to-global dof maps for cell
       for (std::size_t i = 0; i < form_rank; ++i)
@@ -351,7 +349,8 @@ void OpenMpAssembler::assemble_cells_and_exterior_facets(GenericTensor& A,
       // Update to current cell
       cell.get_cell_data(ufc_cell);
       cell.get_vertex_coordinates(vertex_coordinates);
-      ufc.update(cell, vertex_coordinates, ufc_cell);
+      ufc.update(cell, vertex_coordinates, ufc_cell,
+                 cell_integral->enabled_coefficients());
 
       // Get local-to-global dof maps for cell
       for (std::size_t i = 0; i < form_rank; ++i)
@@ -402,13 +401,15 @@ void OpenMpAssembler::assemble_cells_and_exterior_facets(GenericTensor& A,
         //        facet index?
         // Update UFC object
         ufc_cell.local_facet = local_facet;
-        ufc.update(cell, vertex_coordinates, ufc_cell);
+        ufc.update(cell, vertex_coordinates, ufc_cell,
+                  facet_integral->enabled_coefficients());
 
         // Tabulate tensor
         facet_integral->tabulate_tensor(ufc.A_facet.data(),
                                         ufc.w(),
                                         vertex_coordinates.data(),
-                                        local_facet);
+                                        local_facet,
+                                        ufc_cell.orientation);
 
         // Add facet contribution
         for (std::size_t i = 0; i < dim; ++i)
@@ -588,7 +589,8 @@ void OpenMpAssembler::assemble_interior_facets(GenericTensor& A, const Form& a,
 
       // Update to current pair of cells
       ufc.update(cell0, vertex_coordinates0, ufc_cell0,
-                 cell1, vertex_coordinates1, ufc_cell1);
+                 cell1, vertex_coordinates1, ufc_cell1,
+                 integral->enabled_coefficients());
 
       // Tabulate dofs for each dimension on macro element
       for (std::size_t i = 0; i < form_rank; i++)
@@ -614,7 +616,9 @@ void OpenMpAssembler::assemble_interior_facets(GenericTensor& A, const Form& a,
                                 vertex_coordinates0.data(),
                                 vertex_coordinates1.data(),
                                 local_facet0,
-                                local_facet1);
+                                local_facet1,
+                                ufc_cell0.orientation,
+                                ufc_cell1.orientation);
 
       // Add entries to global tensor
       A.add(ufc.macro_A.data(), macro_dofs);
