@@ -63,13 +63,19 @@ void SparsityPatternBuilder::build(GenericSparsityPattern& sparsity_pattern,
 
   // Initialise sparsity pattern
   if (init)
-    sparsity_pattern.init(global_dimensions, local_range, off_process_owner);
+  {
+    sparsity_pattern.init(mesh.mpi_comm(), global_dimensions, local_range,
+                          off_process_owner);
+  }
 
   // Only build for rank >= 2 (matrices and higher order tensors) that
   // require sparsity details
   if (rank < 2)
     return;
 
+  // Vector to store macro-dofs, if required (for interior facets)
+  std::vector<std::vector<dolfin::la_index> > macro_dofs(rank);
+
   // Create vector to point to dofs
   std::vector<const std::vector<dolfin::la_index>* > dofs(rank);
 
@@ -112,9 +118,6 @@ void SparsityPatternBuilder::build(GenericSparsityPattern& sparsity_pattern,
                    "Consider calling mesh.order()");
     }
 
-    // Vector to store macro-dofs (for interior facets)
-    std::vector<std::vector<dolfin::la_index> > macro_dofs(rank);
-
     Progress p("Building sparsity pattern over interior facets", mesh.num_facets());
     for (FacetIterator facet(mesh); !facet.end(); ++facet)
     {
