@@ -70,7 +70,7 @@ Function::Function(const FunctionSpace& V)
   init_vector();
 }
 //-----------------------------------------------------------------------------
-Function::Function(boost::shared_ptr<const FunctionSpace> V)
+Function::Function(std::shared_ptr<const FunctionSpace> V)
   : Hierarchical<Function>(*this), _function_space(V),
     allow_extrapolation(dolfin::parameters["allow_extrapolation"])
 {
@@ -86,8 +86,8 @@ Function::Function(boost::shared_ptr<const FunctionSpace> V)
   init_vector();
 }
 //-----------------------------------------------------------------------------
-Function::Function(boost::shared_ptr<const FunctionSpace> V,
-                   boost::shared_ptr<GenericVector> x)
+Function::Function(std::shared_ptr<const FunctionSpace> V,
+                   std::shared_ptr<GenericVector> x)
   : Hierarchical<Function>(*this), _function_space(V), _vector(x),
     allow_extrapolation(dolfin::parameters["allow_extrapolation"])
 {
@@ -129,7 +129,7 @@ Function::Function(const FunctionSpace& V, std::string filename)
   file >> *this;
 }
 //-----------------------------------------------------------------------------
-Function::Function(boost::shared_ptr<const FunctionSpace> V,
+Function::Function(std::shared_ptr<const FunctionSpace> V,
                    std::string filename)
   : Hierarchical<Function>(*this), _function_space(V),
     allow_extrapolation(dolfin::parameters["allow_extrapolation"])
@@ -259,7 +259,7 @@ Function& Function::operator[] (std::size_t i) const
   {
     // Extract function subspace
     std::vector<std::size_t> component = boost::assign::list_of(i);
-    boost::shared_ptr<const FunctionSpace>
+    std::shared_ptr<const FunctionSpace>
       sub_space(_function_space->extract_sub_space(component));
 
     // Insert sub-Function into map and return reference
@@ -318,13 +318,13 @@ void Function::operator=(const FunctionAXPY& axpy)
     _vector->axpy(it->first, *(it->second->vector()));
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<const FunctionSpace> Function::function_space() const
+std::shared_ptr<const FunctionSpace> Function::function_space() const
 {
   dolfin_assert(_function_space);
   return _function_space;
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<GenericVector> Function::vector()
+std::shared_ptr<GenericVector> Function::vector()
 {
   dolfin_assert(_vector);
   dolfin_assert(_function_space->dofmap());
@@ -340,7 +340,7 @@ boost::shared_ptr<GenericVector> Function::vector()
   return _vector;
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<const GenericVector> Function::vector() const
+std::shared_ptr<const GenericVector> Function::vector() const
 {
   dolfin_assert(_vector);
   return _vector;
@@ -387,7 +387,6 @@ void Function::eval(Array<double>& values, const Array<double>& x) const
     }
     else
     {
-      cout << point << endl;
       dolfin_error("Function.cpp",
                    "evaluate function at point",
                    "The point is not inside the domain. Consider setting \"allow_extrapolation\" to allow extrapolation");
@@ -438,12 +437,11 @@ void Function::eval(Array<double>& values, const Array<double>& x,
     values[j] = 0.0;
 
   // Compute linear combination
-  const int cell_orientation = 0;
   for (std::size_t i = 0; i < element.space_dimension(); ++i)
   {
     element.evaluate_basis(i, basis.data(), x.data(),
                            vertex_coordinates.data(),
-                           cell_orientation);
+                           ufc_cell.orientation);
     for (std::size_t j = 0; j < value_size_loc; ++j)
       values[j] += coefficients[i]*basis[j];
   }
@@ -451,14 +449,10 @@ void Function::eval(Array<double>& values, const Array<double>& x,
 //-----------------------------------------------------------------------------
 void Function::interpolate(const GenericFunction& v)
 {
-  // Gather off-process dofs
-  v.update();
-
-  // Initialise vector
-  init_vector();
+  dolfin_assert(_vector);
+  dolfin_assert(_function_space);
 
   // Interpolate
-  dolfin_assert(_function_space);
   _function_space->interpolate(*_vector, v);
 }
 //-----------------------------------------------------------------------------
@@ -569,8 +563,17 @@ void Function::restrict(double* w, const FiniteElement& element,
     const std::vector<dolfin::la_index>& dofs
       = dofmap.cell_dofs(dolfin_cell.index());
 
-    // Pick values from vector(s)
-    _vector->get_local(w, dofs.size(), dofs.data());
+    if (dofs.size() > 0)
+    {
+      // Note: We should have dofmap.max_cell_dimension() == dofs.size() here.
+      // Pick values from vector(s)
+      _vector->get_local(w, dofs.size(), dofs.data());
+    }
+    else
+    {
+      // Set dofs to zero (zero extension of function space on a Restriction)
+      memset(w, 0, sizeof(*w)*dofmap.max_cell_dimension());
+    }
   }
   else
   {
@@ -596,15 +599,12 @@ void Function::compute_vertex_values(std::vector<double>& vertex_values,
                  "Non-matching mesh");
   }
 
-  // Update ghosts dofs
-  update();
-
   // Get finite element
   dolfin_assert(_function_space->element());
   const FiniteElement& element = *_function_space->element();
 
   // Get restriction if any
-  boost::shared_ptr<const Restriction> restriction
+  std::shared_ptr<const Restriction> restriction
     = _function_space->dofmap()->restriction();
 
   // Local data for interpolation on each cell
@@ -642,11 +642,10 @@ void Function::compute_vertex_values(std::vector<double>& vertex_values,
              ufc_cell);
 
     // Interpolate values at the vertices
-    const int cell_orientation = 0;
     element.interpolate_vertex_values(cell_vertex_values.data(),
                                       coefficients.data(),
                                       vertex_coordinates.data(),
-                                      cell_orientation,
+                                      ufc_cell.orientation,
                                       ufc_cell);
 
     // Copy values to array of vertex values
@@ -671,7 +670,8 @@ void Function::compute_vertex_values(std::vector<double>& vertex_values)
 //-----------------------------------------------------------------------------
 void Function::update() const
 {
-  _vector->update_ghost_values();
+  deprecation("Function::update()", "1.4", "1.5",
+              "Calling Function::update to update ghost values is no longer required.");
 }
 //-----------------------------------------------------------------------------
 void Function::init_vector()
@@ -709,7 +709,16 @@ void Function::init_vector()
   dolfin_assert(_vector);
 
   // Initialize vector of dofs
-  _vector->resize(range, ghost_indices);
+  dolfin_assert(_function_space->mesh());
+  if (_vector->empty())
+    _vector->init(_function_space->mesh()->mpi_comm(), range, ghost_indices);
+  else
+  {
+    dolfin_error("Function.cpp",
+                 "initialize vector of degrees of freedom for function",
+                 "Cannot re-initialize a non-empty vector. Consider creating a new function");
+
+  }
   _vector->zero();
 }
 //-----------------------------------------------------------------------------
