@@ -34,8 +34,8 @@ SubSpace::SubSpace(const FunctionSpace& V, std::size_t component)
   c.push_back(component);
 
   // Extract subspace and assign
-  boost::shared_ptr<FunctionSpace> _V(V.extract_sub_space(c));
-  *static_cast<FunctionSpace*>(this) = *_V;
+  std::shared_ptr<FunctionSpace> _function_space(V.extract_sub_space(c));
+  *static_cast<FunctionSpace*>(this) = *_function_space;
 }
 //-----------------------------------------------------------------------------
 SubSpace::SubSpace(const FunctionSpace& V, std::size_t component, std::size_t sub_component)
@@ -47,15 +47,15 @@ SubSpace::SubSpace(const FunctionSpace& V, std::size_t component, std::size_t su
   c.push_back(sub_component);
 
   // Extract subspace and assign
-  boost::shared_ptr<FunctionSpace> _V(V.extract_sub_space(c));
-  *static_cast<FunctionSpace*>(this) = *_V;
+  std::shared_ptr<FunctionSpace> _function_space(V.extract_sub_space(c));
+  *static_cast<FunctionSpace*>(this) = *_function_space;
 }
 //-----------------------------------------------------------------------------
 SubSpace::SubSpace(const FunctionSpace& V, const std::vector<std::size_t>& component)
   : FunctionSpace(V.mesh(), V.element(), V.dofmap())
 {
   // Extract subspace and assign
-  boost::shared_ptr<FunctionSpace> _V(V.extract_sub_space(component));
-  *static_cast<FunctionSpace*>(this) = *_V;
+  std::shared_ptr<FunctionSpace> _function_space(V.extract_sub_space(component));
+  *static_cast<FunctionSpace*>(this) = *_function_space;
 }
 //-----------------------------------------------------------------------------
