@@ -18,9 +18,11 @@
 // Modified by Joachim B Haga 2012
 //
 // First added:  2012-05-10
-// Last changed: 2012-11-14
+// Last changed: 2014-02-06
 
 
+#include <memory>
+
 #include "CSGCGALMeshGenerator3D.h"
 #include "CSGGeometry.h"
 #include "GeometryToCGALConverter.h"
@@ -28,7 +30,6 @@
 #include <dolfin/log/LogStream.h>
 #include <dolfin/mesh/BoundaryMesh.h>
 #include <dolfin/mesh/MeshEditor.h>
-#include <boost/scoped_ptr.hpp>
 
 using namespace dolfin;
 
@@ -62,8 +63,9 @@ static void build_dolfin_mesh(const csg::C3t3& c3t3, Mesh& mesh)
   // Create and initialize mesh editor
   dolfin::MeshEditor mesh_editor;
   mesh_editor.open(mesh, 3, 3);
-  mesh_editor.init_vertices(triangulation.number_of_vertices());
-  mesh_editor.init_cells(num_cells);
+  mesh_editor.init_vertices_global(triangulation.number_of_vertices(),
+                                   triangulation.number_of_vertices());
+  mesh_editor.init_cells_global(num_cells, num_cells);
 
   // Add vertices to mesh
   std::size_t vertex_index = 0;
@@ -103,12 +105,12 @@ static void build_dolfin_mesh(const csg::C3t3& c3t3, Mesh& mesh)
 //-----------------------------------------------------------------------------
 CSGCGALMeshGenerator3D::CSGCGALMeshGenerator3D(const CSGGeometry& geometry)
 {
-  boost::shared_ptr<const CSGGeometry> tmp = reference_to_no_delete_pointer<const CSGGeometry>(geometry);
+  std::shared_ptr<const CSGGeometry> tmp = reference_to_no_delete_pointer<const CSGGeometry>(geometry);
   _geometry = tmp;
   parameters = default_parameters();
 }
 //-----------------------------------------------------------------------------
-CSGCGALMeshGenerator3D::CSGCGALMeshGenerator3D(boost::shared_ptr<const CSGGeometry> geometry)
+CSGCGALMeshGenerator3D::CSGCGALMeshGenerator3D(std::shared_ptr<const CSGGeometry> geometry)
   : _geometry(geometry)
 {
   parameters = default_parameters();
@@ -135,7 +137,7 @@ void CSGCGALMeshGenerator3D::generate(Mesh& mesh) const
 
   // Workaround, cgal segfaulted when assigning new mesh criterias
   // within the if-else blocks.
-  boost::scoped_ptr<csg::Mesh_criteria> criteria;
+  std::unique_ptr<csg::Mesh_criteria> criteria;
 
   int mesh_resolution = parameters["mesh_resolution"];
   if (mesh_resolution > 0)
@@ -221,7 +223,7 @@ CSGCGALMeshGenerator3D::CSGCGALMeshGenerator3D(const CSGGeometry& geometry)
 	       "Dolfin must be compiled with CGAL to use this feature.");
 }
 //-----------------------------------------------------------------------------
-CSGCGALMeshGenerator3D::CSGCGALMeshGenerator3D(boost::shared_ptr<const CSGGeometry> geometry)
+CSGCGALMeshGenerator3D::CSGCGALMeshGenerator3D(std::shared_ptr<const CSGGeometry> geometry)
 {
   dolfin_error("CSGCGALMeshGenerator3D.cpp",
 	       "Create mesh generator",
