@@ -23,8 +23,10 @@
 
 #include <cstddef>
 #include <set>
+#include <string>
 #include <vector>
 
+#include <dolfin/common/MPI.h>
 #include "Graph.h"
 
 namespace dolfin
@@ -39,25 +41,37 @@ namespace dolfin
   public:
 
     // Compute cell partition
-    static void compute_partition(std::vector<std::size_t>& cell_partition,
+    static void compute_partition(const MPI_Comm mpi_comm,
+                                  std::vector<std::size_t>& cell_partition,
                                   const LocalMeshData& mesh_data);
 
-    // Compute graph re-ordering
-    static std::vector<std::size_t> compute_reordering(const Graph& graph);
+    /// Compute reordering (map[old] -> new) using
+    /// Gibbs-Poole-Stockmeyer re-ordering
+    static std::vector<std::size_t> compute_gps(const Graph& graph,
+                                                std::size_t num_passes=5);
 
     // Compute graph re-ordering
-    static void compute_reordering(const Graph& graph,
-                                   std::vector<std::size_t>& permutation,
-                                   std::vector<std::size_t>& inverse_permutation);
+    static std::vector<std::size_t>
+      compute_reordering(const Graph& graph,
+                         std::string scotch_strategy="");
+
+    // Compute graph re-ordering
+    static
+      void compute_reordering(const Graph& graph,
+                              std::vector<std::size_t>& permutation,
+                              std::vector<std::size_t>& inverse_permutation,
+                              std::string scotch_strategy="");
 
   private:
 
     // Compute cell partitions from distribted dual graph
-    static void partition(const std::vector<std::set<std::size_t> >& local_graph,
-                          const std::set<std::size_t>& ghost_vertices,
-                          const std::vector<std::size_t>& global_cell_indices,
-                          const std::size_t num_global_vertices,
-                          std::vector<std::size_t>& cell_partition);
+    static
+      void partition(const MPI_Comm mpi_comm,
+                     const std::vector<std::set<std::size_t> >& local_graph,
+                     const std::set<std::size_t>& ghost_vertices,
+                     const std::vector<std::size_t>& global_cell_indices,
+                     const std::size_t num_global_vertices,
+                     std::vector<std::size_t>& cell_partition);
 
   };
 
