@@ -16,9 +16,6 @@
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 //
 // Modified by Garth N. Wells, 2012
-//
-// First added:  2012-06-01
-// Last changed: 2013-11-05
 
 #ifdef HAS_HDF5
 
@@ -58,16 +55,38 @@
 using namespace dolfin;
 
 //-----------------------------------------------------------------------------
-HDF5File::HDF5File(const std::string filename, const std::string file_mode,
-                   bool use_mpiio)
-  : hdf5_file_open(false), hdf5_file_id(0),
-    mpi_io(MPI::num_processes() > 1 && use_mpiio ? true : false)
+HDF5File::HDF5File(MPI_Comm comm, const std::string filename,
+                   const std::string file_mode)
+  : hdf5_file_open(false), hdf5_file_id(0), _mpi_comm(comm)
 {
   // HDF5 chunking
   parameters.add("chunking", false);
 
+  // Create directory if required (create on rank 0)
+  if (MPI::rank(_mpi_comm) == 0)
+  {
+    const boost::filesystem::path path(filename);
+    if (path.has_parent_path()
+        && !boost::filesystem::is_directory(path.parent_path()))
+    {
+      boost::filesystem::create_directories(path.parent_path());
+      if (!boost::filesystem::is_directory(path.parent_path()))
+      {
+        dolfin_error("HDF5File.cpp",
+                     "open file",
+                     "Could not create directory \"%s\"",
+                     path.parent_path().string().c_str());
+      }
+    }
+  }
+
+  // Wait until directory has been created
+  MPI::barrier(_mpi_comm);
+
   // Open HDF5 file
-  hdf5_file_id = HDF5Interface::open_file(filename, file_mode, mpi_io);
+  const bool mpi_io = MPI::size(_mpi_comm) > 1 ? true : false;
+  hdf5_file_id = HDF5Interface::open_file(_mpi_comm, filename, file_mode,
+                                          mpi_io);
   hdf5_file_open = true;
 }
 //-----------------------------------------------------------------------------
@@ -89,6 +108,43 @@ void HDF5File::flush()
   HDF5Interface::flush_file(hdf5_file_id);
 }
 //-----------------------------------------------------------------------------
+void HDF5File::write(const std::vector<Point>& points,
+                     const std::string dataset_name)
+{
+  dolfin_assert(points.size() > 0);
+  dolfin_assert(hdf5_file_open);
+
+  // Get number of points (global)
+  std::size_t num_points_global = MPI::sum(_mpi_comm, points.size());
+
+  // Data set name
+  const std::string coord_dataset =  dataset_name + "/coordinates";
+
+  // Pack data
+  const std::size_t n = points.size();
+  std::vector<double> x(3*n);
+  for (std::size_t i = 0; i< n; ++i)
+    for (std::size_t j = 0; j < 3; ++j)
+      x[3*i + j] = points[i][j];
+
+  // Write data to file
+  //  const bool chunking = parameters["chunking"];
+  std::vector<std::size_t> global_size(2);
+  global_size[0] = num_points_global;
+  global_size[1] = 3;
+
+  const bool mpi_io = MPI::size(_mpi_comm) > 1 ? true : false;
+  write_data(coord_dataset, x, global_size, mpi_io);
+}
+//-----------------------------------------------------------------------------
+void HDF5File::write(const std::vector<double>& values,
+                     const std::string dataset_name)
+{
+  std::vector<std::size_t> global_size(1, MPI::sum(_mpi_comm, values.size()));
+  const bool mpi_io = MPI::size(_mpi_comm) > 1 ? true : false;
+  write_data(dataset_name, values, global_size, mpi_io);
+}
+//-----------------------------------------------------------------------------
 void HDF5File::write(const GenericVector& x, const std::string dataset_name)
 {
   dolfin_assert(x.size() > 0);
@@ -102,13 +158,15 @@ void HDF5File::write(const GenericVector& x, const std::string dataset_name)
   std::pair<std::size_t, std::size_t> local_range = x.local_range();
   const bool chunking = parameters["chunking"];
   const std::vector<std::size_t> global_size(1, x.size());
+  const bool mpi_io = MPI::size(_mpi_comm) > 1 ? true : false;
   HDF5Interface::write_dataset(hdf5_file_id, dataset_name, local_data,
                                local_range, global_size, mpi_io, chunking);
 
   // Add partitioning attribute to dataset
   std::vector<std::size_t> partitions;
-  MPI::gather(local_range.first, partitions);
-  MPI::broadcast(partitions);
+  std::vector<std::size_t> local_range_first(1, local_range.first);
+  MPI::gather(_mpi_comm, local_range_first, partitions);
+  MPI::broadcast(_mpi_comm, partitions);
 
   HDF5Interface::add_attribute(hdf5_file_id, dataset_name, "partition",
                                partitions);
@@ -136,9 +194,9 @@ void HDF5File::read(GenericVector& x, const std::string dataset_name,
   dolfin_assert(data_size.size() == 1);
 
   // Check input vector, and re-size if not already sized
-  if (x.size() == 0)
+  if (x.empty())
   {
-    // Resize vector
+    // Intialize vector
     if (use_partition_from_file)
     {
       // Get partition from file
@@ -147,7 +205,7 @@ void HDF5File::read(GenericVector& x, const std::string dataset_name,
                                    partitions);
 
       // Check that number of MPI processes matches partitioning
-      if (MPI::num_processes() != partitions.size())
+      if (MPI::size(_mpi_comm) != partitions.size())
       {
         dolfin_error("HDF5File.cpp",
                      "read vector from file",
@@ -158,13 +216,13 @@ void HDF5File::read(GenericVector& x, const std::string dataset_name,
       partitions.push_back(data_size[0]);
 
       // Initialise vector
-      const std::size_t process_num = MPI::process_number();
+      const std::size_t process_num = MPI::rank(_mpi_comm);
       const std::pair<std::size_t, std::size_t>
         local_range(partitions[process_num], partitions[process_num + 1]);
-      x.resize(local_range);
+      x.init(_mpi_comm, local_range);
     }
     else
-      x.resize(data_size[0]);
+      x.init(_mpi_comm, data_size[0]);
   }
   else if (x.size() != data_size[0])
   {
@@ -182,6 +240,7 @@ void HDF5File::read(GenericVector& x, const std::string dataset_name,
 
   // Set data
   x.set_local(data);
+  x.apply("insert");
 }
 //-----------------------------------------------------------------------------
 void HDF5File::write(const Mesh& mesh, const std::string name)
@@ -208,10 +267,11 @@ void HDF5File::write(const Mesh& mesh, std::size_t cell_dim,
 
     // Write coordinates out from each process
     std::vector<std::size_t> global_size(2);
-    global_size[0] = MPI::sum(vertex_coords.size()/gdim);
+    global_size[0] = MPI::sum(_mpi_comm, vertex_coords.size()/gdim);
     global_size[1] = gdim;
     dolfin_assert(global_size[0] == mesh.size_global(0));
-    write_data(coord_dataset, vertex_coords, global_size);
+    const bool mpi_io = MPI::size(_mpi_comm) > 1 ? true : false;
+    write_data(coord_dataset, vertex_coords, global_size, mpi_io);
   }
 
   // ---------- Topology
@@ -219,7 +279,7 @@ void HDF5File::write(const Mesh& mesh, std::size_t cell_dim,
     std::vector<std::size_t> topological_data;
     topological_data.reserve(mesh.num_entities(cell_dim)*(cell_dim + 1));
 
-    if (cell_dim == mesh.topology().dim() || MPI::num_processes() == 1)
+    if (cell_dim == mesh.topology().dim() || MPI::size(_mpi_comm) == 1)
     {
       // Usual case, with cell output, and/or none shared with another
       // process.
@@ -237,7 +297,7 @@ void HDF5File::write(const Mesh& mesh, std::size_t cell_dim,
       // we can get shared_entities
       DistributedMeshTools::number_entities(mesh, cell_dim);
 
-      const std::size_t my_rank = MPI::process_number();
+      const std::size_t my_rank = MPI::rank(_mpi_comm);
       const std::map<unsigned int, std::set<unsigned int> >& shared_entities
         = mesh.topology().shared_entities(cell_dim);
 
@@ -268,10 +328,12 @@ void HDF5File::write(const Mesh& mesh, std::size_t cell_dim,
     // Write topology data
     const std::string topology_dataset =  name + "/topology";
     std::vector<std::size_t> global_size(2);
-    global_size[0] = MPI::sum(topological_data.size()/(cell_dim + 1));
+    global_size[0] = MPI::sum(_mpi_comm,
+                              topological_data.size()/(cell_dim + 1));
     global_size[1] = cell_dim + 1;
     dolfin_assert(global_size[0] == mesh.size_global(cell_dim));
-    write_data(topology_dataset, topological_data, global_size);
+    const bool mpi_io = MPI::size(_mpi_comm) > 1 ? true : false;
+    write_data(topology_dataset, topological_data, global_size, mpi_io);
 
     // For cells, write the global cell index
     if (cell_dim == mesh.topology().dim())
@@ -280,7 +342,8 @@ void HDF5File::write(const Mesh& mesh, std::size_t cell_dim,
       global_size.pop_back();
       const std::vector<std::size_t>& cells =
         mesh.topology().global_indices(mesh.topology().dim());
-      write_data(cell_index_dataset, cells, global_size);
+      const bool mpi_io = MPI::size(_mpi_comm) > 1 ? true : false;
+      write_data(cell_index_dataset, cells, global_size, mpi_io);
     }
 
     // Add cell type attribute
@@ -290,9 +353,12 @@ void HDF5File::write(const Mesh& mesh, std::size_t cell_dim,
     // Add partitioning attribute to dataset
     std::vector<std::size_t> partitions;
     const std::size_t topology_offset
-      = MPI::global_offset(topological_data.size()/(cell_dim + 1), true);
-    MPI::gather(topology_offset, partitions);
-    MPI::broadcast(partitions);
+      = MPI::global_offset(_mpi_comm, topological_data.size()/(cell_dim + 1),
+                           true);
+
+    std::vector<std::size_t> topology_offset_tmp(1, topology_offset);
+    MPI::gather(_mpi_comm, topology_offset_tmp, partitions);
+    MPI::broadcast(_mpi_comm, partitions);
     HDF5Interface::add_attribute(hdf5_file_id, topology_dataset,
                                  "partition", partitions);
   }
@@ -434,7 +500,7 @@ void HDF5File::read_mesh_function(MeshFunction<T>& meshfunction,
 
   // Divide up cells ~equally between processes
   const std::pair<std::size_t, std::size_t> cell_range
-    = MPI::local_range(num_global_cells);
+    = MPI::local_range(_mpi_comm, num_global_cells);
   const std::size_t num_read_cells = cell_range.second - cell_range.first;
 
   // Read a block of cells
@@ -454,14 +520,11 @@ void HDF5File::read_mesh_function(MeshFunction<T>& meshfunction,
 
   // Now send the read data to each process on the basis of the first
   // vertex of the entity, since we do not know the global_index
-  const std::size_t num_processes = MPI::num_processes();
+  const std::size_t num_processes = MPI::size(_mpi_comm);
   const std::size_t max_vertex = mesh.size_global(0);
 
   std::vector<std::vector<std::size_t> > send_topology(num_processes);
-  std::vector<std::vector<std::size_t> > receive_topology(num_processes);
   std::vector<std::vector<T> > send_values(num_processes);
-  std::vector<std::vector<T> > receive_values(num_processes);
-
   for (std::size_t i = 0; i < num_read_cells ; ++i)
   {
     std::vector<std::size_t> cell_topology(topology_array[i].begin(),
@@ -470,25 +533,25 @@ void HDF5File::read_mesh_function(MeshFunction<T>& meshfunction,
 
     // Use first vertex to decide where to send this data
     const std::size_t send_to_process
-      = MPI::index_owner(cell_topology.front(), max_vertex);
+      = MPI::index_owner(_mpi_comm, cell_topology.front(), max_vertex);
 
     send_topology[send_to_process].insert(send_topology[send_to_process].end(),
-                              cell_topology.begin(), cell_topology.end());
+                                          cell_topology.begin(),
+                                          cell_topology.end());
     send_values[send_to_process].push_back(value_data[i]);
   }
 
-  MPI::all_to_all(send_topology, receive_topology);
-  MPI::all_to_all(send_values, receive_values);
+  std::vector<std::vector<std::size_t> > receive_topology(num_processes);
+  std::vector<std::vector<T> > receive_values(num_processes);
+  MPI::all_to_all(_mpi_comm, send_topology, receive_topology);
+  MPI::all_to_all(_mpi_comm, send_values, receive_values);
 
   // Generate requests for data from remote processes, based on the
   // first vertex of the MeshEntities which belong on this process
   // Send our process number, and our local index, so it can come back
   // directly to the right place
   std::vector<std::vector<std::size_t> > send_requests(num_processes);
-  std::vector<std::vector<std::size_t> > receive_requests(num_processes);
-
-  const std::size_t process_number = MPI::process_number();
-
+  const std::size_t process_number = MPI::rank(_mpi_comm);
   for (MeshEntityIterator cell(mesh, cell_dim); !cell.end(); ++cell)
   {
     std::vector<std::size_t> cell_topology;
@@ -499,7 +562,8 @@ void HDF5File::read_mesh_function(MeshFunction<T>& meshfunction,
     std::sort(cell_topology.begin(), cell_topology.end());
 
     // Use first vertex to decide where to send this request
-    std::size_t send_to_process = MPI::index_owner(cell_topology.front(),
+    std::size_t send_to_process = MPI::index_owner(_mpi_comm,
+                                                   cell_topology.front(),
                                                    max_vertex);
     // Map to this process and local index by appending to send data
     cell_topology.push_back(cell->index());
@@ -509,7 +573,8 @@ void HDF5File::read_mesh_function(MeshFunction<T>& meshfunction,
                                           cell_topology.end());
   }
 
-  MPI::all_to_all(send_requests, receive_requests);
+  std::vector<std::vector<std::size_t> > receive_requests(num_processes);
+  MPI::all_to_all(_mpi_comm, send_requests, receive_requests);
 
   // At this point, the data with its associated vertices is in
   // receive_values and receive_topology and the final destinations
@@ -545,7 +610,7 @@ void HDF5File::read_mesh_function(MeshFunction<T>& meshfunction,
   for (std::size_t i = 0; i < receive_requests.size(); ++i)
   {
     for (std::vector<std::size_t>::iterator p = receive_requests[i].begin();
-        p != receive_requests[i].end(); p += (vert_per_cell + 2))
+         p != receive_requests[i].end(); p += (vert_per_cell + 2))
     {
       const std::vector<std::size_t> cell(p, p + vert_per_cell);
       const std::size_t remote_index = *(p + vert_per_cell);
@@ -558,19 +623,16 @@ void HDF5File::read_mesh_function(MeshFunction<T>& meshfunction,
     }
   }
 
-  MPI::all_to_all(send_topology, receive_topology);
-  MPI::all_to_all(send_values, receive_values);
+  MPI::all_to_all(_mpi_comm, send_topology, receive_topology);
+  MPI::all_to_all(_mpi_comm, send_values, receive_values);
 
   // At this point, receive_topology should only list the local indices
   // and received values should have the appropriate values for each
-
   for (std::size_t i = 0; i < receive_values.size(); ++i)
   {
     dolfin_assert(receive_values[i].size() == receive_topology[i].size());
     for (std::size_t j = 0; j < receive_values[i].size(); ++j)
-    {
       meshfunction[receive_topology[i][j]] = receive_values[i][j];
-    }
   }
 }
 //-----------------------------------------------------------------------------
@@ -578,7 +640,6 @@ template <typename T>
 void HDF5File::write_mesh_function(const MeshFunction<T>& meshfunction,
                                    const std::string name)
 {
-
   if (meshfunction.size() == 0)
   {
     dolfin_error("HDF5File.cpp",
@@ -596,7 +657,7 @@ void HDF5File::write_mesh_function(const MeshFunction<T>& meshfunction,
   // Storage for output values
   std::vector<T> data_values;
 
-  if (cell_dim == mesh.topology().dim() || MPI::num_processes() == 1)
+  if (cell_dim == mesh.topology().dim() || MPI::size(_mpi_comm) == 1)
   {
     // No duplicates
     data_values.assign(meshfunction.values(),
@@ -607,7 +668,7 @@ void HDF5File::write_mesh_function(const MeshFunction<T>& meshfunction,
     data_values.reserve(mesh.size(cell_dim));
 
     // Drop duplicate data
-    const std::size_t my_rank = MPI::process_number();
+    const std::size_t my_rank = MPI::rank(_mpi_comm);
     const std::map<unsigned int, std::set<unsigned int> >& shared_entities
       = mesh.topology().shared_entities(cell_dim);
 
@@ -629,8 +690,10 @@ void HDF5File::write_mesh_function(const MeshFunction<T>& meshfunction,
   }
 
   // Write values to HDF5
-  std::vector<std::size_t> global_size(1, MPI::sum(data_values.size()));
-  write_data(name + "/values", data_values, global_size);
+  std::vector<std::size_t> global_size(1, MPI::sum(_mpi_comm,
+                                                   data_values.size()));
+  const bool mpi_io = MPI::size(_mpi_comm) > 1 ? true : false;
+  write_data(name + "/values", data_values, global_size, mpi_io);
 }
 //-----------------------------------------------------------------------------
 void HDF5File::write(const Function& u,  const std::string name,
@@ -695,25 +758,28 @@ void HDF5File::write(const Function& u, const std::string name)
   }
 
   // Add offset to CSR index to be seamless in parallel
-  std::size_t offset = MPI::global_offset(cell_dofs.size(), true);
+  std::size_t offset = MPI::global_offset(_mpi_comm, cell_dofs.size(), true);
   std::transform(x_cell_dofs.begin(),
                  x_cell_dofs.end(),
                  x_cell_dofs.begin(),
                  std::bind2nd(std::plus<std::size_t>(), offset));
 
+  const bool mpi_io = MPI::size(_mpi_comm) > 1 ? true : false;
+
   // Save DOFs on each cell
-  std::vector<std::size_t> global_size(1, MPI::sum(cell_dofs.size()));
-  write_data(name + "/cell_dofs", cell_dofs, global_size);
-  if (MPI::process_number() == MPI::num_processes() - 1)
+  std::vector<std::size_t> global_size(1, MPI::sum(_mpi_comm,
+                                                   cell_dofs.size()));
+  write_data(name + "/cell_dofs", cell_dofs, global_size, mpi_io);
+  if (MPI::rank(_mpi_comm) == MPI::size(_mpi_comm) - 1)
     x_cell_dofs.push_back(global_size[0]);
   global_size[0] = mesh.size_global(mesh.topology().dim()) + 1;
-  write_data(name + "/x_cell_dofs", x_cell_dofs, global_size);
+  write_data(name + "/x_cell_dofs", x_cell_dofs, global_size, mpi_io);
 
   // Save cell ordering
   const std::vector<std::size_t>& cells =
     mesh.topology().global_indices(mesh.topology().dim());
   global_size[0] = mesh.size_global(mesh.topology().dim());
-  write_data(name + "/cells", cells, global_size);
+  write_data(name + "/cells", cells, global_size, mpi_io);
 
   // Save vector
   write(*u.vector(), name + "/vector");
@@ -784,7 +850,7 @@ void HDF5File::read(Function& u, const std::string name)
 
   // Divide cells equally between processes
   const std::pair<std::size_t, std::size_t> cell_range
-    = MPI::local_range(num_global_cells);
+    = MPI::local_range(_mpi_comm, num_global_cells);
 
   // Read cells
   std::vector<std::size_t> input_cells;
@@ -812,7 +878,7 @@ void HDF5File::read(Function& u, const std::string name)
   const std::size_t num_global_dofs = vector_size[0];
   dolfin_assert(num_global_dofs == x.size(0));
   const std::pair<dolfin::la_index, dolfin::la_index>
-    input_vector_range = MPI::local_range(vector_size[0]);
+    input_vector_range = MPI::local_range(_mpi_comm, vector_size[0]);
 
   std::vector<double> input_values;
   HDF5Interface::read_dataset(hdf5_file_id, vector_dataset_name,
@@ -825,7 +891,8 @@ void HDF5File::read(Function& u, const std::string name)
   std::vector<std::size_t> global_cells;
   std::vector<std::size_t> remote_local_dofi;
 
-  HDF5Utility::map_gdof_to_cell(input_cells, input_cell_dofs,
+  HDF5Utility::map_gdof_to_cell(_mpi_comm,
+                                input_cells, input_cell_dofs,
                                 x_cell_dofs, input_vector_range,
                                 global_cells, remote_local_dofi);
 
@@ -841,13 +908,13 @@ void HDF5File::read(Function& u, const std::string name)
   // Having found the cell location, the actual global_dof index
   // held by that (cell, local_dof) is needed on the process
   // which holds the data values
-
   std::vector<dolfin::la_index> global_dof;
-  HDF5Utility::get_global_dof(cell_ownership, remote_local_dofi,
-                              input_vector_range, dofmap, global_dof);
+  HDF5Utility::get_global_dof(_mpi_comm, cell_ownership,
+                              remote_local_dofi, input_vector_range, dofmap,
+                              global_dof);
 
 
-  const std::size_t num_processes = MPI::num_processes();
+  const std::size_t num_processes = MPI::size(_mpi_comm);
 
   // Shift to dividing things into the vector range of Function Vector
   const std::pair<dolfin::la_index, dolfin::la_index>
@@ -861,8 +928,10 @@ void HDF5File::read(Function& u, const std::string name)
     const std::size_t
       n_vector_vals = input_vector_range.second - input_vector_range.first;
     std::vector<dolfin::la_index> all_vec_range;
-    MPI::gather(vector_range.second, all_vec_range);
-    MPI::broadcast(all_vec_range);
+
+    std::vector<dolfin::la_index> vector_range_second(1, vector_range.second);
+    MPI::gather(_mpi_comm, vector_range_second, all_vec_range);
+    MPI::broadcast(_mpi_comm, all_vec_range);
 
     for (std::size_t i = 0; i != n_vector_vals; ++i)
     {
@@ -876,8 +945,8 @@ void HDF5File::read(Function& u, const std::string name)
       send_values[dest].push_back(input_values[i]);
     }
 
-    MPI::all_to_all(send_values, receive_values);
-    MPI::all_to_all(send_indices, receive_indices);
+    MPI::all_to_all(_mpi_comm, send_values, receive_values);
+    MPI::all_to_all(_mpi_comm, send_indices, receive_indices);
   }
 
   std::vector<double>
@@ -898,6 +967,7 @@ void HDF5File::read(Function& u, const std::string name)
   }
 
   x.set_local(vector_values);
+  x.apply("insert");
 }
 //-----------------------------------------------------------------------------
 void HDF5File::write(const MeshValueCollection<std::size_t>& mesh_values,
@@ -987,10 +1057,12 @@ void HDF5File::write_mesh_value_collection(const MeshValueCollection<T>& mesh_va
     data_values.push_back(p->second);
   }
 
-  std::vector<std::size_t> global_size(1, MPI::sum(data_values.size()));
-  write_data(name + "/values", data_values, global_size);
-  write_data(name + "/entities", entities, global_size);
-  write_data(name + "/cells", cells, global_size);
+  std::vector<std::size_t> global_size(1, MPI::sum(_mpi_comm,
+                                                   data_values.size()));
+  const bool mpi_io = MPI::size(_mpi_comm) > 1 ? true : false;
+  write_data(name + "/values", data_values, global_size, mpi_io);
+  write_data(name + "/entities", entities, global_size, mpi_io);
+  write_data(name + "/cells", cells, global_size, mpi_io);
 
   HDF5Interface::add_attribute(hdf5_file_id, name, "dimension",
                                mesh_values.dim());
@@ -1099,7 +1171,7 @@ void HDF5File::read_mesh_value_collection(MeshValueCollection<T>& mesh_vc,
 
     // Divide range between processes
     const std::pair<std::size_t, std::size_t> data_range
-      = MPI::local_range(values_dim[0]);
+      = MPI::local_range(_mpi_comm, values_dim[0]);
     const std::size_t local_size = data_range.second - data_range.first;
 
     // Read local range of values, entities and cells
@@ -1119,14 +1191,10 @@ void HDF5File::read_mesh_value_collection(MeshValueCollection<T>& mesh_vc,
     std::vector<std::pair<std::size_t, std::size_t> > cell_ownership;
     cell_ownership = HDF5Utility::cell_owners(mesh, cells_data);
 
-    const std::size_t num_processes = MPI::num_processes();
+    const std::size_t num_processes = MPI::size(_mpi_comm);
     std::vector<std::vector<std::size_t> > send_entities(num_processes);
     std::vector<std::vector<std::size_t> > send_local(num_processes);
     std::vector<std::vector<T> > send_values(num_processes);
-    std::vector<std::vector<std::size_t> > recv_entities(num_processes);
-    std::vector<std::vector<std::size_t> > recv_local(num_processes);
-    std::vector<std::vector<T> > recv_values(num_processes);
-
     for (std::size_t i = 0; i != cells_data.size(); ++i)
     {
       const std::size_t dest = cell_ownership[i].first;
@@ -1135,9 +1203,12 @@ void HDF5File::read_mesh_value_collection(MeshValueCollection<T>& mesh_vc,
       send_values[dest].push_back(values_data[i]);
     }
 
-    MPI::all_to_all(send_entities, recv_entities);
-    MPI::all_to_all(send_local, recv_local);
-    MPI::all_to_all(send_values, recv_values);
+    std::vector<std::vector<T> > recv_values(num_processes);
+    std::vector<std::vector<std::size_t> > recv_entities(num_processes);
+    std::vector<std::vector<std::size_t> > recv_local(num_processes);
+    MPI::all_to_all(_mpi_comm, send_entities, recv_entities);
+    MPI::all_to_all(_mpi_comm, send_local, recv_local);
+    MPI::all_to_all(_mpi_comm, send_values, recv_values);
 
     // Reference to actual map of MeshValueCollection
     std::map<std::pair<std::size_t, std::size_t>, T>& mvc_map
@@ -1161,7 +1232,8 @@ void HDF5File::read_mesh_value_collection(MeshValueCollection<T>& mesh_vc,
   }
 }
 //-----------------------------------------------------------------------------
-void HDF5File::read(Mesh& input_mesh, const std::string mesh_name) const
+void HDF5File::read(Mesh& input_mesh, const std::string mesh_name,
+                    bool use_partition_from_file) const
 {
   Timer t("HDF5: read mesh");
 
@@ -1183,7 +1255,8 @@ void HDF5File::read(Mesh& input_mesh, const std::string mesh_name) const
                  "Dataset \"%s\" not found", coordinates_name.c_str());
   }
 
-  LocalMeshData mesh_data;
+  // Structure to store local mesh
+  LocalMeshData mesh_data(_mpi_comm);
   mesh_data.clear();
 
   // --- Topology ---
@@ -1210,16 +1283,25 @@ void HDF5File::read(Mesh& input_mesh, const std::string mesh_name) const
 
   // Check whether number of MPI processes matches partitioning, and
   // restore if possible
-  if (MPI::num_processes() == partitions.size())
+  if (MPI::size(_mpi_comm) == partitions.size())
   {
     partitions.push_back(num_global_cells);
-    const std::size_t proc = MPI::process_number();
+    const std::size_t proc = MPI::rank(_mpi_comm);
     cell_range = std::make_pair(partitions[proc], partitions[proc + 1]);
+
+    // Restore partitioning if requested
+    if (use_partition_from_file)
+    {
+      mesh_data.cell_partition
+        = std::vector<std::size_t>(cell_range.second - cell_range.first, proc);
+    }
   }
   else
   {
+    if (use_partition_from_file)
+      warning("Could not use partition from file: wrong size");
     // Divide up cells ~equally between processes
-    cell_range = MPI::local_range(num_global_cells);
+    cell_range = MPI::local_range(_mpi_comm, num_global_cells);
   }
 
   const std::size_t num_local_cells = cell_range.second - cell_range.first;
@@ -1258,7 +1340,7 @@ void HDF5File::read(Mesh& input_mesh, const std::string mesh_name) const
 
   // Divide range into equal blocks for each process
   const std::pair<std::size_t, std::size_t> vertex_range
-    = MPI::local_range(mesh_data.num_global_vertices);
+    = MPI::local_range(_mpi_comm, mesh_data.num_global_vertices);
   const std::size_t num_local_vertices
     = vertex_range.second - vertex_range.first;
 
@@ -1282,7 +1364,7 @@ void HDF5File::read(Mesh& input_mesh, const std::string mesh_name) const
 
   t.stop();
 
-  if (MPI::num_processes() == 1)
+  if (MPI::size(_mpi_comm) == 1)
     HDF5Utility::build_local_mesh(input_mesh, mesh_data);
   else
     MeshPartitioning::build_distributed_mesh(input_mesh, mesh_data);
