@@ -118,7 +118,6 @@ void VTKFile::operator<<(const MeshFunction<double>& meshfunction)
 //----------------------------------------------------------------------------
 void VTKFile::operator<<(const Function& u)
 {
-  u.update();
   write_function(u, counter);
 }
 //----------------------------------------------------------------------------
@@ -156,7 +155,6 @@ void VTKFile::operator<<(const std::pair<const MeshFunction<bool>*, double> f)
 void VTKFile::operator<<(const std::pair<const Function*, double> u)
 {
   dolfin_assert(u.first);
-  u.first->update();
   write_function(*(u.first), u.second);
 }
 //----------------------------------------------------------------------------
@@ -165,6 +163,9 @@ void VTKFile::write_function(const Function& u, double time)
   dolfin_assert(u.function_space()->mesh());
   const Mesh& mesh = *u.function_space()->mesh();
 
+  // Get MPI communicator
+  const MPI_Comm mpi_comm = mesh.mpi_comm();
+
   // Get vtu file name and intialise
   std::string vtu_filename = init(mesh, mesh.topology().dim());
 
@@ -176,13 +177,14 @@ void VTKFile::write_function(const Function& u, double time)
   results_write(u, vtu_filename);
 
   // Parallel-specfic files
-  if (MPI::num_processes() > 1 && MPI::process_number() == 0)
+  const std::size_t num_processes = MPI::size(mpi_comm);
+  if (num_processes > 1 && MPI::rank(mpi_comm) == 0)
   {
     std::string pvtu_filename = vtu_name(0, 0, counter, ".pvtu");
     pvtu_write(u, pvtu_filename);
     pvd_file_write(counter, time, pvtu_filename);
   }
-  else if (MPI::num_processes() == 1)
+  else if (num_processes == 1)
     pvd_file_write(counter, time, vtu_filename);
 
   // Finalise and write pvd files
@@ -196,6 +198,9 @@ void VTKFile::write_mesh(const Mesh& mesh, double time)
 {
   Timer t("Write mesh to PVD/VTK file");
 
+  // Get MPI communicator
+  const MPI_Comm mpi_comm = mesh.mpi_comm();
+
   // Get vtu file name and intialise out files
   std::string vtu_filename = init(mesh, mesh.topology().dim());
 
@@ -204,13 +209,14 @@ void VTKFile::write_mesh(const Mesh& mesh, double time)
                         compress);
 
   // Parallel-specific files
-  if (MPI::num_processes() > 1 && MPI::process_number() == 0)
+  const std::size_t num_processes = MPI::size(mpi_comm);
+  if (num_processes > 1 && MPI::rank(mpi_comm) == 0)
   {
     std::string pvtu_filename = vtu_name(0, 0, counter, ".pvtu");
-    pvtu_write_mesh(pvtu_filename);
+    pvtu_write_mesh(pvtu_filename, num_processes);
     pvd_file_write(counter, time, pvtu_filename);
   }
-  else if (MPI::num_processes() == 1)
+  else if (num_processes == 1)
     pvd_file_write(counter, time, vtu_filename);
 
   // Finalise
@@ -222,11 +228,13 @@ void VTKFile::write_mesh(const Mesh& mesh, double time)
 //----------------------------------------------------------------------------
 std::string VTKFile::init(const Mesh& mesh, std::size_t cell_dim) const
 {
+  // Get MPI communicators
+  const MPI_Comm mpi_comm = mesh.mpi_comm();
+
   // Get vtu file name and clear file
-  std::string vtu_filename = vtu_name(MPI::process_number(),
-                                      MPI::num_processes(),
-                                      counter,
-                                      ".vtu");
+  std::string vtu_filename = vtu_name(MPI::rank(mpi_comm),
+                                      MPI::size(mpi_comm),
+                                      counter, ".vtu");
   clear_file(vtu_filename);
 
   // Number of cells
@@ -473,7 +481,8 @@ void VTKFile::pvtu_write_mesh(pugi::xml_node xml_node) const
 void VTKFile::pvtu_write_function(std::size_t dim, std::size_t rank,
                                   const std::string data_location,
                                   const std::string name,
-                                  const std::string fname) const
+                                  const std::string fname,
+                                  std::size_t num_processes) const
 {
   // Create xml doc
   pugi::xml_document xml_doc;
@@ -538,9 +547,9 @@ void VTKFile::pvtu_write_function(std::size_t dim, std::size_t rank,
     = (unsigned int) num_components;
 
   // Write vtu file list
-  for(std::size_t i = 0; i < MPI::num_processes(); i++)
+  for(std::size_t i = 0; i <num_processes; i++)
   {
-    const std::string tmp_string = strip_path(vtu_name(i, MPI::num_processes(),
+    const std::string tmp_string = strip_path(vtu_name(i, num_processes,
                                                        counter, ".vtu"));
     pugi::xml_node piece_node = grid_node.append_child("Piece");
     piece_node.append_attribute("Source") = tmp_string.c_str();
@@ -549,7 +558,8 @@ void VTKFile::pvtu_write_function(std::size_t dim, std::size_t rank,
   xml_doc.save_file(fname.c_str(), "  ");
 }
 //----------------------------------------------------------------------------
-void VTKFile::pvtu_write_mesh(const std::string fname) const
+void VTKFile::pvtu_write_mesh(const std::string fname,
+                              const std::size_t num_processes) const
 {
   // Create xml doc
   pugi::xml_document xml_doc;
@@ -563,9 +573,9 @@ void VTKFile::pvtu_write_mesh(const std::string fname) const
   pvtu_write_mesh(grid_node);
 
   // Write vtu file list
-  for(std::size_t i = 0; i < MPI::num_processes(); i++)
+  for (std::size_t i = 0; i < num_processes; i++)
   {
-    const std::string tmp_string = strip_path(vtu_name(i, MPI::num_processes(),
+    const std::string tmp_string = strip_path(vtu_name(i, num_processes,
                                                        counter, ".vtu"));
     pugi::xml_node piece_node = grid_node.append_child("Piece");
     piece_node.append_attribute("Source") = tmp_string.c_str();
@@ -588,17 +598,21 @@ void VTKFile::pvtu_write(const Function& u, const std::string fname) const
   // Get number of components
   const std::size_t dim = u.value_size();
 
+  // Get mesh
+  dolfin_assert(u.function_space()->mesh());
+  const Mesh& mesh = *(u.function_space()->mesh());
+
   // Test for cell-based element type
   std::string data_type = "point";
   std::size_t cell_based_dim = 1;
-  dolfin_assert(u.function_space()->mesh());
   dolfin_assert(u.function_space()->dofmap());
   for (std::size_t i = 0; i < rank; i++)
-    cell_based_dim *= u.function_space()->mesh()->topology().dim();
+    cell_based_dim *= mesh.topology().dim();
   if (u.function_space()->dofmap()->max_cell_dimension() == cell_based_dim)
     data_type = "cell";
 
-  pvtu_write_function(dim, rank, data_type, u.name(), fname);
+  const std::size_t num_processes = MPI::size(mesh.mpi_comm());
+  pvtu_write_function(dim, rank, data_type, u.name(), fname, num_processes);
 }
 //----------------------------------------------------------------------------
 void VTKFile::vtk_header_open(std::size_t num_vertices, std::size_t num_cells,
@@ -724,13 +738,16 @@ void VTKFile::mesh_function_write(T& meshfunction, double time)
   fp.close();
 
   // Parallel-specfic files
-  if (MPI::num_processes() > 1 && MPI::process_number() == 0)
+  const std::size_t num_processes = MPI::size(mesh.mpi_comm());
+  const std::size_t process_number = MPI::rank(mesh.mpi_comm());
+  if (num_processes > 1 && process_number == 0)
   {
     std::string pvtu_filename = vtu_name(0, 0, counter, ".pvtu");
-    pvtu_write_function(1, 0, "cell", meshfunction.name(), pvtu_filename);
+    pvtu_write_function(1, 0, "cell", meshfunction.name(), pvtu_filename,
+                        num_processes);
     pvd_file_write(counter, time, pvtu_filename);
   }
-  else if (MPI::num_processes() == 1)
+  else if (num_processes == 1)
     pvd_file_write(counter, time, vtu_filename);
 
   // Write pvd files
