@@ -16,12 +16,10 @@
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 //
 // Modified by Garth N. Wells, 2012
-//
-// First added:  2012-05-28
-// Last changed: 2013-10-15
 
 #ifdef HAS_HDF5
 
+#include <iomanip>
 #include <ostream>
 #include <sstream>
 #include <vector>
@@ -33,6 +31,7 @@
 
 #include "pugixml.hpp"
 
+#include <dolfin/common/MPI.h>
 #include <dolfin/function/Function.h>
 #include <dolfin/function/FunctionSpace.h>
 #include <dolfin/fem/GenericDofMap.h>
@@ -49,7 +48,8 @@
 using namespace dolfin;
 
 //----------------------------------------------------------------------------
-XDMFFile::XDMFFile(const std::string filename) : GenericFile(filename, "XDMF")
+XDMFFile::XDMFFile(MPI_Comm comm, const std::string filename)
+  : GenericFile(filename, "XDMF"), _mpi_comm(comm)
 {
   // Make name for HDF5 file (used to store data)
   boost::filesystem::path p(filename);
@@ -66,6 +66,10 @@ XDMFFile::XDMFFile(const std::string filename) : GenericFile(filename, "XDMF")
   // Flush datasets to disk at each timestep. Allows inspection of the
   // HDF5 file whilst running, at some performance cost.
   parameters.add("flush_output", false);
+
+  // HDF5 file restart interval. Use 0 to collect all output in one file.
+  parameters.add("multi_file", 0);
+  
 }
 //----------------------------------------------------------------------------
 XDMFFile::~XDMFFile()
@@ -81,13 +85,29 @@ void XDMFFile::operator<< (const Function& u)
 //----------------------------------------------------------------------------
 void XDMFFile::operator<< (const std::pair<const Function*, double> ut)
 {
-  // Prepate HDF5 file
-  if (hdf5_filemode != "w")
-  {
-    // Create HDF5 file (truncate)
-    hdf5_file.reset(new HDF5File(hdf5_filename, "w"));
+  const int mf_interval = parameters["multi_file"];
+
+  // Conditions for starting a new HDF5 file
+  if ( (mf_interval != 0 and counter%mf_interval == 0) or hdf5_filemode != "w" )
+  { 
+    // Make name for HDF5 file (used to store data)
+    boost::filesystem::path p(_filename);
+    p.replace_extension(".h5");
+    hdf5_filename = p.string();
+
+    if (mf_interval != 0)
+    {
+      std::stringstream s;
+      s << std::setw(6) << std::setfill('0') << counter;
+      hdf5_filename += "_" + s.str();
+    }
+    
+    // Create new HDF5 file (truncate),
+    // closing any open file from a previous timestep
+    hdf5_file.reset(new HDF5File(_mpi_comm, hdf5_filename, "w"));
     hdf5_filemode = "w";
   }
+
   dolfin_assert(hdf5_file);
 
   // Access Function, Mesh, dofmap  and time step
@@ -102,10 +122,6 @@ void XDMFFile::operator<< (const std::pair<const Function*, double> ut)
 
   const double time_step = ut.second;
 
-  // FIXME: Can we avoid this?
-  // Update any ghost values
-  u.update();
-
   // Geometric and topological dimension
   const std::size_t gdim = mesh.geometry().dim();
   const std::size_t tdim = mesh.topology().dim();
@@ -239,8 +255,10 @@ void XDMFFile::operator<< (const std::pair<const Function*, double> ut)
   // Write mesh to HDF5 file
   if (parameters["rewrite_function_mesh"] || counter == 0)
   {
-      current_mesh_name = "/Mesh/" + boost::lexical_cast<std::string>(counter);
-      hdf5_file->write(mesh, current_mesh_name);
+    const std::string h5_mesh_name = "/Mesh/" + boost::lexical_cast<std::string>(counter);
+    boost::filesystem::path p(hdf5_filename);
+    current_mesh_name = p.filename().string() + ":" + h5_mesh_name;
+    hdf5_file->write(mesh, h5_mesh_name);
   }
 
   // Remove duplicates for vertex-based data
@@ -248,7 +266,7 @@ void XDMFFile::operator<< (const std::pair<const Function*, double> ut)
   global_size[1] = padded_value_size;
   if (vertex_data)
   {
-    HDF5Utility::reorder_values_by_global_indices(mesh, data_values, 
+    HDF5Utility::reorder_values_by_global_indices(mesh, data_values,
                                                   padded_value_size);
     global_size[0] = mesh.size_global(0);
   }
@@ -261,7 +279,8 @@ void XDMFFile::operator<< (const std::pair<const Function*, double> ut)
   const std::string dataset_name = "/VisualisationVector/"
     + boost::lexical_cast<std::string>(counter);
 
-  hdf5_file->write_data(dataset_name, data_values, global_size);
+  const bool mpi_io = MPI::size(mesh.mpi_comm()) > 1 ? true : false;
+  hdf5_file->write_data(dataset_name, data_values, global_size, mpi_io);
 
   // Flush file. Improves chances of recovering data if
   // interrupted. Also makes file somewhat readable between writes.
@@ -271,7 +290,7 @@ void XDMFFile::operator<< (const std::pair<const Function*, double> ut)
   // Write the XML meta description (see http://www.xdmf.org) on
   // process zero
   const std::size_t num_total_vertices = mesh.size_global(0);
-  if (MPI::process_number() == 0)
+  if (MPI::rank(mesh.mpi_comm()) == 0)
   {
     output_xml(time_step, vertex_data, tdim, num_global_cells, gdim,
                num_total_vertices, value_rank, padded_value_size,
@@ -283,11 +302,16 @@ void XDMFFile::operator<< (const std::pair<const Function*, double> ut)
 }
 //----------------------------------------------------------------------------
 void XDMFFile::operator>> (Mesh& mesh)
+{
+  read(mesh, false);
+}
+//-----------------------------------------------------------------------------
+void XDMFFile::read(Mesh& mesh, bool use_partition_from_file)
 {
   // Prepare HDF5 file
   if (hdf5_filemode != "r")
   {
-    hdf5_file.reset(new HDF5File(hdf5_filename, "r"));
+    hdf5_file.reset(new HDF5File(_mpi_comm, hdf5_filename, "r"));
     hdf5_filemode = "r";
   }
   dolfin_assert(hdf5_file);
@@ -353,7 +377,7 @@ void XDMFFile::operator>> (Mesh& mesh)
   dolfin_assert(geom_bits[4] == "coordinates");
 
   // Try to read the mesh from the associated HDF5 file
-  hdf5_file->read(mesh, "/Mesh/" + geom_bits[3]);
+  hdf5_file->read(mesh, "/Mesh/" + geom_bits[3], use_partition_from_file);
 }
 //----------------------------------------------------------------------------
 void XDMFFile::operator<< (const Mesh& mesh)
@@ -363,7 +387,7 @@ void XDMFFile::operator<< (const Mesh& mesh)
   if (hdf5_filemode != "w")
   {
     // Create HDF5 file (truncate)
-    hdf5_file.reset(new HDF5File(hdf5_filename, "w"));
+    hdf5_file.reset(new HDF5File(mesh.mpi_comm(), hdf5_filename, "w"));
     hdf5_filemode = "w";
   }
 
@@ -387,13 +411,8 @@ void XDMFFile::operator<< (const Mesh& mesh)
   const std::string group_name = "/Mesh/" + name;
   hdf5_file->write(mesh, cell_dim, group_name);
 
-  // FIXME: Names should be returned by HDF5::write_mesh
-  // Mesh data set names
-  const std::string mesh_topology_name = group_name + "/topology";
-  const std::string mesh_coords_name = group_name + "/coordinates";
-
   // Write the XML meta description on process zero
-  if (MPI::process_number() == 0)
+  if (MPI::rank(mesh.mpi_comm()) == 0)
   {
     // Create XML document
     pugi::xml_document xml_doc;
@@ -410,13 +429,18 @@ void XDMFFile::operator<< (const Mesh& mesh)
 
     // Describe topological connectivity
     pugi::xml_node xdmf_topology = xdmf_grid.append_child("Topology");
+    boost::filesystem::path p(hdf5_filename);
+    const std::string topology_reference = p.filename().string() + ":"
+      + group_name + "/topology";
     xml_mesh_topology(xdmf_topology, cell_dim, num_global_cells,
-                      mesh_topology_name);
+                      topology_reference);
 
     // Describe geometric coordinates
     pugi::xml_node xdmf_geometry = xdmf_grid.append_child("Geometry");
+    const std::string geometry_reference = p.filename().string() + ":"
+      + group_name + "/coordinates";
     xml_mesh_geometry(xdmf_geometry, num_total_vertices, gdim,
-                      mesh_coords_name);
+                      geometry_reference);
 
     xml_doc.save_file(_filename.c_str(), "  ");
   }
@@ -442,13 +466,135 @@ void XDMFFile::operator<< (const MeshFunction<double>& meshfunction)
   write_mesh_function(meshfunction);
 }
 //----------------------------------------------------------------------------
-template<typename T>
-void XDMFFile::write_mesh_function(const MeshFunction<T>& meshfunction)
+void XDMFFile::write(const std::vector<Point>& points)
 {
+  // Intialise HDF5 file
   if (hdf5_filemode != "w")
   {
     // Create HDF5 file (truncate)
-    hdf5_file.reset(new HDF5File(hdf5_filename, "w"));
+    hdf5_file.reset(new HDF5File(_mpi_comm, hdf5_filename, "w"));
+    hdf5_filemode = "w";
+  }
+
+  // Get number of points (global)
+  const std::size_t num_global_points = MPI::sum(_mpi_comm, points.size());
+
+  // Write HDF5 file
+  const std::string group_name = "/Points";
+  hdf5_file->write(points, group_name);
+
+  // The XML created below will obliterate any existing XDMF file
+  write_point_xml(group_name, num_global_points, 0);
+}
+//----------------------------------------------------------------------------
+void XDMFFile::write(const std::vector<Point>& points,
+                     const std::vector<double>& values)
+{
+  // Write clouds of points to XDMF/HDF5 with values
+
+  dolfin_assert(points.size() == values.size());
+
+  // Intialise HDF5 file
+  if (hdf5_filemode != "w")
+  {
+    // Create HDF5 file (truncate)
+    hdf5_file.reset(new HDF5File(_mpi_comm, hdf5_filename, "w"));
+    hdf5_filemode = "w";
+  }
+
+  // Get number of points (global)
+  const std::size_t num_global_points = MPI::sum(_mpi_comm, points.size());
+
+  // Write HDF5 file
+  const std::string group_name = "/Points";
+  hdf5_file->write(points, group_name);
+
+  const std::string values_name = group_name + "/values";
+  hdf5_file->write(values, values_name);
+
+  // The XML created will obliterate any existing XDMF file
+  write_point_xml(group_name, num_global_points, 1);
+}
+//----------------------------------------------------------------------------
+void XDMFFile::write_point_xml(const std::string group_name,
+                               const std::size_t num_global_points,
+                               const unsigned int value_size)
+{
+  // Write the XML meta description on process zero
+  if (MPI::rank(_mpi_comm) == 0)
+  {
+
+    // Create XML document
+    pugi::xml_document xml_doc;
+
+    // XML headers
+    xml_doc.append_child(pugi::node_doctype).set_value("Xdmf SYSTEM \"Xdmf.dtd\" []");
+    pugi::xml_node xdmf = xml_doc.append_child("Xdmf");
+    xdmf.append_attribute("Version") = "2.0";
+    xdmf.append_attribute("xmlns:xi") = "http://www.w3.org/2001/XInclude";
+    pugi::xml_node xdmf_domain = xdmf.append_child("Domain");
+    pugi::xml_node xdmf_grid = xdmf_domain.append_child("Grid");
+    xdmf_grid.append_attribute("Name") = "Point cloud";
+    xdmf_grid.append_attribute("GridType") = "Uniform";
+
+
+    // Describe topological connectivity
+    pugi::xml_node xdmf_topology = xdmf_grid.append_child("Topology");
+    xdmf_topology.append_attribute("NumberOfElements")
+      = (unsigned int) num_global_points;
+    xdmf_topology.append_attribute("TopologyType") = "PolyVertex";
+    xdmf_topology.append_attribute("NodesPerElement") = "1";
+
+    // Describe geometric coordinates
+    pugi::xml_node xdmf_geometry = xdmf_grid.append_child("Geometry");
+    const std::string geometry_reference = current_mesh_name + "/coordinates";
+    xml_mesh_geometry(xdmf_geometry, num_global_points, 3,
+		      geometry_reference);
+
+    if(value_size != 0)
+    {
+      dolfin_assert(value_size == 1 || value_size == 3);
+
+      // Grid/Attribute (value data)
+      pugi::xml_node xdmf_values = xdmf_grid.append_child("Attribute");
+      xdmf_values.append_attribute("Name") = "point_values";
+
+      if(value_size == 1)
+        xdmf_values.append_attribute("AttributeType") = "Scalar";
+      else
+        xdmf_values.append_attribute("AttributeType") = "Vector";
+
+      xdmf_values.append_attribute("Center") = "Node";
+
+      pugi::xml_node xdmf_data = xdmf_values.append_child("DataItem");
+      xdmf_data.append_attribute("Format") = "HDF";
+
+      std::string s
+        = boost::lexical_cast<std::string>(num_global_points) + " "
+        + boost::lexical_cast<std::string>(value_size);
+
+      xdmf_data.append_attribute("Dimensions") = s.c_str();
+
+      boost::filesystem::path p(hdf5_filename);
+      s = p.filename().string() + ":" + group_name + "/values";
+      xdmf_data.append_child(pugi::node_pcdata).set_value(s.c_str());
+    }
+
+    xml_doc.save_file(_filename.c_str(), "  ");
+  }
+}
+//----------------------------------------------------------------------------
+template<typename T>
+void XDMFFile::write_mesh_function(const MeshFunction<T>& meshfunction)
+{
+  // Get mesh
+  dolfin_assert(meshfunction.mesh());
+  const Mesh& mesh = *meshfunction.mesh();
+
+  if (hdf5_filemode != "w")
+  {
+    // Create HDF5 file (truncate)
+    hdf5_file.reset(new HDF5File(mesh.mpi_comm(), hdf5_filename, "w"));
     hdf5_filemode = "w";
   }
 
@@ -459,22 +605,21 @@ void XDMFFile::write_mesh_function(const MeshFunction<T>& meshfunction)
                  "No values in MeshFunction");
   }
 
-  // Get mesh
-  const Mesh& mesh = *meshfunction.mesh();
-
   const std::size_t cell_dim = meshfunction.dim();
   dolfin_assert(cell_dim <= mesh.topology().dim());
 
   // Use HDF5 function to output MeshFunction
-  current_mesh_name = "/Mesh/" + boost::lexical_cast<std::string>(counter);
-  hdf5_file->write(meshfunction, current_mesh_name);
+  const std::string h5_mesh_name = "/Mesh/" + boost::lexical_cast<std::string>(counter);
+  boost::filesystem::path p(hdf5_filename);
+  current_mesh_name = p.filename().string() + ":" + h5_mesh_name;
+  hdf5_file->write(meshfunction, h5_mesh_name);
 
   // Saved MeshFunction values are in the /Mesh group
   const std::string dataset_name =  current_mesh_name + "/values";
 
   // Write the XML meta description (see http://www.xdmf.org) on
   // process zero
-  if (MPI::process_number() == 0)
+  if (MPI::rank(mesh.mpi_comm()) == 0)
   {
     output_xml((double)counter, false,
                cell_dim, mesh.size_global(cell_dim),
@@ -517,7 +662,7 @@ void XDMFFile::read_mesh_function(MeshFunction<T>& meshfunction)
 {
   if (hdf5_filemode != "r")
   {
-    hdf5_file.reset(new HDF5File(hdf5_filename, "r"));
+    hdf5_file.reset(new HDF5File(_mpi_comm, hdf5_filename, "r"));
     hdf5_filemode = "r";
   }
 
@@ -594,7 +739,7 @@ void XDMFFile::read_mesh_function(MeshFunction<T>& meshfunction)
 void XDMFFile::xml_mesh_topology(pugi::xml_node &xdmf_topology,
                                  const std::size_t cell_dim,
                                  const std::size_t num_global_cells,
-                                 const std::string topology_dataset_name) const
+                                 const std::string topology_reference) const
 {
   xdmf_topology.append_attribute("NumberOfElements")
     = (unsigned int) num_global_cells;
@@ -623,19 +768,13 @@ void XDMFFile::xml_mesh_topology(pugi::xml_node &xdmf_topology,
     + " " + boost::lexical_cast<std::string>(cell_dim + 1);
   xdmf_topology_data.append_attribute("Dimensions") = cell_dims.c_str();
 
-  // For XDMF file need to remove path from filename so that xdmf
-  // filenames such as "results/data.xdmf" correctly index h5 files in
-  // the same directory
-  boost::filesystem::path p(hdf5_filename);
-  std::string topology_reference = p.filename().string() + ":"
-    + topology_dataset_name;
   xdmf_topology_data.append_child(pugi::node_pcdata).set_value(topology_reference.c_str());
 }
 //----------------------------------------------------------------------------
 void XDMFFile::xml_mesh_geometry(pugi::xml_node& xdmf_geometry,
                                  const std::size_t num_total_vertices,
                                  const std::size_t gdim,
-                                 const std::string geometry_dataset_name) const
+                                 const std::string geometry_reference) const
 {
   dolfin_assert(0 < gdim && gdim <= 3);
   std::string geometry_type;
@@ -688,9 +827,6 @@ void XDMFFile::xml_mesh_geometry(pugi::xml_node& xdmf_geometry,
     xdmf_geom_2.append_child(pugi::node_pcdata).set_value(dummy_zeros.c_str());
   }
 
-  boost::filesystem::path p(hdf5_filename);
-  const std::string geometry_reference
-    = p.filename().string() + ":" + geometry_dataset_name;
   xdmf_geom_data.append_child(pugi::node_pcdata).set_value(geometry_reference.c_str());
 }
 //----------------------------------------------------------------------------
@@ -768,18 +904,25 @@ void XDMFFile::output_xml(const double time_step, const bool vertex_data,
 
   // Grid/Topology
   pugi::xml_node xdmf_topology = xdmf_grid.append_child("Topology");
+
+  // For XDMF file need to remove path from filename so that xdmf
+  // filenames such as "results/data.xdmf" correctly index h5 files in
+  // the same directory
+
+  std::string topology_reference = current_mesh_name + "/topology";
   xml_mesh_topology(xdmf_topology, cell_dim, num_global_cells,
-                    current_mesh_name + "/topology");
+                    topology_reference);
 
   // Grid/Geometry
   pugi::xml_node xdmf_geometry = xdmf_grid.append_child("Geometry");
+  std::string geometry_reference = current_mesh_name + "/coordinates";
   xml_mesh_geometry(xdmf_geometry, num_total_vertices, gdim,
-                    current_mesh_name + "/coordinates");
+                    geometry_reference);
 
   // Grid/Attribute (Function value data)
   pugi::xml_node xdmf_values = xdmf_grid.append_child("Attribute");
   xdmf_values.append_attribute("Name") = name.c_str();
-  
+
   if (value_rank == 0)
     xdmf_values.append_attribute("AttributeType") = "Scalar";
   else if (value_rank == 1)
@@ -791,11 +934,11 @@ void XDMFFile::output_xml(const double time_step, const bool vertex_data,
     xdmf_values.append_attribute("Center") = "Node";
   else
     xdmf_values.append_attribute("Center") = "Cell";
-  
+
   pugi::xml_node xdmf_data = xdmf_values.append_child("DataItem");
   xdmf_data.append_attribute("Format") = "HDF";
-  
-  const std::size_t num_total_entities 
+
+  const std::size_t num_total_entities
     = vertex_data ? num_total_vertices : num_global_cells;
 
   s = boost::lexical_cast<std::string>(num_total_entities) + " "
@@ -803,10 +946,10 @@ void XDMFFile::output_xml(const double time_step, const bool vertex_data,
 
   xdmf_data.append_attribute("Dimensions") = s.c_str();
 
-  boost::filesystem::path p(hdf5_filename);
+  boost::filesystem::path p(hdf5_filename);  
   s = p.filename().string() + ":" + dataset_name;
   xdmf_data.append_child(pugi::node_pcdata).set_value(s.c_str());
-  
+
   // Write XML file
   xml_doc.save_file(_filename.c_str(), "  ");
 }
