@@ -16,21 +16,19 @@
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 //
 // Modified by Garth N. Wells, 2012
-//
-// First added:  2012-05-22
-// Last changed: 2013-05-09
 
 #ifndef __DOLFIN_XDMFFILE_H
 #define __DOLFIN_XDMFFILE_H
 
 #ifdef HAS_HDF5
 
+#include <memory>
 #include <string>
 #include <utility>
-#include <boost/scoped_ptr.hpp>
-
-#include "dolfin/common/Variable.h"
+#include <vector>
 
+#include <dolfin/common/MPI.h>
+#include <dolfin/common/Variable.h>
 #include "GenericFile.h"
 
 namespace pugi
@@ -46,6 +44,7 @@ namespace dolfin
   class HDF5File;
   class Mesh;
   template<typename T> class MeshFunction;
+  class Point;
 
   /// This class supports the output of meshes and functions in XDMF
   /// (http://www.xdmf.org) format. It creates an XML file that describes
@@ -60,7 +59,7 @@ namespace dolfin
   public:
 
     /// Constructor
-    explicit XDMFFile(const std::string filename);
+    XDMFFile(MPI_Comm comm, const std::string filename);
 
     /// Destructor
     ~XDMFFile();
@@ -69,6 +68,11 @@ namespace dolfin
     /// file to store the mesh, and a related XDMF file with metadata.
     void operator<< (const Mesh& mesh);
 
+    /// Read in a mesh from the associated HDF5 file,
+    /// optionally using stored partitioning, if possible
+    /// when the same number of processes are being used.
+    void read(Mesh& mesh, bool use_partition_from_file);
+
     /// Read in a mesh from the associated HDF5 file
     void operator>> (Mesh& mesh);
 
@@ -84,6 +88,15 @@ namespace dolfin
     void operator<< (const MeshFunction<std::size_t>& meshfunction);
     void operator<< (const MeshFunction<double>& meshfunction);
 
+    /// Save a cloud of points to file
+    void write(const std::vector<Point>& points);
+
+    /// Save a cloud of points, with scalar values
+    void write(const std::vector<Point>& points,
+               const std::vector<double>& values);
+
+    using GenericFile::write;
+
     /// Read first MeshFunction from file
     void operator>> (MeshFunction<bool>& meshfunction);
     void operator>> (MeshFunction<int>& meshfunction);
@@ -92,8 +105,11 @@ namespace dolfin
 
   private:
 
+    // MPI communicator
+    MPI_Comm _mpi_comm;
+
     // HDF5 data file
-    boost::scoped_ptr<HDF5File> hdf5_file;
+    std::unique_ptr<HDF5File> hdf5_file;
 
     // HDF5 filename
     std::string hdf5_filename;
@@ -109,6 +125,12 @@ namespace dolfin
     template<typename T>
       void read_mesh_function(MeshFunction<T>& meshfunction);
 
+    // Write XML description of point clouds, with value_size = 0, 1 or 3
+    // (for either no point data, scalar, or vector)
+    void write_point_xml(const std::string dataset_name,
+                         const std::size_t num_global_points,
+                         const unsigned int value_size);
+
     // Write XML description for Function and MeshFunction output
     // updating time-series if need be
     void output_xml(const double time_step, const bool vertex_data,
