@@ -28,7 +28,7 @@ using namespace dolfin;
 //-----------------------------------------------------------------------------
 CoordinateMatrix::CoordinateMatrix(const GenericMatrix& A, bool symmetric,
                                    bool base_one)
-  : _symmetric(symmetric), _base_one(base_one)
+  : _mpi_comm(A.mpi_comm()), _symmetric(symmetric), _base_one(base_one)
 {
   _size[0] = A.size(0);
   _size[1] = A.size(1);
@@ -49,11 +49,11 @@ CoordinateMatrix::CoordinateMatrix(const GenericMatrix& A, bool symmetric,
       _cols.insert(_cols.end(), columns.begin(), columns.end());
       _vals.insert(_vals.end(), values.begin(), values.end());
     }
-    assert(_rows.size() == _cols.size());
+    dolfin_assert(_rows.size() == _cols.size());
   }
   else
   {
-    assert(_size[0] == _size[1]);
+    dolfin_assert(_size[0] == _size[1]);
     for (std::size_t i = local_row_range.first; i < local_row_range.second; ++i)
     {
       // Get column and value data for row
@@ -71,7 +71,7 @@ CoordinateMatrix::CoordinateMatrix(const GenericMatrix& A, bool symmetric,
         }
       }
     }
-    assert(_rows.size() == _cols.size());
+    dolfin_assert(_rows.size() == _cols.size());
   }
 
   // Add 1 for Fortran-style indices
@@ -109,6 +109,6 @@ double CoordinateMatrix::norm(std::string norm_type) const
         _norm += 2.0*_vals[i]*_vals[i];
     }
   }
-  return std::sqrt(dolfin::MPI::sum(_norm));
+  return std::sqrt(dolfin::MPI::sum(_mpi_comm, _norm));
 }
 //-----------------------------------------------------------------------------
