@@ -83,7 +83,7 @@ std::string EpetraLUSolver::choose_method(std::string method) const
   else if (method == "mumps")
     method = "Amesos_mumps";
   else if (method == "klu")
-    method = "Amesos_klu";
+    method = "Amesos_Klu";
   else
   {
     dolfin_error("EpetraLUSolver.cpp",
@@ -128,7 +128,7 @@ EpetraLUSolver::EpetraLUSolver(std::string method)
   }
 }
 //-----------------------------------------------------------------------------
-EpetraLUSolver::EpetraLUSolver(boost::shared_ptr<const GenericLinearOperator> A,
+EpetraLUSolver::EpetraLUSolver(std::shared_ptr<const GenericLinearOperator> A,
                                std::string method)
   : symbolic_factorized(false),
     numeric_factorized(false),
@@ -138,8 +138,8 @@ EpetraLUSolver::EpetraLUSolver(boost::shared_ptr<const GenericLinearOperator> A,
   parameters = default_parameters();
 
   // Set operator
-  _A = as_type<const EpetraMatrix>(require_matrix(A));
-  dolfin_assert(_A);
+  _matA = as_type<const EpetraMatrix>(require_matrix(A));
+  dolfin_assert(_matA);
 
   // Choose method
   _method = choose_method(method);
@@ -163,13 +163,14 @@ EpetraLUSolver::~EpetraLUSolver()
   // Do nothing
 }
 //-----------------------------------------------------------------------------
-void EpetraLUSolver::set_operator(const boost::shared_ptr<const GenericLinearOperator> A)
+void
+EpetraLUSolver::set_operator(std::shared_ptr<const GenericLinearOperator> A)
 {
   dolfin_assert(linear_problem);
 
-  _A = as_type<const EpetraMatrix>(require_matrix(A));
-  dolfin_assert(_A);
-  linear_problem->SetOperator(_A->mat().get());
+  _matA = as_type<const EpetraMatrix>(require_matrix(A));
+  dolfin_assert(_matA);
+  linear_problem->SetOperator(_matA->mat().get());
 
   symbolic_factorized = false;
   numeric_factorized  = false;
@@ -177,13 +178,13 @@ void EpetraLUSolver::set_operator(const boost::shared_ptr<const GenericLinearOpe
 //-----------------------------------------------------------------------------
 const GenericLinearOperator& EpetraLUSolver::get_operator() const
 {
-  if (!_A)
+  if (!_matA)
   {
     dolfin_error("EpetraLUSolver.cpp",
                  "access operator for Epetra LU solver",
                  "Operator has not been set");
   }
-  return *_A;
+  return *_matA;
 }
 //-----------------------------------------------------------------------------
 std::size_t EpetraLUSolver::solve(GenericVector& x, const GenericVector& b)
@@ -194,17 +195,17 @@ std::size_t EpetraLUSolver::solve(GenericVector& x, const GenericVector& b)
   dolfin_assert(solver);
 
   // Write a message
-  if (parameters["report"] && dolfin::MPI::process_number() == 0)
+  if (parameters["report"] && dolfin::MPI::rank(MPI_COMM_WORLD) == 0)
   {
     if (solver->UseTranspose())
     {
       info("Solving linear system transposed of size %d x %d using Epetra LU solver (%s).",
-           _A->size(0), _A->size(1), _method.c_str());
+           _matA->size(0), _matA->size(1), _method.c_str());
     }
     else
     {
       info("Solving linear system of size %d x %d using Epetra LU solver (%s).",
-           _A->size(0), _A->size(1), _method.c_str());
+           _matA->size(0), _matA->size(1), _method.c_str());
     }
   }
 
@@ -221,7 +222,6 @@ std::size_t EpetraLUSolver::solve(GenericVector& x, const GenericVector& b)
                  "Operator has not been set");
   }
 
-  const std::size_t M = A->NumGlobalRows64();
   const std::size_t N = A->NumGlobalCols64();
   if (N != b.size())
   {
@@ -231,9 +231,9 @@ std::size_t EpetraLUSolver::solve(GenericVector& x, const GenericVector& b)
   }
 
   // Initialize solution vector
-  if (x.size() != M)
+  if (x.empty())
   {
-    _A->resize(x, 1);
+    _matA->init_vector(x, 1);
     x.zero();
   }
 
@@ -272,6 +272,9 @@ std::size_t EpetraLUSolver::solve(GenericVector& x, const GenericVector& b)
   // Solve
   AMESOS_CHK_ERR(solver->Solve());
 
+  // Update ghost values
+  _x.update_ghost_values();
+
   return 1;
 }
 //-----------------------------------------------------------------------------
@@ -287,7 +290,7 @@ std::size_t EpetraLUSolver::solve(const GenericLinearOperator& A,
 std::size_t EpetraLUSolver::solve(const EpetraMatrix& A, EpetraVector& x,
                                   const EpetraVector& b)
 {
-  boost::shared_ptr<const EpetraMatrix> Atmp(&A, NoDeleter());
+  std::shared_ptr<const EpetraMatrix> Atmp(&A, NoDeleter());
   set_operator(Atmp);
   return solve(x, b);
 }
