@@ -15,19 +15,20 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 //
-// Modified by Anders Logg, 2008.
-// Modified by Garth N. Wells, 2008, 2009.
+// Modified by Anders Logg 2008-2012
+// Modified by Garth N. Wells 2008, 2009
 //
 // First added:  2008-04-21
-// Last changed: 2009-09-08
+// Last changed: 2012-08-20
 
 #ifndef __EPETRA_MATRIX_H
 #define __EPETRA_MATRIX_H
 
 #ifdef HAS_TRILINOS
 
-#include <boost/shared_ptr.hpp>
+#include <memory>
 #include "GenericMatrix.h"
+#include <Teuchos_RCP.hpp>
 
 class Epetra_FECrsMatrix;
 class Epetra_CrsGraph;
@@ -36,8 +37,7 @@ namespace dolfin
 {
 
   /// Forward declarations
-  class EpetraSparsityPattern;
-  class GenericSparsityPattern;
+  class TensorLayout;
 
   /// This class provides a simple matrix class based on Epetra.
   /// It is a simple wrapper for an Epetra matrix object (Epetra_FECrsMatrix)
@@ -58,7 +58,10 @@ namespace dolfin
     EpetraMatrix(const EpetraMatrix& A);
 
     /// Create matrix from given Epetra_FECrsMatrix pointer
-    explicit EpetraMatrix(boost::shared_ptr<Epetra_FECrsMatrix> A);
+    explicit EpetraMatrix(Teuchos::RCP<Epetra_FECrsMatrix> A);
+
+    /// Create matrix from given Epetra_FECrsMatrix pointer
+    explicit EpetraMatrix(std::shared_ptr<Epetra_FECrsMatrix> A);
 
     /// Create matrix from given Epetra_CrsGraph
     explicit EpetraMatrix(const Epetra_CrsGraph& graph);
@@ -68,20 +71,18 @@ namespace dolfin
 
     //--- Implementation of the GenericTensor interface ---
 
-    /// Return true if matrix is distributed
-    virtual bool distributed() const;
+    /// Initialize zero tensor using tensor layout
+    virtual void init(const TensorLayout& tensor_layout);
 
-    /// Initialize zero tensor using sparsity pattern
-    virtual void init(const GenericSparsityPattern& sparsity_pattern);
-
-    /// Return copy of tensor
-    virtual EpetraMatrix* copy() const;
+    /// Return true if empty
+    virtual bool empty() const;
 
     /// Return size of given dimension
-    virtual uint size(uint dim) const;
+    virtual std::size_t size(std::size_t dim) const;
 
     /// Return local ownership range
-    virtual std::pair<uint, uint> local_range(uint dim) const;
+    virtual std::pair<std::size_t, std::size_t>
+      local_range(std::size_t dim) const;
 
     /// Set all entries to zero and keep any sparse structure
     virtual void zero();
@@ -93,42 +94,61 @@ namespace dolfin
     ///   insert - corresponding to Epetra GlobalAssemble(Insert)
     virtual void apply(std::string mode);
 
+    /// Return MPI communicator
+    virtual MPI_Comm mpi_comm() const;
+
     /// Return informal string representation (pretty-print)
     virtual std::string str(bool verbose) const;
 
     //--- Implementation of the GenericMatrix interface ---
 
-    /// Resize vector y such that is it compatible with matrix for
-    /// multuplication Ax = b (dim = 0 -> b, dim = 1 -> x) In parallel
-    /// case, size and layout are important.
-    virtual void resize(GenericVector& y, uint dim) const;
+    /// Return copy of matrix
+    virtual std::shared_ptr<GenericMatrix> copy() const;
+
+    /// Intialize vector z to be compatible with the matrix-vector
+    /// product y = Ax. In the parallel case, both size and layout are
+    /// important.
+    ///
+    /// *Arguments*
+    ///     dim (std::size_t)
+    ///         The dimension (axis): dim = 0 --> z = y, dim = 1 --> z = x
+    virtual void init_vector(GenericVector& z, std::size_t dim) const;
 
     /// Get block of values
-    virtual void get(double* block, uint m, const uint* rows, uint n, const uint* cols) const;
+    virtual void get(double* block, std::size_t m, const dolfin::la_index* rows,
+                     std::size_t n, const dolfin::la_index* cols) const;
 
     /// Set block of values
-    virtual void set(const double* block, uint m, const uint* rows, uint n, const uint* cols);
+    virtual void set(const double* block, std::size_t m,
+                     const dolfin::la_index* rows, std::size_t n,
+                     const dolfin::la_index* cols);
 
     /// Add block of values
-    virtual void add(const double* block, uint m, const uint* rows, uint n, const uint* cols);
+    virtual void add(const double* block, std::size_t m,
+                     const dolfin::la_index* rows, std::size_t n,
+                     const dolfin::la_index* cols);
 
     /// Add multiple of given matrix (AXPY operation)
-    virtual void axpy(double a, const GenericMatrix& A, bool same_nonzero_pattern);
+    virtual void axpy(double a, const GenericMatrix& A,
+                      bool same_nonzero_pattern);
 
     /// Return norm of matrix
     virtual double norm(std::string norm_type) const;
 
     /// Get non-zero values of given row
-    virtual void getrow(uint row, std::vector<uint>& columns, std::vector<double>& values) const;
+    virtual void getrow(std::size_t row, std::vector<std::size_t>& columns,
+                        std::vector<double>& values) const;
 
     /// Set values for given row
-    virtual void setrow(uint row, const std::vector<uint>& columns, const std::vector<double>& values);
+    virtual void setrow(std::size_t row,
+                        const std::vector<std::size_t>& columns,
+                        const std::vector<double>& values);
 
     /// Set given rows to zero
-    virtual void zero(uint m, const uint* rows);
+    virtual void zero(std::size_t m, const dolfin::la_index* rows);
 
     /// Set given rows to identity matrix
-    virtual void ident(uint m, const uint* rows);
+    virtual void ident(std::size_t m, const dolfin::la_index* rows);
 
     // Matrix-vector product, y = Ax
     virtual void mult(const GenericVector& x, GenericVector& y) const;
@@ -136,6 +156,9 @@ namespace dolfin
     // Matrix-vector product, y = A^T x
     virtual void transpmult(const GenericVector& x, GenericVector& y) const;
 
+    /// Set diagonal of a matrix
+    virtual void set_diagonal(const GenericVector& x);
+
     /// Multiply matrix by given number
     virtual const EpetraMatrix& operator*= (double a);
 
@@ -148,15 +171,12 @@ namespace dolfin
     //--- Special functions ---
 
     /// Return linear algebra backend factory
-    virtual LinearAlgebraFactory& factory() const;
+    virtual GenericLinearAlgebraFactory& factory() const;
 
     //--- Special Epetra functions ---
 
-    /// Initialize zero tensor using sparsity pattern
-    virtual void init(const EpetraSparsityPattern& sparsity_pattern);
-
     /// Return Epetra_FECrsMatrix pointer
-    boost::shared_ptr<Epetra_FECrsMatrix> mat() const;
+    std::shared_ptr<Epetra_FECrsMatrix> mat() const;
 
     /// Assignment operator
     const EpetraMatrix& operator= (const EpetraMatrix& x);
@@ -164,8 +184,11 @@ namespace dolfin
   private:
 
     // Epetra_FECrsMatrix pointer
-    boost::shared_ptr<Epetra_FECrsMatrix> A;
+    std::shared_ptr<Epetra_FECrsMatrix> _matA;
 
+    // Epetra_FECrsMatrix pointer, used when initialized with a
+    // Teuchos::RCP shared_ptr
+    Teuchos::RCP<Epetra_FECrsMatrix> ref_keeper;
   };
 
 }
