@@ -94,7 +94,7 @@ KrylovSolver::KrylovSolver(std::string method, std::string preconditioner)
   init(method, preconditioner);
 }
 //-----------------------------------------------------------------------------
-KrylovSolver::KrylovSolver(boost::shared_ptr<const GenericLinearOperator> A,
+KrylovSolver::KrylovSolver(std::shared_ptr<const GenericLinearOperator> A,
                            std::string method, std::string preconditioner)
 {
   // Initialize solver
@@ -109,15 +109,17 @@ KrylovSolver::~KrylovSolver()
   // Do nothing
 }
 //-----------------------------------------------------------------------------
-void KrylovSolver::set_operator(const boost::shared_ptr<const GenericLinearOperator> A)
+void
+KrylovSolver::set_operator(std::shared_ptr<const GenericLinearOperator> A)
 {
   dolfin_assert(solver);
   solver->parameters.update(parameters);
   solver->set_operator(A);
 }
 //-----------------------------------------------------------------------------
-void KrylovSolver::set_operators(const boost::shared_ptr<const GenericLinearOperator> A,
-                                 const boost::shared_ptr<const GenericLinearOperator> P)
+void
+KrylovSolver::set_operators(std::shared_ptr<const GenericLinearOperator> A,
+                            std::shared_ptr<const GenericLinearOperator> P)
 {
   dolfin_assert(solver);
   solver->parameters.update(parameters);
