@@ -21,13 +21,13 @@
 // Modified by Kent-Andre Mardal 2008
 //
 // First added:  2007-07-03
-// Last changed: 2013-11-25
+// Last changed: 2014-05-27
 
 #ifndef __LU_SOLVER_H
 #define __LU_SOLVER_H
 
 #include <string>
-#include <boost/shared_ptr.hpp>
+#include <memory>
 #include "GenericLUSolver.h"
 
 namespace dolfin
@@ -47,14 +47,14 @@ namespace dolfin
     LUSolver(std::string method= "default");
 
     /// Constructor
-    LUSolver(boost::shared_ptr<const GenericLinearOperator> A,
+    LUSolver(std::shared_ptr<const GenericLinearOperator> A,
              std::string method="default");
 
     /// Destructor
     ~LUSolver();
 
     /// Set operator (matrix)
-    void set_operator(const boost::shared_ptr<const GenericLinearOperator> A);
+    void set_operator(std::shared_ptr<const GenericLinearOperator> A);
 
     /// Solve linear system Ax = b
     std::size_t solve(GenericVector& x, const GenericVector& b);
@@ -89,13 +89,21 @@ namespace dolfin
       solver->parameters.update(parameters);
     }
 
+    // FIXME: This should not be needed. Need to cleanup linear solver
+    // name jungle: default, lu, iterative, direct, krylov, etc
+    /// Return parameter type: "krylov_solver" or "lu_solver"
+    std::string parameter_type() const
+    {
+      return "lu_solver";
+    }
+
   private:
 
     // Initialize solver
     void init(std::string method);
 
     // Solver
-    boost::shared_ptr<GenericLinearSolver> solver;
+    std::shared_ptr<GenericLinearSolver> solver;
 
   };
 }
