@@ -27,6 +27,28 @@
 
 using namespace dolfin;
 
+//-----------------------------------------------------------------------------
+PETScBaseMatrix::PETScBaseMatrix(Mat A) : _matA(A)
+{
+  // Increase reference count
+  if (_matA)
+    PetscObjectReference((PetscObject)_matA);
+}
+//-----------------------------------------------------------------------------
+PETScBaseMatrix::~PETScBaseMatrix()
+{
+  // Decrease reference count (PETSc will destroy object once
+  // reference counts reached zero)
+  if (_matA)
+    MatDestroy(&_matA);
+}
+//-----------------------------------------------------------------------------
+PETScBaseMatrix::PETScBaseMatrix(const PETScBaseMatrix& A)
+{
+  dolfin_error("PETScBaseMatrix.cpp",
+               "copy constructor",
+               "PETScBaseMatrix does not provide a copy constructor");
+}
 //-----------------------------------------------------------------------------
 std::size_t PETScBaseMatrix::size(std::size_t dim) const
 {
@@ -37,10 +59,10 @@ std::size_t PETScBaseMatrix::size(std::size_t dim) const
                  "Illegal axis (%d), must be 0 or 1", dim);
   }
 
-  if (_A)
+  if (_matA)
   {
     PetscInt m(0), n(0);
-    PetscErrorCode ierr = MatGetSize(*_A, &m, &n);
+    PetscErrorCode ierr = MatGetSize(_matA, &m, &n);
     if (ierr != 0) petsc_error(ierr, __FILE__, "MetGetSize");
     if (dim == 0)
       return m;
@@ -62,10 +84,10 @@ PETScBaseMatrix::local_range(std::size_t dim) const
                  "Only local row range is available for PETSc matrices");
   }
 
-  if (_A)
+  if (_matA)
   {
     PetscInt m(0), n(0);
-    PetscErrorCode ierr = MatGetOwnershipRange(*_A, &m, &n);
+    PetscErrorCode ierr = MatGetOwnershipRange(_matA, &m, &n);
     if (ierr != 0) petsc_error(ierr, __FILE__, "MatGetOwnershipRange");
     return std::make_pair(m, n);
   }
@@ -73,34 +95,31 @@ PETScBaseMatrix::local_range(std::size_t dim) const
     return std::make_pair(0, 0);
 }
 //-----------------------------------------------------------------------------
-void PETScBaseMatrix::resize(GenericVector& z, std::size_t dim) const
+void PETScBaseMatrix::init_vector(GenericVector& z, std::size_t dim) const
 {
-  dolfin_assert(_A);
+  dolfin_assert(_matA);
 
   PetscErrorCode ierr;
 
   // Downcast vector
   PETScVector& _z = as_type<PETScVector>(z);
 
-  // Clear data
-  _z.reset();
-
   // Create new PETSc vector
-  boost::shared_ptr<Vec> x(new Vec(0), PETScVectorDeleter());
+  Vec x = NULL;
   if (dim == 0)
   {
-    ierr = MatGetVecs(*_A, PETSC_NULL, x.get());
+    ierr = MatGetVecs(_matA, PETSC_NULL, &x);
     if (ierr != 0) petsc_error(ierr, __FILE__, "MatGetVecs");
   }
   else if (dim == 1)
   {
-    ierr = MatGetVecs(*_A, x.get(), PETSC_NULL);
+    ierr = MatGetVecs(_matA, &x, PETSC_NULL);
     if (ierr != 0) petsc_error(ierr, __FILE__, "MatGetVecs");
   }
   else
   {
     dolfin_error("PETScBaseMatrix.cpp",
-                 "resize PETSc vector to match PETSc matrix",
+                 "intialize PETSc vector to match PETSc matrix",
                  "Dimension must be 0 or 1, not %d", dim);
   }
 
