@@ -23,7 +23,7 @@
 // Modified by Jan Blechta 2013
 //
 // First added:  2004
-// Last changed: 2012-03-15
+// Last changed: 2013-09-24
 
 #ifdef HAS_PETSC
 
@@ -51,7 +51,8 @@ const std::map<std::string, NormType> PETScMatrix::norm_types
                               ("frobenius", NORM_FROBENIUS);
 
 //-----------------------------------------------------------------------------
-PETScMatrix::PETScMatrix(bool use_gpu) : _use_gpu(use_gpu)
+PETScMatrix::PETScMatrix(bool use_gpu) : PETScBaseMatrix(NULL),
+                                         _use_gpu(use_gpu)
 {
 #ifndef HAS_PETSC_CUSP
   if (use_gpu)
@@ -65,71 +66,60 @@ PETScMatrix::PETScMatrix(bool use_gpu) : _use_gpu(use_gpu)
   // Do nothing else
 }
 //-----------------------------------------------------------------------------
-PETScMatrix::PETScMatrix(boost::shared_ptr<Mat> A, bool use_gpu) :
-  PETScBaseMatrix(A), _use_gpu(use_gpu)
+PETScMatrix::PETScMatrix(Mat A) : PETScBaseMatrix(A), _use_gpu(false)
 {
-#ifndef HAS_PETSC_CUSP
-  if (use_gpu)
-  {
-    dolfin_error("PETScMatrix.cpp",
-                 "create GPU matrix",
-                 "PETSc not compiled with Cusp support");
-  }
-#endif
+  // Do nothing (reference count to A is incremented in base class)
 }
 //-----------------------------------------------------------------------------
-PETScMatrix::PETScMatrix(const PETScMatrix& A): _use_gpu(false)
+PETScMatrix::PETScMatrix(const PETScMatrix& A) : PETScBaseMatrix(NULL),
+                                                 _use_gpu(false)
 {
-  *this = A;
+  if (A.mat())
+  {
+    PetscErrorCode ierr = MatDuplicate(A.mat(), MAT_COPY_VALUES, &_matA);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "MatDuplicate");
+  }
 }
 //-----------------------------------------------------------------------------
 PETScMatrix::~PETScMatrix()
 {
-  // Do nothing
+  // Do nothing (PETSc matrix is destroyed in base class)
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<GenericMatrix> PETScMatrix::copy() const
+std::shared_ptr<GenericMatrix> PETScMatrix::copy() const
 {
-  boost::shared_ptr<GenericMatrix> B;
-  if (!_A)
-    B.reset(new PETScMatrix());
-  else
-  {
-    // Create copy of PETSc matrix
-    boost::shared_ptr<Mat> _Acopy(new Mat, PETScMatrixDeleter());
-    MatDuplicate(*_A, MAT_COPY_VALUES, _Acopy.get());
-
-    // Create PETScMatrix
-    B.reset(new PETScMatrix(_Acopy));
-  }
-
-  return B;
+  return std::shared_ptr<GenericMatrix>(new PETScMatrix(*this));
 }
 //-----------------------------------------------------------------------------
 void PETScMatrix::init(const TensorLayout& tensor_layout)
 {
+  PetscErrorCode ierr;
+
   // Get global dimensions and local range
   dolfin_assert(tensor_layout.rank() == 2);
   const std::size_t M = tensor_layout.size(0);
   const std::size_t N = tensor_layout.size(1);
-  const std::pair<std::size_t, std::size_t> row_range = tensor_layout.local_range(0);
-  const std::pair<std::size_t, std::size_t> col_range = tensor_layout.local_range(1);
+  const std::pair<std::size_t, std::size_t> row_range
+    = tensor_layout.local_range(0);
+  const std::pair<std::size_t, std::size_t> col_range
+    = tensor_layout.local_range(1);
   const std::size_t m = row_range.second - row_range.first;
   const std::size_t n = col_range.second - col_range.first;
-  dolfin_assert(M > 0 && N > 0 && m > 0 && n > 0);
 
   // Get sparsity pattern
   dolfin_assert(tensor_layout.sparsity_pattern());
-  const GenericSparsityPattern& sparsity_pattern = *tensor_layout.sparsity_pattern();
+  const GenericSparsityPattern& sparsity_pattern
+    = *tensor_layout.sparsity_pattern();
 
-  // Create matrix (any old matrix is destroyed automatically)
-  if (_A && !_A.unique())
+  if (_matA)
   {
-    dolfin_error("PETScMatrix.cpp",
-                 "initialize PETSc matrix",
-                 "More than one object points to the underlying PETSc object");
+    #ifdef DOLFIN_DEPRECATION_ERROR
+    error("PETScMatrix may not be initialized more than once. Remove build definiton -DDOLFIN_DEPRECATION_ERROR to change this to a warning.");
+    #else
+    warning("PETScMatrix may not be initialized more than once. In version > 1.4, this will become an error.");
+    #endif
+    MatDestroy(&_matA);
   }
-  _A.reset(new Mat, PETScMatrixDeleter());
 
   // Initialize matrix
   if (row_range.first == 0 && row_range.second == M)
@@ -140,26 +130,43 @@ void PETScMatrix::init(const TensorLayout& tensor_layout)
     sparsity_pattern.num_nonzeros_diagonal(num_nonzeros);
 
     // Create matrix
-    MatCreate(PETSC_COMM_SELF, _A.get());
+    ierr = MatCreate(PETSC_COMM_SELF, &_matA);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "MatCreate");
 
     // Set size
-    MatSetSizes(*_A, M, N, M, N);
+    ierr = MatSetSizes(_matA, M, N, M, N);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "MatSetSizes");
 
     // Set matrix type according to chosen architecture
     if (!_use_gpu)
-      MatSetType(*_A, MATSEQAIJ);
+    {
+      ierr = MatSetType(_matA, MATSEQAIJ);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "MatSetType");
+    }
     #ifdef HAS_PETSC_CUSP
     else
-      MatSetType(*_A, MATSEQAIJCUSP);
+    {
+      ierr = MatSetType(_matA, MATSEQAIJCUSP);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "MatSetType");
+    }
     #endif
 
+    // Set block size
+    if (tensor_layout.block_size > 1)
+    {
+     ierr =  MatSetBlockSize(_matA, tensor_layout.block_size);
+     if (ierr != 0) petsc_error(ierr, __FILE__, "MatSetBlockSize");
+    }
+
     // FIXME: Change to MatSeqAIJSetPreallicationCSR for improved performance?
 
     // Allocate space (using data from sparsity pattern)
 
     // Copy number of non-zeros to PetscInt type
-    const std::vector<PetscInt> _num_nonzeros(num_nonzeros.begin(), num_nonzeros.end());
-    MatSeqAIJSetPreallocation(*_A, 0, _num_nonzeros.data());
+    const std::vector<PetscInt> _num_nonzeros(num_nonzeros.begin(),
+                                              num_nonzeros.end());
+    ierr = MatSeqAIJSetPreallocation(_matA, 0, _num_nonzeros.data());
+    if (ierr != 0) petsc_error(ierr, __FILE__, "MatSeqAIJSetPreallocation");
 
     // Set column indices
     /*
@@ -174,7 +181,7 @@ void PETScMatrix::init(const TensorLayout& tensor_layout)
       //  cout << "  Col: " << _column_indices[i][j] << endl;
       column_indices.insert(column_indices.end(), _column_indices[i].begin(), _column_indices[i].end());
     }
-    MatSeqAIJSetColumnIndices(*_A, &column_indices[0]);
+    MatSeqAIJSetColumnIndices(_matA, &column_indices[0]);
     */
   }
   else
@@ -182,15 +189,11 @@ void PETScMatrix::init(const TensorLayout& tensor_layout)
     if (_use_gpu)
     {
       not_working_in_parallel("Due to limitations in PETSc, "
-          "distributed PETSc Cusp matrices");
+                              "distributed PETSc Cusp matrices");
     }
 
     // FIXME: Try using MatStashSetInitialSize to optimise performance
 
-    //info("Initializing parallel PETSc matrix (MPIAIJ) of size %d x %d.", M, N);
-    //info("Local range is [%d, %d] x [%d, %d].",
-    //     row_range.first, row_range.second, col_range.first, col_range.second);
-
     // Get number of nonzeros for each row from sparsity pattern
     std::vector<std::size_t> num_nonzeros_diagonal;
     std::vector<std::size_t> num_nonzeros_off_diagonal;
@@ -198,91 +201,132 @@ void PETScMatrix::init(const TensorLayout& tensor_layout)
     sparsity_pattern.num_nonzeros_off_diagonal(num_nonzeros_off_diagonal);
 
     // Create matrix
-    MatCreate(PETSC_COMM_WORLD, _A.get());
+    ierr = MatCreate(PETSC_COMM_WORLD, &_matA);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "MatCreate");
 
     // Set size
-    MatSetSizes(*_A, m, n, M, N);
+    ierr = MatSetSizes(_matA, m, n, M, N);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "MatSetSizes");
 
     // Set matrix type
-    MatSetType(*_A, MATMPIAIJ);
+    ierr = MatSetType(_matA, MATMPIAIJ);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "MatSetType");
 
+    // Set block size
+    if (tensor_layout.block_size > 1)
+    {
+      ierr = MatSetBlockSize(_matA, tensor_layout.block_size);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "MatSetBlockSize");
+    }
     // Allocate space (using data from sparsity pattern)
-    const std::vector<PetscInt> _num_nonzeros_diagonal(num_nonzeros_diagonal.begin(),
-                                                       num_nonzeros_diagonal.end());
-    const std::vector<PetscInt> _num_nonzeros_off_diagonal(num_nonzeros_off_diagonal.begin(),
-                                                           num_nonzeros_off_diagonal.end());
-    MatMPIAIJSetPreallocation(*_A, 0, _num_nonzeros_diagonal.data(),
-                                   0, _num_nonzeros_off_diagonal.data());
+    const std::vector<PetscInt>
+      _num_nonzeros_diagonal(num_nonzeros_diagonal.begin(),
+                             num_nonzeros_diagonal.end());
+    const std::vector<PetscInt>
+      _num_nonzeros_off_diagonal(num_nonzeros_off_diagonal.begin(),
+                                 num_nonzeros_off_diagonal.end());
+    ierr = MatMPIAIJSetPreallocation(_matA, 0, _num_nonzeros_diagonal.data(),
+                                     0, _num_nonzeros_off_diagonal.data());
+    if (ierr != 0) petsc_error(ierr, __FILE__, "MatMPIAIJSetPreallocation");
   }
 
   // Set some options
 
   // Do not allow more entries than have been pre-allocated
-  MatSetOption(*_A, MAT_NEW_NONZERO_ALLOCATION_ERR, PETSC_TRUE);
-  MatSetOption(*_A, MAT_KEEP_NONZERO_PATTERN, PETSC_TRUE);
-  MatSetFromOptions(*_A);
+  ierr = MatSetOption(_matA, MAT_NEW_NONZERO_ALLOCATION_ERR, PETSC_TRUE);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatSetOption");
+
+  ierr = MatSetOption(_matA, MAT_KEEP_NONZERO_PATTERN, PETSC_TRUE);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatSetOption");
+
+  ierr = MatSetFromOptions(_matA);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatSetFromOptions");
 
   #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR > 2
-  MatSetUp(*_A.get());
+  ierr = MatSetUp(_matA);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatSetUp");
   #endif
 }
 //-----------------------------------------------------------------------------
-void PETScMatrix::get(double* block, std::size_t m, const dolfin::la_index* rows,
-                                     std::size_t n, const dolfin::la_index* cols) const
+bool PETScMatrix::empty() const
+{
+  return _matA ? false : true;
+}
+//-----------------------------------------------------------------------------
+void PETScMatrix::get(double* block,
+                      std::size_t m, const dolfin::la_index* rows,
+                      std::size_t n, const dolfin::la_index* cols) const
 {
   // Get matrix entries (must be on this process)
-  dolfin_assert(_A);
-  MatGetValues(*_A, m, rows, n, cols, block);
+  dolfin_assert(_matA);
+  PetscErrorCode ierr = MatGetValues(_matA, m, rows, n, cols, block);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatGetValues");
 }
 //-----------------------------------------------------------------------------
-void PETScMatrix::set(const double* block, std::size_t m, const dolfin::la_index* rows,
-                                           std::size_t n, const dolfin::la_index* cols)
+void PETScMatrix::set(const double* block,
+                      std::size_t m, const dolfin::la_index* rows,
+                      std::size_t n, const dolfin::la_index* cols)
 {
-  dolfin_assert(_A);
-  MatSetValues(*_A, m, rows, n, cols, block, INSERT_VALUES);
+  dolfin_assert(_matA);
+  PetscErrorCode ierr = MatSetValues(_matA, m, rows, n, cols, block,
+                                    INSERT_VALUES);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatSetValues");
 }
 //-----------------------------------------------------------------------------
-void PETScMatrix::add(const double* block, std::size_t m, const dolfin::la_index* rows,
-                                           std::size_t n, const dolfin::la_index* cols)
+void PETScMatrix::add(const double* block,
+                      std::size_t m, const dolfin::la_index* rows,
+                      std::size_t n, const dolfin::la_index* cols)
 {
-  dolfin_assert(_A);
-  MatSetValues(*_A, m, rows, n, cols, block, ADD_VALUES);
+  dolfin_assert(_matA);
+  PetscErrorCode ierr = MatSetValues(_matA, m, rows, n, cols, block, ADD_VALUES);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatSetValues");
 }
 //-----------------------------------------------------------------------------
 void PETScMatrix::axpy(double a, const GenericMatrix& A,
                        bool same_nonzero_pattern)
 {
+  PetscErrorCode ierr;
+
   const PETScMatrix* AA = &as_type<const PETScMatrix>(A);
-  dolfin_assert(_A);
+  dolfin_assert(_matA);
   dolfin_assert(AA->mat());
   if (same_nonzero_pattern)
-    MatAXPY(*_A, a, *AA->mat(), SAME_NONZERO_PATTERN);
+  {
+    ierr = MatAXPY(_matA, a, AA->mat(), SAME_NONZERO_PATTERN);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "MatAXPY");
+  }
   else
-    MatAXPY(*_A, a, *AA->mat(), DIFFERENT_NONZERO_PATTERN);
+  {
+    ierr = MatAXPY(_matA, a, AA->mat(), DIFFERENT_NONZERO_PATTERN);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "MatAXPY");
+  }
 }
 //-----------------------------------------------------------------------------
 void PETScMatrix::getrow(std::size_t row, std::vector<std::size_t>& columns,
                          std::vector<double>& values) const
 {
-  dolfin_assert(_A);
+  dolfin_assert(_matA);
 
+  PetscErrorCode ierr;
   const PetscInt *cols = 0;
   const double *vals = 0;
   PetscInt ncols = 0;
-  MatGetRow(*_A, row, &ncols, &cols, &vals);
+  ierr = MatGetRow(_matA, row, &ncols, &cols, &vals);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatGetRow");
 
   // Assign values to std::vectors
   columns.assign(cols, cols + ncols);
   values.assign(vals, vals + ncols);
 
-  MatRestoreRow(*_A, row, &ncols, &cols, &vals);
+  ierr = MatRestoreRow(_matA, row, &ncols, &cols, &vals);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatRestorRow");
 }
 //-----------------------------------------------------------------------------
 void PETScMatrix::setrow(std::size_t row,
                          const std::vector<std::size_t>& columns,
                          const std::vector<double>& values)
 {
-  dolfin_assert(_A);
+  dolfin_assert(_matA);
 
   // Check size of arrays
   if (columns.size() != values.size())
@@ -305,26 +349,31 @@ void PETScMatrix::setrow(std::size_t row,
 //-----------------------------------------------------------------------------
 void PETScMatrix::zero(std::size_t m, const dolfin::la_index* rows)
 {
-  dolfin_assert(_A);
+  dolfin_assert(_matA);
 
+  PetscErrorCode ierr;
   IS is = 0;
   PetscScalar null = 0.0;
   const PetscInt _m = m;
-  ISCreateGeneral(PETSC_COMM_SELF, _m, rows, PETSC_COPY_VALUES, &is);
-  MatZeroRowsIS(*_A, is, null, NULL, NULL);
-  ISDestroy(&is);
+  ierr = ISCreateGeneral(PETSC_COMM_SELF, _m, rows, PETSC_COPY_VALUES, &is);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "ISCreateGeneral");
+  ierr = MatZeroRowsIS(_matA, is, null, NULL, NULL);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatZeroRowsIS");
+  ierr = ISDestroy(&is);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "ISDestroy");
 }
 //-----------------------------------------------------------------------------
 void PETScMatrix::ident(std::size_t m, const dolfin::la_index* rows)
 {
-  dolfin_assert(_A);
+  dolfin_assert(_matA);
 
   PetscErrorCode ierr;
   IS is = 0;
   PetscScalar one = 1.0;
   const PetscInt _m = m;
-  ISCreateGeneral(PETSC_COMM_SELF, _m, rows, PETSC_COPY_VALUES, &is);
-  ierr = MatZeroRowsIS(*_A, is, one, NULL, NULL);
+  ierr = ISCreateGeneral(PETSC_COMM_SELF, _m, rows, PETSC_COPY_VALUES, &is);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "ISCreateGeneral");
+  ierr = MatZeroRowsIS(_matA, is, one, NULL, NULL);
   if (ierr == PETSC_ERR_ARG_WRONGSTATE)
   {
     dolfin_error("PETScMatrix.cpp",
@@ -332,12 +381,15 @@ void PETScMatrix::ident(std::size_t m, const dolfin::la_index* rows)
                  "some diagonal elements not preallocated "
                  "(try assembler option keep_diagonal)");
   }
-  ISDestroy(&is);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatZeroRowsIS");
+
+  ierr = ISDestroy(&is);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "ISDestroy");
 }
 //-----------------------------------------------------------------------------
 void PETScMatrix::mult(const GenericVector& x, GenericVector& y) const
 {
-  dolfin_assert(_A);
+  dolfin_assert(_matA);
 
   const PETScVector& xx = as_type<const PETScVector>(x);
   PETScVector& yy = as_type<PETScVector>(y);
@@ -351,7 +403,7 @@ void PETScMatrix::mult(const GenericVector& x, GenericVector& y) const
 
   // Resize RHS if empty
   if (yy.size() == 0)
-    resize(yy, 0);
+    init_vector(yy, 0);
 
   if (size(0) != yy.size())
   {
@@ -360,12 +412,13 @@ void PETScMatrix::mult(const GenericVector& x, GenericVector& y) const
                  "Vector for matrix-vector result has wrong size");
   }
 
-  MatMult(*_A, *xx.vec(), *yy.vec());
+  PetscErrorCode ierr = MatMult(_matA, xx.vec(), yy.vec());
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatMult");
 }
 //-----------------------------------------------------------------------------
 void PETScMatrix::transpmult(const GenericVector& x, GenericVector& y) const
 {
-  dolfin_assert(_A);
+  dolfin_assert(_matA);
 
   const PETScVector& xx = as_type<const PETScVector>(x);
   PETScVector& yy = as_type<PETScVector>(y);
@@ -379,7 +432,7 @@ void PETScMatrix::transpmult(const GenericVector& x, GenericVector& y) const
 
   // Resize RHS if empty
   if (yy.size() == 0)
-    resize(yy, 1);
+    init_vector(yy, 1);
 
   if (size(1) != yy.size())
   {
@@ -388,12 +441,30 @@ void PETScMatrix::transpmult(const GenericVector& x, GenericVector& y) const
                  "Vector for transpose matrix-vector result has wrong size");
   }
 
-  MatMultTranspose(*_A, *xx.vec(), *yy.vec());
+  PetscErrorCode ierr = MatMultTranspose(_matA, xx.vec(), yy.vec());
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatMultTranspose");
+}
+//-----------------------------------------------------------------------------
+void PETScMatrix::set_diagonal(const GenericVector& x)
+{
+  dolfin_assert(_matA);
+
+  const PETScVector& xx = x.down_cast<PETScVector>();
+  if (size(1) != size(0) || size(0) != xx.size())
+  {
+    dolfin_error("PETScMatrix.cpp",
+                 "set diagonal of a PETSc matrix",
+                 "Matrix and vector dimensions don't match for matrix-vector set");
+  }
+
+  PetscErrorCode ierr = MatDiagonalSet(_matA, xx.vec(), INSERT_VALUES);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatDiagonalSet");
+  apply("insert");
 }
 //-----------------------------------------------------------------------------
 double PETScMatrix::norm(std::string norm_type) const
 {
-  dolfin_assert(_A);
+  dolfin_assert(_matA);
 
   // Check that norm is known
   if (norm_types.count(norm_type) == 0)
@@ -404,29 +475,38 @@ double PETScMatrix::norm(std::string norm_type) const
   }
 
   double value = 0.0;
-  MatNorm(*_A, norm_types.find(norm_type)->second, &value);
+  PetscErrorCode ierr = MatNorm(_matA, norm_types.find(norm_type)->second,
+                                &value);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatNorm");
   return value;
 }
 //-----------------------------------------------------------------------------
 void PETScMatrix::apply(std::string mode)
 {
-  Timer("Apply (matrix)");
+  Timer timer("Apply (PETScMatrix)");
 
-  dolfin_assert(_A);
+  dolfin_assert(_matA);
+  PetscErrorCode ierr;
   if (mode == "add")
   {
-    MatAssemblyBegin(*_A, MAT_FINAL_ASSEMBLY);
-    MatAssemblyEnd(*_A, MAT_FINAL_ASSEMBLY);
+    ierr = MatAssemblyBegin(_matA, MAT_FINAL_ASSEMBLY);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "MatAssemblyBegin");
+    ierr = MatAssemblyEnd(_matA, MAT_FINAL_ASSEMBLY);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "MatAssemblyEnd");
   }
   else if (mode == "insert")
   {
-    MatAssemblyBegin(*_A, MAT_FINAL_ASSEMBLY);
-    MatAssemblyEnd(*_A, MAT_FINAL_ASSEMBLY);
+    ierr = MatAssemblyBegin(_matA, MAT_FINAL_ASSEMBLY);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "MatAssemblyBegin");
+    ierr = MatAssemblyEnd(_matA, MAT_FINAL_ASSEMBLY);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "MatAssemblyEnd");
   }
   else if (mode == "flush")
   {
-    MatAssemblyBegin(*_A, MAT_FLUSH_ASSEMBLY);
-    MatAssemblyEnd(*_A, MAT_FLUSH_ASSEMBLY);
+    ierr = MatAssemblyBegin(_matA, MAT_FLUSH_ASSEMBLY);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "MatAssemblyBegin");
+    ierr = MatAssemblyEnd(_matA, MAT_FLUSH_ASSEMBLY);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "MatAssemblyEnd");
   }
   else
   {
@@ -436,23 +516,33 @@ void PETScMatrix::apply(std::string mode)
   }
 }
 //-----------------------------------------------------------------------------
+MPI_Comm PETScMatrix::mpi_comm() const
+{
+  dolfin_assert(_matA);
+  MPI_Comm mpi_comm = MPI_COMM_NULL;
+  PetscObjectGetComm((PetscObject)_matA, &mpi_comm);
+  return mpi_comm;
+}
+//-----------------------------------------------------------------------------
 void PETScMatrix::zero()
 {
-  dolfin_assert(_A);
-  MatZeroEntries(*_A);
+  dolfin_assert(_matA);
+  PetscErrorCode ierr = MatZeroEntries(_matA);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatZeroEntries");
 }
 //-----------------------------------------------------------------------------
 const PETScMatrix& PETScMatrix::operator*= (double a)
 {
-  dolfin_assert(_A);
-  MatScale(*_A, a);
+  dolfin_assert(_matA);
+  PetscErrorCode ierr = MatScale(_matA, a);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatScale");
   return *this;
 }
 //-----------------------------------------------------------------------------
 const PETScMatrix& PETScMatrix::operator/= (double a)
 {
-  dolfin_assert(_A);
-  MatScale(*_A, 1.0/a);
+  dolfin_assert(_matA);
+  MatScale(_matA, 1.0/a);
   return *this;
 }
 //-----------------------------------------------------------------------------
@@ -464,9 +554,10 @@ const GenericMatrix& PETScMatrix::operator= (const GenericMatrix& A)
 //-----------------------------------------------------------------------------
 bool PETScMatrix::is_symmetric(double tol) const
 {
-  dolfin_assert(_A);
+  dolfin_assert(_matA);
   PetscBool symmetric = PETSC_FALSE;
-  MatIsSymmetric(*_A, tol, &symmetric);
+  PetscErrorCode ierr = MatIsSymmetric(_matA, tol, &symmetric);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatIsSymmetric");
   return symmetric == PETSC_TRUE ? true : false;
 }
 //-----------------------------------------------------------------------------
@@ -479,56 +570,93 @@ GenericLinearAlgebraFactory& PETScMatrix::factory() const
     return PETScCuspFactory::instance();
   #endif
 
-  // Return something to keep the compiler happy. Code will never be reached.
+  // Return something to keep the compiler happy. Code will never be
+  // reached.
   return PETScFactory::instance();
 }
 //-----------------------------------------------------------------------------
 const PETScMatrix& PETScMatrix::operator= (const PETScMatrix& A)
 {
   if (!A.mat())
-    _A.reset();
+  {
+    if (_matA)
+    {
+      #ifdef DOLFIN_DEPRECATION_ERROR
+      error("PETScVector may not be initialized more than once. Remove build definiton -DDOLFIN_DEPRECATION_ERROR to change this to a warning. Error is in PETScMatrix::operator=.");
+      #else
+      warning("PETScVector may not be initialized more than once. In version > 1.4, this will become an error. Warning is in PETScMatrix::operator=.");
+      #endif
+      MatDestroy(&_matA);
+    }
+    _matA = NULL;
+  }
   else if (this != &A) // Check for self-assignment
   {
-    if (_A && !_A.unique())
+    if (_matA)
     {
-      dolfin_error("PETScMatrix.cpp",
-                   "assign to PETSc matrix",
-                   "More than one object points to the underlying PETSc object");
+      // Get reference count to _matA
+      PetscInt ref_count = 0;
+      PetscObjectGetReference((PetscObject)_matA, &ref_count);
+      if (ref_count > 1)
+      {
+        dolfin_error("PETScMatrix.cpp",
+                     "assign to PETSc matrix",
+                     "More than one object points to the underlying PETSc object");
+      }
+      #ifdef DOLFIN_DEPRECATION_ERROR
+      error("PETScMatrix may not be initialized more than once. Remove build definiton -DDOLFIN_DEPRECATION_ERROR to change this to a warning. Error is in PETScMatrix::operator=.");
+      #else
+      warning("PETScMatrix may not be initialized more than once. In version > 1.4, this will become an error. Warning is in PETScMatrix::operator=.");
+      #endif
+      MatDestroy(&_matA);
     }
-    _A.reset(new Mat, PETScMatrixDeleter());
 
     // Duplicate with the same pattern as A.A
-    MatDuplicate(*A.mat(), MAT_COPY_VALUES, _A.get());
+    PetscErrorCode ierr = MatDuplicate(A.mat(), MAT_COPY_VALUES, &_matA);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "MatDuplicate");
   }
   return *this;
 }
 //-----------------------------------------------------------------------------
 void PETScMatrix::binary_dump(std::string file_name) const
 {
+  PetscErrorCode ierr;
+
   PetscViewer view_out;
-  PetscViewerBinaryOpen(PETSC_COMM_WORLD, file_name.c_str(),
-                        FILE_MODE_WRITE, &view_out);
-  MatView(*(_A.get()), view_out);
-  PetscViewerDestroy(&view_out);
+  ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD, file_name.c_str(),
+                               FILE_MODE_WRITE, &view_out);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "PetscViewerBinaryOpen");
+
+  ierr = MatView(_matA, view_out);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "MatView");
+
+  ierr = PetscViewerDestroy(&view_out);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "PetscViewerDestroy");
 }
 //-----------------------------------------------------------------------------
 std::string PETScMatrix::str(bool verbose) const
 {
-  if (!_A)
+  if (!_matA)
     return "<Uninitialized PETScMatrix>";
 
   std::stringstream s;
-
   if (verbose)
   {
     warning("Verbose output for PETScMatrix not implemented, calling PETSc MatView directly.");
 
     // FIXME: Maybe this could be an option?
-    dolfin_assert(_A);
-    if (MPI::num_processes() > 1)
-      MatView(*_A, PETSC_VIEWER_STDOUT_WORLD);
+    dolfin_assert(_matA);
+    PetscErrorCode ierr;
+    if (MPI::size(MPI_COMM_WORLD) > 1)
+    {
+      ierr = MatView(_matA, PETSC_VIEWER_STDOUT_WORLD);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "MatView");
+    }
     else
-      MatView(*_A, PETSC_VIEWER_STDOUT_SELF);
+    {
+      ierr = MatView(_matA, PETSC_VIEWER_STDOUT_SELF);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "MatView");
+    }
   }
   else
     s << "<PETScMatrix of size " << size(0) << " x " << size(1) << ">";
