@@ -27,7 +27,7 @@
 
 #include <string>
 #include <vector>
-#include <boost/shared_ptr.hpp>
+#include <memory>
 #include <petscpc.h>
 
 #include <dolfin/common/types.h>
@@ -69,13 +69,13 @@ namespace dolfin
     void set_nullspace(const VectorSpaceBasis& near_nullspace);
 
     /// Return the PETSc null space
-    boost::shared_ptr<const MatNullSpace> near_nullspace() const
+    MatNullSpace near_nullspace() const
     { return petsc_near_nullspace; }
 
-    /// Set the coordinates of the operator (matrix) rows and geometric
-    /// dimension d. This is can be used by required for certain
-    /// preconditioners, e.g. ML. The input for this function can be
-    /// generated using GenericDofMap::tabulate_all_dofs.
+    /// Set the coordinates of the operator (matrix) rows and
+    /// geometric dimension d. This is can be used by required for
+    /// certain preconditioners, e.g. ML. The input for this function
+    /// can be generated using GenericDofMap::tabulate_all_dofs.
     void set_coordinates(const std::vector<double>& x, std::size_t dim);
 
     // FIXME: Document
@@ -110,10 +110,8 @@ namespace dolfin
     // Near null space vectors
     std::vector<PETScVector> _near_nullspace;
 
-    // PETSc null space. Would like this to be a scoped_ptr, but it
-    // doesn't support custom deleters. Change to std::unique_ptr in
-    // the future.
-    boost::shared_ptr<MatNullSpace> petsc_near_nullspace;
+    // PETSc near null space.
+    MatNullSpace petsc_near_nullspace;
 
     // Operator row coordinates
     std::vector<double> _coordinates;
