@@ -33,27 +33,33 @@ using namespace dolfin;
 
 //-----------------------------------------------------------------------------
 SparsityPattern::SparsityPattern(std::size_t primary_dim)
-    : GenericSparsityPattern(primary_dim), distributed(false)
+  : GenericSparsityPattern(primary_dim), _mpi_comm(MPI_COMM_NULL)
 {
   // Do nothing
 }
 //-----------------------------------------------------------------------------
-SparsityPattern::SparsityPattern(const std::vector<std::size_t>& dims,
+SparsityPattern::SparsityPattern(
+  const MPI_Comm mpi_comm,
+  const std::vector<std::size_t>& dims,
   const std::vector<std::pair<std::size_t, std::size_t> >& local_range,
   const std::vector<const boost::unordered_map<std::size_t, unsigned int>* > off_process_owner,
-    std::size_t primary_dim)
-  : GenericSparsityPattern(primary_dim), distributed(false)
+  std::size_t primary_dim)
+  : GenericSparsityPattern(primary_dim), _mpi_comm(MPI_COMM_NULL)
 {
-  init(dims, local_range, off_process_owner);
+  init(mpi_comm, dims, local_range, off_process_owner);
 }
 //-----------------------------------------------------------------------------
-void SparsityPattern::init(const std::vector<std::size_t>& dims,
+void SparsityPattern::init(
+  const MPI_Comm mpi_comm,
+  const std::vector<std::size_t>& dims,
   const std::vector<std::pair<std::size_t, std::size_t> >& local_range,
   const std::vector<const boost::unordered_map<std::size_t, unsigned int>* > off_process_owner)
 {
   // Only rank 2 sparsity patterns are supported
   dolfin_assert(dims.size() == 2);
 
+  _mpi_comm = mpi_comm;
+
   const std::size_t _primary_dim = primary_dim();
 
   // Check that dimensions match
@@ -85,13 +91,6 @@ void SparsityPattern::init(const std::vector<std::size_t>& dims,
                  "Primary dimension must be less than 2 (0=row major, 1=column major");
   }
 
-  // Check if sparsity pattern is distributed
-  if (_local_range[_primary_dim].first != 0
-      || _local_range[_primary_dim].second != dims[_primary_dim])
-  {
-    distributed = true;
-  }
-
   // Resize diagonal block
   dolfin_assert(_local_range[_primary_dim].second
                 > _local_range[_primary_dim].first);
@@ -99,11 +98,8 @@ void SparsityPattern::init(const std::vector<std::size_t>& dims,
                   - _local_range[_primary_dim].first);
 
   // Resize off-diagonal block (only needed when local range != global range)
-  if (distributed)
-  {
-    off_diagonal.resize(_local_range[_primary_dim].second
-                        - _local_range[_primary_dim].first);
-  }
+  off_diagonal.resize(_local_range[_primary_dim].second
+                      - _local_range[_primary_dim].first);
 }
 //-----------------------------------------------------------------------------
 void SparsityPattern::insert(const std::vector<const std::vector<dolfin::la_index>* >& entries)
@@ -139,7 +135,7 @@ void SparsityPattern::insert(const std::vector<const std::vector<dolfin::la_inde
                  _local_range[primary_codim].second);
 
   // Check local range
-  if (!distributed)
+  if (MPI::size(_mpi_comm) == 1)
   {
     // Sequential mode, do simple insertion
     std::vector<dolfin::la_index>::const_iterator i_index;
@@ -312,15 +308,15 @@ void SparsityPattern::apply()
   else
     primary_codim = 0;
 
-  const std::size_t num_processes = MPI::num_processes();
-  const std::size_t proc_number = MPI::process_number();
+  const std::size_t num_processes = MPI::size(_mpi_comm);
+  const std::size_t proc_number = MPI::rank(_mpi_comm);
 
   // Print some useful information
   if (get_log_level() <= DBG)
     info_statistics();
 
   // Communicate non-local blocks if any
-  if (distributed)
+  if (MPI::size(_mpi_comm) > 1)
   {
     // Figure out correct process for each non-local entry
     dolfin_assert(non_local.size() % 2 == 0);
@@ -347,7 +343,7 @@ void SparsityPattern::apply()
 
     // Communicate non-local entries to other processes
     std::vector<std::vector<std::size_t> > non_local_received;
-    MPI::all_to_all(non_local_send, non_local_received);
+    MPI::all_to_all(_mpi_comm, non_local_send, non_local_received);
 
     // Insert non-local entries received from other processes
     for (std::size_t p = 0; p < num_processes; ++p)
@@ -472,11 +468,8 @@ void SparsityPattern::info_statistics() const
 
   std::size_t size0 = _local_range[0].second;
   std::size_t size1 = _local_range[1].second;
-  if (distributed)
-  {
-    size0 = MPI::max(size0);
-    size1 = MPI::max(size1);
-  }
+  size0 = MPI::max(_mpi_comm, size0);
+  size1 = MPI::max(_mpi_comm, size1);
 
   // Return number of entries
   cout << "Matrix of size " << size0 << " x " << size1 << " has "
