@@ -24,9 +24,10 @@
 #include <string>
 #include <utility>
 #include <vector>
-#include <boost/shared_ptr.hpp>
+#include <memory>
 
 #include "dolfin/common/types.h"
+#include "dolfin/common/MPI.h"
 #include "GenericSparsityPattern.h"
 
 namespace dolfin
@@ -45,16 +46,18 @@ namespace dolfin
     TensorLayout(std::size_t primary_dim, bool sparsity_pattern);
 
     /// Create a tensor layout
-    TensorLayout(const std::vector<std::size_t>& dims, std::size_t primary_dim,
+    TensorLayout(const MPI_Comm mpi_comm,
+                 const std::vector<std::size_t>& dims,
+                 std::size_t primary_dim,
                  std::size_t block_size,
-                 const std::vector<std::pair<std::size_t,
-                 std::size_t> >& ownership_range,
+                 const std::vector<std::pair<std::size_t, std::size_t> >& ownership_range,
                  bool sparsity_pattern);
 
     /// Initialize tensor layout
-    void init(const std::vector<std::size_t>& dims, std::size_t block_size,
-              const std::vector<std::pair<std::size_t,
-              std::size_t> >& ownership_range);
+    void init(const MPI_Comm mpi_comm,
+              const std::vector<std::size_t>& dims,
+              std::size_t block_size,
+              const std::vector<std::pair<std::size_t, std::size_t> >& ownership_range);
 
     /// Return rank
     std::size_t rank() const;
@@ -67,11 +70,11 @@ namespace dolfin
     std::pair<std::size_t, std::size_t> local_range(std::size_t dim) const;
 
     /// Return sparsity pattern (possibly null)
-    boost::shared_ptr<GenericSparsityPattern> sparsity_pattern()
+    std::shared_ptr<GenericSparsityPattern> sparsity_pattern()
     { return _sparsity_pattern; }
 
     /// Return sparsity pattern (possibly null), const version
-    const boost::shared_ptr<GenericSparsityPattern> sparsity_pattern() const
+    std::shared_ptr<const GenericSparsityPattern> sparsity_pattern() const
     { return _sparsity_pattern; }
 
     /// Return informal string representation (pretty-print)
@@ -81,11 +84,18 @@ namespace dolfin
     const std::size_t primary_dim;
 
     /// Dofmap block size, e.g. 3 for 3D elasticity with a suitable
-    /// ordered dofmao
+    /// ordered dofmap
     std::size_t block_size;
 
+    /// Return MPI communicator
+    MPI_Comm mpi_comm() const
+    { return _mpi_comm; }
+
   private:
 
+    // MPI communicator
+    MPI_Comm _mpi_comm;
+
     // Shape of tensor
     std::vector<std::size_t> _shape;
 
@@ -93,7 +103,7 @@ namespace dolfin
     std::vector<std::pair<std::size_t, std::size_t> > _ownership_range;
 
     // Sparsity pattern
-    boost::shared_ptr<GenericSparsityPattern> _sparsity_pattern;
+    std::shared_ptr<GenericSparsityPattern> _sparsity_pattern;
 
   };
 
