@@ -15,19 +15,14 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 //
-// Modified by Anders Logg 2011
-//
-// First added:  2010-02-25
-// Last changed: 2011-10-19
-
-#ifndef __DOFLIN_TRILINOS_PRECONDITIONER_H
-#define __DOFLIN_TRILINOS_PRECONDITIONER_H
+#ifndef __DOLFIN_TRILINOS_PRECONDITIONER_H
+#define __DOLFIN_TRILINOS_PRECONDITIONER_H
 
 #ifdef HAS_TRILINOS
 
 #include <string>
 #include <vector>
-#include <boost/shared_ptr.hpp>
+#include <memory>
 #include <Teuchos_ParameterList.hpp>
 #include <Teuchos_RCP.hpp>
 
@@ -39,16 +34,31 @@
 // Trilinos forward declarations
 class Epetra_MultiVector;
 class Epetra_RowMatrix;
+class Epetra_Operator;
 class Ifpack_Preconditioner;
+namespace Belos
+{
+  template<class ScalarType, class MV, class OP>
+  class LinearProblem;
+}
+
 namespace ML_Epetra
 {
   class MultiLevelPreconditioner;
 }
+
 namespace Teuchos
 {
   class ParameterList;
 }
 
+// some typdefs for Belos
+typedef double BelosScalarType;
+typedef Epetra_MultiVector BelosMultiVector;
+typedef Epetra_Operator BelosOperator;
+typedef Belos::LinearProblem<BelosScalarType, BelosMultiVector, BelosOperator>
+  BelosLinearProblem;
+
 namespace dolfin
 {
 
@@ -74,17 +84,19 @@ namespace dolfin
     virtual ~TrilinosPreconditioner();
 
     /// Set the precondtioner and matrix used in preconditioner
-    virtual void set(EpetraKrylovSolver& solver, const EpetraMatrix& P);
+    virtual void set(BelosLinearProblem& problem,
+                     const EpetraMatrix& P
+                     );
 
     /// Set the Trilonos preconditioner parameters list
-    void set_parameters(boost::shared_ptr<const Teuchos::ParameterList> list);
+    void set_parameters(std::shared_ptr<const Teuchos::ParameterList> list);
 
     /// Set the Trilonos preconditioner parameters list (for use from
     /// Python)
     void set_parameters(Teuchos::RCP<Teuchos::ParameterList> list);
 
-    /// Set basis for the null space of the operator. Setting this
-    /// is critical to the performance of some preconditioners, e.g. ML.
+    /// Set basis for the null space of the operator. Setting this is
+    /// critical to the performance of some preconditioners, e.g. ML.
     /// The vectors spanning the null space are copied.
     void set_nullspace(const VectorSpaceBasis& null_space);
 
@@ -103,7 +115,8 @@ namespace dolfin
   private:
 
     /// Setup the ML precondtioner
-    void set_ml(AztecOO& solver, const Epetra_RowMatrix& P);
+    void set_ml(BelosLinearProblem& problem,
+                const Epetra_RowMatrix& P);
 
     /// Named preconditioner
     std::string _preconditioner;
@@ -116,14 +129,14 @@ namespace dolfin
       _preconditioners_descr;
 
     // The Preconditioner
-    boost::shared_ptr<Ifpack_Preconditioner> ifpack_preconditioner;
-    boost::shared_ptr<ML_Epetra::MultiLevelPreconditioner> ml_preconditioner;
+    std::shared_ptr<Ifpack_Preconditioner> _ifpack_preconditioner;
+    std::shared_ptr<ML_Epetra::MultiLevelPreconditioner> _ml_preconditioner;
 
     // Parameter list
-    boost::shared_ptr<const Teuchos::ParameterList> parameter_list;
+    std::shared_ptr<const Teuchos::ParameterList> parameter_list;
 
     // Vectors spanning the null space
-    boost::shared_ptr<Epetra_MultiVector> _nullspace;
+    std::shared_ptr<Epetra_MultiVector> _nullspace;
 
     // Teuchos::ParameterList pointer, used when initialized with a
     // Teuchos::RCP shared_ptr
