@@ -22,6 +22,8 @@
 // Last changed: 2012-08-20
 
 
+#include <memory>
+
 #include <dolfin/common/NoDeleter.h>
 #include <dolfin/log/dolfin_log.h>
 #include "UmfpackLUSolver.h"
@@ -111,8 +113,8 @@ UmfpackLUSolver::UmfpackLUSolver()
   parameters = default_parameters();
 }
 //-----------------------------------------------------------------------------
-UmfpackLUSolver::UmfpackLUSolver(boost::shared_ptr<const GenericLinearOperator> A)
-  : _A(A)
+UmfpackLUSolver::UmfpackLUSolver(std::shared_ptr<const GenericLinearOperator> A)
+  : _matA(A)
 {
   // Set parameter values
   parameters = default_parameters();
@@ -123,28 +125,29 @@ UmfpackLUSolver::~UmfpackLUSolver()
   // Do nothing
 }
 //-----------------------------------------------------------------------------
-void UmfpackLUSolver::set_operator(const boost::shared_ptr<const GenericLinearOperator> A)
+void
+UmfpackLUSolver::set_operator(std::shared_ptr<const GenericLinearOperator> A)
 {
   symbolic.reset();
   numeric.reset();
-  _A = A;
-  dolfin_assert(_A);
+  _matA = A;
+  dolfin_assert(_matA);
 }
 //-----------------------------------------------------------------------------
 const GenericLinearOperator& UmfpackLUSolver::get_operator() const
 {
-  if (!_A)
+  if (!_matA)
   {
     dolfin_error("UmfpackLUSolver.cpp",
                  "access operator for PETSc Krylov solver",
                  "Operator has not been set");
   }
-  return *_A;
+  return *_matA;
 }
 //-----------------------------------------------------------------------------
 std::size_t UmfpackLUSolver::solve(GenericVector& x, const GenericVector& b)
 {
-  dolfin_assert(_A);
+  dolfin_assert(_matA);
 
   // Get some parameters
   const bool reuse_fact   = parameters["reuse_factorization"];
@@ -170,14 +173,14 @@ std::size_t
 UmfpackLUSolver::solve(const GenericLinearOperator& A, GenericVector& x,
                        const GenericVector& b)
 {
-  boost::shared_ptr<const GenericLinearOperator> Atmp(&A, NoDeleter());
+  std::shared_ptr<const GenericLinearOperator> Atmp(&A, NoDeleter());
   set_operator(Atmp);
   return solve(x, b);
 }
 //-----------------------------------------------------------------------------
 void UmfpackLUSolver::symbolic_factorize()
 {
-  if (!_A)
+  if (!_matA)
   {
     dolfin_error("UmfpackLUSolver.cpp",
                  "factorize matrix with UMFPACK LU solver (symbolic)",
@@ -189,7 +192,7 @@ void UmfpackLUSolver::symbolic_factorize()
   numeric.reset();
 
   // Need matrix data
-  boost::shared_ptr<const GenericMatrix> A = require_matrix(_A);
+  std::shared_ptr<const GenericMatrix> A = require_matrix(_matA);
 
   // Get matrix data
   boost::tuples::tuple<const std::size_t*, const std::size_t*,
@@ -205,7 +208,8 @@ void UmfpackLUSolver::symbolic_factorize()
   dolfin_assert(nnz >= M);
 
   // Factorize and solve
-  log(PROGRESS, "Symbolic factorization of a matrix of size %d x %d (UMFPACK).", M, N);
+  log(PROGRESS, "Symbolic factorization of a matrix of size %d x %d (UMFPACK).",
+      M, N);
 
   // Perform symbolic factorisation
   symbolic = umfpack_factorize_symbolic(M, N, Ap, Ai, Ax);
@@ -213,7 +217,7 @@ void UmfpackLUSolver::symbolic_factorize()
 //-----------------------------------------------------------------------------
 void UmfpackLUSolver::numeric_factorize()
 {
-  if (!_A)
+  if (!_matA)
   {
     dolfin_error("UmfpackLUSolver.cpp",
                  "factorize matrix with UMFPACK LU solver (numeric)",
@@ -221,7 +225,7 @@ void UmfpackLUSolver::numeric_factorize()
   }
 
   // Need matrix data
-  boost::shared_ptr<const GenericMatrix> A = require_matrix(_A);
+  std::shared_ptr<const GenericMatrix> A = require_matrix(_matA);
 
   // Get matrix data
   boost::tuples::tuple<const std::size_t*, const std::size_t*,
@@ -237,7 +241,8 @@ void UmfpackLUSolver::numeric_factorize()
   dolfin_assert(nnz >= M);
 
   // Factorize and solve
-  log(PROGRESS, "LU factorization of a matrix of size %d x %d (UMFPACK).", M, N);
+  log(PROGRESS, "LU factorization of a matrix of size %d x %d (UMFPACK).",
+      M, N);
 
   dolfin_assert(symbolic);
   numeric.reset();
@@ -249,7 +254,7 @@ void UmfpackLUSolver::numeric_factorize()
 std::size_t UmfpackLUSolver::solve_factorized(GenericVector& x,
                                               const GenericVector& b) const
 {
-  if (!_A)
+  if (!_matA)
   {
     dolfin_error("UmfpackLUSolver.cpp",
                  "solve linear system with UMFPACK LU solver",
@@ -257,14 +262,14 @@ std::size_t UmfpackLUSolver::solve_factorized(GenericVector& x,
   }
 
   // Need matrix data
-  const boost::shared_ptr<const GenericMatrix> A = require_matrix(_A);
+  std::shared_ptr<const GenericMatrix> A = require_matrix(_matA);
 
   dolfin_assert(A->size(0) == A->size(0));
   dolfin_assert(A->size(0) == b.size());
 
   // Resize x if required
-  if (A->size(1) != x.size())
-    x.resize(A->size(1));
+  if (x.empty())
+    x.init(MPI_COMM_SELF, A->size(1));
 
   if (!symbolic)
   {
@@ -281,7 +286,8 @@ std::size_t UmfpackLUSolver::solve_factorized(GenericVector& x,
   }
 
   // Get matrix data
-  boost::tuples::tuple<const std::size_t*, const std::size_t*, const double*, int> data = A->data();
+  boost::tuples::tuple<const std::size_t*, const std::size_t*, const double*,
+                       int> data = A->data();
   const std::size_t* Ap  = boost::tuples::get<0>(data);
   const std::size_t* Ai  = boost::tuples::get<1>(data);
   const double*      Ax  = boost::tuples::get<2>(data);
@@ -298,7 +304,7 @@ std::size_t UmfpackLUSolver::solve_factorized(GenericVector& x,
 //-----------------------------------------------------------------------------
 #ifdef HAS_UMFPACK
 //-----------------------------------------------------------------------------
-boost::shared_ptr<void>
+std::shared_ptr<void>
 UmfpackLUSolver::umfpack_factorize_symbolic(std::size_t M, std::size_t N,
                                             const std::size_t* Ap,
                                             const std::size_t* Ai,
@@ -309,7 +315,7 @@ UmfpackLUSolver::umfpack_factorize_symbolic(std::size_t M, std::size_t N,
   dolfin_assert(Ax);
 
   void* symbolic = 0;
-  boost::scoped_ptr<double> dnull;
+  std::unique_ptr<double> dnull;
 
   // Symbolic factorisation step (reordering, etc)
   if (sizeof(std::size_t) == sizeof(int))
@@ -319,7 +325,7 @@ UmfpackLUSolver::umfpack_factorize_symbolic(std::size_t M, std::size_t N,
     long int status = umfpack_di_symbolic(M, N, _Ap, _Ai, Ax, &symbolic,
                                           dnull.get(), dnull.get());
     umfpack_check_status(status, "symbolic");
-    return boost::shared_ptr<void>(symbolic, UmfpackIntSymbolicDeleter());
+    return std::shared_ptr<void>(symbolic, UmfpackIntSymbolicDeleter());
   }
   else if (sizeof(std::size_t) == sizeof(UF_long))
   {
@@ -328,7 +334,7 @@ UmfpackLUSolver::umfpack_factorize_symbolic(std::size_t M, std::size_t N,
     long int status = umfpack_dl_symbolic(M, N, _Ap, _Ai, Ax, &symbolic,
                                           dnull.get(), dnull.get());
     umfpack_check_status(status, "symbolic");
-    return boost::shared_ptr<void>(symbolic, UmfpackLongIntSymbolicDeleter());
+    return std::shared_ptr<void>(symbolic, UmfpackLongIntSymbolicDeleter());
   }
   else
   {
@@ -337,10 +343,10 @@ UmfpackLUSolver::umfpack_factorize_symbolic(std::size_t M, std::size_t N,
                  "Could not determine correct types for casting integers to pass to UMFPACK");
   }
 
-  return boost::shared_ptr<void>();
+  return std::shared_ptr<void>();
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<void>
+std::shared_ptr<void>
 UmfpackLUSolver::umfpack_factorize_numeric(const std::size_t* Ap,
                                            const std::size_t* Ai,
                                            const double* Ax,
@@ -352,7 +358,7 @@ UmfpackLUSolver::umfpack_factorize_numeric(const std::size_t* Ap,
   dolfin_assert(symbolic);
 
   void* numeric = 0;
-  boost::scoped_ptr<double> dnull;
+  std::unique_ptr<double> dnull;
 
   // Factorization step
   long int status = 0;
@@ -363,7 +369,7 @@ UmfpackLUSolver::umfpack_factorize_numeric(const std::size_t* Ap,
     status = umfpack_di_numeric(_Ap, _Ai, Ax, symbolic, &numeric, dnull.get(),
                                 dnull.get());
     umfpack_check_status(status, "numeric");
-    return boost::shared_ptr<void>(numeric, UmfpackIntNumericDeleter());
+    return std::shared_ptr<void>(numeric, UmfpackIntNumericDeleter());
   }
   else if (sizeof(std::size_t) == sizeof(UF_long))
   {
@@ -372,7 +378,7 @@ UmfpackLUSolver::umfpack_factorize_numeric(const std::size_t* Ap,
     status = umfpack_dl_numeric(_Ap, _Ai, Ax, symbolic, &numeric, dnull.get(),
                                 dnull.get());
     umfpack_check_status(status, "numeric");
-    return boost::shared_ptr<void>(numeric, UmfpackLongIntNumericDeleter());
+    return std::shared_ptr<void>(numeric, UmfpackLongIntNumericDeleter());
   }
   else
   {
@@ -381,7 +387,7 @@ UmfpackLUSolver::umfpack_factorize_numeric(const std::size_t* Ap,
                  "Could not determine correct types for casting integers to pass to UMFPACK");
   }
 
-  return boost::shared_ptr<void>();
+  return std::shared_ptr<void>();
 }
 //-----------------------------------------------------------------------------
 void UmfpackLUSolver::umfpack_solve(const std::size_t* Ap,
@@ -396,7 +402,7 @@ void UmfpackLUSolver::umfpack_solve(const std::size_t* Ap,
   dolfin_assert(b);
   dolfin_assert(numeric);
 
-  boost::scoped_ptr<double> dnull;
+  std::unique_ptr<double> dnull;
 
   // Solve system. We assume CSR storage, but UMFPACK expects CSC, so solve
   // for the transpose
@@ -466,7 +472,7 @@ void UmfpackLUSolver::umfpack_check_status(long int status,
 //-----------------------------------------------------------------------------
 #else
 //-----------------------------------------------------------------------------
-boost::shared_ptr<void>
+std::shared_ptr<void>
 UmfpackLUSolver::umfpack_factorize_symbolic(std::size_t M, std::size_t N,
                                             const std::size_t* Ap,
                                             const std::size_t* Ai,
@@ -475,10 +481,10 @@ UmfpackLUSolver::umfpack_factorize_symbolic(std::size_t M, std::size_t N,
   dolfin_error("UmfpackLUSolver.cpp",
                "factorize matrix with UMFPACK LU solver (symbolic)",
                "UMFPACK has not been installed");
-  return boost::shared_ptr<void>();
+  return std::shared_ptr<void>();
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<void>
+std::shared_ptr<void>
 UmfpackLUSolver::umfpack_factorize_numeric(const std::size_t* Ap,
                                            const std::size_t* Ai,
                                            const double* Ax,
@@ -487,7 +493,7 @@ UmfpackLUSolver::umfpack_factorize_numeric(const std::size_t* Ap,
   dolfin_error("UmfpackLUSolver.cpp",
                "factorize matrix with UMFPACK LU solver (numeric)",
                "UMFPACK has not been installed");
-  return boost::shared_ptr<void>();
+  return std::shared_ptr<void>();
 }
 //-----------------------------------------------------------------------------
 void UmfpackLUSolver::umfpack_solve(const std::size_t* Ap,
