@@ -28,7 +28,7 @@
 
 #include <string>
 #include <utility>
-#include <boost/shared_ptr.hpp>
+#include <memory>
 #include <dolfin/common/types.h>
 #include "DefaultFactory.h"
 #include "GenericVector.h"
@@ -53,11 +53,11 @@ namespace dolfin
     }
 
     /// Create vector of size N
-    explicit Vector(std::size_t N)
+    Vector(MPI_Comm comm, std::size_t N)
     {
       DefaultFactory factory;
       vector = factory.create_vector();
-      vector->resize(N);
+      vector->init(comm, N);
     }
 
     /// Copy constructor
@@ -69,9 +69,9 @@ namespace dolfin
     //--- Implementation of the GenericTensor interface ---
 
     /// Return copy of vector
-    virtual boost::shared_ptr<GenericVector> copy() const
+    virtual std::shared_ptr<GenericVector> copy() const
     {
-      boost::shared_ptr<Vector> x(new Vector(*this));
+      std::shared_ptr<Vector> x(new Vector(*this));
       return x;
     }
 
@@ -83,24 +83,33 @@ namespace dolfin
     virtual void apply(std::string mode)
     { vector->apply(mode); }
 
+    /// Return MPI communicator
+    virtual MPI_Comm mpi_comm() const
+    { return vector->mpi_comm(); }
+
     /// Return informal string representation (pretty-print)
     virtual std::string str(bool verbose) const
     { return "<Vector wrapper of " + vector->str(verbose) + ">"; }
 
     //--- Implementation of the GenericVector interface ---
 
-    /// Resize vector to size N
-    virtual void resize(std::size_t N)
-    { vector->resize(N); }
+    /// Initialize vector to size N
+    virtual void init(MPI_Comm comm, std::size_t N)
+    { vector->init(comm, N); }
 
-    /// Resize vector with given ownership range
-    virtual void resize(std::pair<std::size_t, std::size_t> range)
-    { vector->resize(range); }
+    /// Initlialize vector with given ownership range
+    virtual void init(MPI_Comm comm, std::pair<std::size_t, std::size_t> range)
+    { vector->init(comm, range); }
 
-    /// Resize vector with given ownership range and with ghost values
-    virtual void resize(std::pair<std::size_t, std::size_t> range,
-                        const std::vector<la_index>& ghost_indices)
-    { vector->resize(range, ghost_indices); }
+    /// Initialize vector with given ownership range and with ghost
+    /// values
+    virtual void init(MPI_Comm comm,
+                      std::pair<std::size_t, std::size_t> range,
+                      const std::vector<la_index>& ghost_indices)
+    { vector->init(comm, range, ghost_indices); }
+
+    // Bring init function from GenericVector into scope
+    using GenericVector::init;
 
     /// Return true if vector is empty
     virtual bool empty() const
@@ -238,11 +247,6 @@ namespace dolfin
     virtual double* data()
     { return vector->data(); }
 
-    /// Update ghost values
-    virtual void update_ghost_values()
-    {
-      vector->update_ghost_values();
-    }
     //--- Special functions ---
 
     /// Return linear algebra backend factory
@@ -259,10 +263,10 @@ namespace dolfin
     virtual GenericVector* instance()
     { return vector.get(); }
 
-    virtual boost::shared_ptr<const LinearAlgebraObject> shared_instance() const
+    virtual std::shared_ptr<const LinearAlgebraObject> shared_instance() const
     { return vector; }
 
-    virtual boost::shared_ptr<LinearAlgebraObject> shared_instance()
+    virtual std::shared_ptr<LinearAlgebraObject> shared_instance()
     { return vector; }
 
     //--- Special Vector functions ---
@@ -274,7 +278,7 @@ namespace dolfin
   private:
 
     // Pointer to concrete implementation
-    boost::shared_ptr<GenericVector> vector;
+    std::shared_ptr<GenericVector> vector;
 
   };
 
