@@ -86,11 +86,16 @@ namespace dolfin
     /// Initialize zero tensor using tenor layout
     virtual void init(const TensorLayout& tensor_layout);
 
+    /// Return true if empty
+    virtual bool empty() const
+    { return size(0) == 0; }
+
     /// Return size of given dimension
     virtual std::size_t size(std::size_t dim) const;
 
     /// Return local ownership range
-    virtual std::pair<std::size_t, std::size_t> local_range(std::size_t dim) const
+    virtual std::pair<std::size_t, std::size_t>
+      local_range(std::size_t dim) const
     { return std::make_pair(0, size(dim)); }
 
     /// Set all entries to zero and keep any sparse structure
@@ -99,34 +104,43 @@ namespace dolfin
     /// Finalize assembly of tensor
     virtual void apply(std::string mode);
 
+    /// Return MPI communicator
+    virtual MPI_Comm mpi_comm() const
+    { return MPI_COMM_SELF; }
+
     /// Return informal string representation (pretty-print)
     virtual std::string str(bool verbose) const;
 
     //--- Implementation of the GenericMatrix interface ---
 
     /// Return copy of matrix
-    virtual boost::shared_ptr<GenericMatrix> copy() const;
+    virtual std::shared_ptr<GenericMatrix> copy() const;
 
     /// Resize matrix to M x N
     virtual void resize(std::size_t M, std::size_t N);
 
-    /// Resize vector z to be compatible with the matrix-vector product
+    /// Intialixe vector z to be compatible with the matrix-vector product
     /// y = Ax. In the parallel case, both size and layout are
     /// important.
     ///
     /// *Arguments*
     ///     dim (std::size_t)
     ///         The dimension (axis): dim = 0 --> z = y, dim = 1 --> z = x
-    virtual void resize(GenericVector& z, std::size_t dim) const;
+    virtual void init_vector(GenericVector& z, std::size_t dim) const;
 
     /// Get block of values
-    virtual void get(double* block, std::size_t m, const dolfin::la_index* rows, std::size_t n, const dolfin::la_index* cols) const;
+    virtual void get(double* block, std::size_t m, const dolfin::la_index* rows,
+                     std::size_t n, const dolfin::la_index* cols) const;
 
     /// Set block of values
-    virtual void set(const double* block, std::size_t m, const dolfin::la_index* rows, std::size_t n, const dolfin::la_index* cols);
+    virtual void set(const double* block, std::size_t m,
+                     const dolfin::la_index* rows, std::size_t n,
+                     const dolfin::la_index* cols);
 
     /// Add block of values
-    virtual void add(const double* block, std::size_t m, const dolfin::la_index* rows, std::size_t n, const dolfin::la_index* cols);
+    virtual void add(const double* block, std::size_t m,
+                     const dolfin::la_index* rows, std::size_t n,
+                     const dolfin::la_index* cols);
 
     /// Add multiple of given matrix (AXPY operation)
     virtual void axpy(double a, const GenericMatrix& A,
@@ -140,7 +154,8 @@ namespace dolfin
                         std::vector<double>& values) const;
 
     /// Set values for given row
-    virtual void setrow(std::size_t row_idx, const std::vector<std::size_t>& columns,
+    virtual void setrow(std::size_t row_idx,
+                        const std::vector<std::size_t>& columns,
                         const std::vector<double>& values);
 
     /// Set given rows to zero
@@ -155,6 +170,9 @@ namespace dolfin
     /// Matrix-vector product, y = A^T x
     virtual void transpmult(const GenericVector& x, GenericVector& y) const;
 
+    /// Set diagonal of a matrix
+    virtual void set_diagonal(const GenericVector& x);
+
     /// Multiply matrix by given number
     virtual const uBLASMatrix<Mat>& operator*= (double a);
 
@@ -166,7 +184,8 @@ namespace dolfin
 
     /// Return pointers to underlying compresssed storage data
     /// See GenericMatrix for documentation.
-    virtual boost::tuples::tuple<const std::size_t*, const std::size_t*, const double*, int> data() const;
+    virtual boost::tuples::tuple<const std::size_t*, const std::size_t*,
+      const double*, int> data() const;
 
     //--- Special functions ---
 
@@ -178,11 +197,11 @@ namespace dolfin
 
     /// Return reference to uBLAS matrix (const version)
     const Mat& mat() const
-    { return _A; }
+    { return _matA; }
 
     /// Return reference to uBLAS matrix (non-const version)
     Mat& mat()
-    { return _A; }
+    { return _matA; }
 
     /// Solve Ax = b out-of-place using uBLAS (A is not destroyed)
     void solve(uBLASVector& x, const uBLASVector& b) const;
@@ -201,19 +220,20 @@ namespace dolfin
 
     /// Access value of given entry
     double operator() (dolfin::la_index i, dolfin::la_index j) const
-    { return _A(i, j); }
+    { return _matA(i, j); }
 
     /// Assignment operator
     const uBLASMatrix<Mat>& operator= (const uBLASMatrix<Mat>& A);
 
   private:
 
-    /// General uBLAS LU solver which accepts both vector and matrix right-hand sides
+    // General uBLAS LU solver which accepts both vector and matrix
+    // right-hand sides
     template<typename B>
     void solve_in_place(B& X);
 
     // uBLAS matrix object
-    Mat _A;
+    Mat _matA;
 
   };
 
@@ -221,20 +241,21 @@ namespace dolfin
   // Implementation of uBLASMatrix
   //---------------------------------------------------------------------------
   template <typename Mat>
-  uBLASMatrix<Mat>::uBLASMatrix() : GenericMatrix(), _A(0, 0)
+  uBLASMatrix<Mat>::uBLASMatrix() : GenericMatrix(), _matA(0, 0)
   {
     // Do nothing
   }
   //---------------------------------------------------------------------------
   template <typename Mat>
   uBLASMatrix<Mat>::uBLASMatrix(std::size_t M, std::size_t N)
-    : GenericMatrix(), _A(M, N)
+    : GenericMatrix(), _matA(M, N)
   {
     // Do nothing
   }
   //---------------------------------------------------------------------------
   template <typename Mat>
-  uBLASMatrix<Mat>::uBLASMatrix(const uBLASMatrix& A) : GenericMatrix(), _A(A._A)
+  uBLASMatrix<Mat>::uBLASMatrix(const uBLASMatrix& A)
+    : GenericMatrix(), _matA(A._matA)
   {
     // Do nothing
   }
@@ -246,9 +267,9 @@ namespace dolfin
   }
   //---------------------------------------------------------------------------
   template <typename Mat>
-  boost::shared_ptr<GenericMatrix> uBLASMatrix<Mat>::copy() const
+  std::shared_ptr<GenericMatrix> uBLASMatrix<Mat>::copy() const
   {
-    boost::shared_ptr<GenericMatrix> A(new uBLASMatrix<Mat>(*this));
+    std::shared_ptr<GenericMatrix> A(new uBLASMatrix<Mat>(*this));
     return A;
   }
   //---------------------------------------------------------------------------
@@ -257,7 +278,7 @@ namespace dolfin
   {
     // Resize matrix
     if( size(0) != M || size(1) != N )
-      _A.Mat::resize(M, N, false);
+      _matA.Mat::resize(M, N, false);
   }
   //---------------------------------------------------------------------------
   template <typename Mat>
@@ -271,18 +292,18 @@ namespace dolfin
     }
 
     dolfin_assert(dim < 2);
-    return (dim == 0 ? _A.Mat::size1() : _A.Mat::size2());
+    return (dim == 0 ? _matA.Mat::size1() : _matA.Mat::size2());
   }
   //---------------------------------------------------------------------------
   template <typename Mat>
   double uBLASMatrix<Mat>::norm(std::string norm_type) const
   {
     if (norm_type == "l1")
-      return norm_1(_A);
+      return norm_1(_matA);
     else if (norm_type == "linf")
-      return norm_inf(_A);
+      return norm_inf(_matA);
     else if (norm_type == "frobenius")
-      return norm_frobenius(_A);
+      return norm_frobenius(_matA);
     else
     {
       dolfin_error("uBLASMatrix.h",
@@ -301,7 +322,7 @@ namespace dolfin
     dolfin_assert(row_idx < this->size(0));
 
     // Reference to matrix row
-    const ublas::matrix_row<const Mat> row(_A, row_idx);
+    const ublas::matrix_row<const Mat> row(_matA, row_idx);
 
     // Insert values into std::vectors
     columns.clear();
@@ -313,7 +334,7 @@ namespace dolfin
       values.push_back(*component );
     }
   }
-  //-----------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   template <typename Mat>
   void uBLASMatrix<Mat>::setrow(std::size_t row_idx,
                                 const std::vector<std::size_t>& columns,
@@ -322,27 +343,28 @@ namespace dolfin
     dolfin_assert(columns.size() == values.size());
     dolfin_assert(row_idx < this->size(0));
 
-    ublas::matrix_row<Mat> row(_A, row_idx);
+    ublas::matrix_row<Mat> row(_matA, row_idx);
     dolfin_assert(columns.size() <= row.size());
 
     row *= 0;
     for(std::size_t i = 0; i < columns.size(); i++)
       row(columns[i])=values[i];
   }
-  //-----------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   template <typename Mat>
-  void uBLASMatrix<Mat>::resize(GenericVector& z, std::size_t dim) const
+  void uBLASMatrix<Mat>::init_vector(GenericVector& z, std::size_t dim) const
   {
-    z.resize(size(dim));
+    z.init(mpi_comm(), size(dim));
   }
-  //-----------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   template <typename Mat>
-  void uBLASMatrix<Mat>::set(const double* block, std::size_t m, const dolfin::la_index* rows,
+  void uBLASMatrix<Mat>::set(const double* block, std::size_t m,
+                             const dolfin::la_index* rows,
                              std::size_t n, const dolfin::la_index* cols)
   {
     for (std::size_t i = 0; i < m; i++)
       for (std::size_t j = 0; j < n; j++)
-        _A(rows[i] , cols[j]) = block[i*n + j];
+        _matA(rows[i] , cols[j]) = block[i*n + j];
   }
   //---------------------------------------------------------------------------
   template <typename Mat>
@@ -351,46 +373,47 @@ namespace dolfin
   {
     for (std::size_t i = 0; i < m; i++)
       for (std::size_t j = 0; j < n; j++)
-        _A(rows[i] , cols[j]) += block[i*n + j];
+        _matA(rows[i] , cols[j]) += block[i*n + j];
   }
   //---------------------------------------------------------------------------
   template <typename Mat>
-  void uBLASMatrix<Mat>::get(double* block, std::size_t m, const dolfin::la_index* rows,
+  void uBLASMatrix<Mat>::get(double* block, std::size_t m,
+                             const dolfin::la_index* rows,
                              std::size_t n, const dolfin::la_index* cols) const
   {
     for(std::size_t i = 0; i < m; ++i)
       for(std::size_t j = 0; j < n; ++j)
-        block[i*n + j] = _A(rows[i], cols[j]);
+        block[i*n + j] = _matA(rows[i], cols[j]);
   }
   //---------------------------------------------------------------------------
   template <typename Mat>
   void uBLASMatrix<Mat>::lump(uBLASVector& m) const
   {
     const std::size_t n = size(1);
-    m.resize(n);
+    m.init(mpi_comm(), n);
     m.zero();
     ublas::scalar_vector<double> one(n, 1.0);
-    ublas::axpy_prod(_A, one, m.vec(), true);
+    ublas::axpy_prod(_matA, one, m.vec(), true);
   }
-  //-----------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   template <typename Mat>
   void uBLASMatrix<Mat>::solve(uBLASVector& x, const uBLASVector& b) const
   {
     // Make copy of matrix and vector
     uBLASMatrix<Mat> Atemp;
     Atemp.mat().resize(size(0), size(1));
-    Atemp.mat().assign(_A);
+    Atemp.mat().assign(_matA);
     x.vec().resize(b.vec().size());
     x.vec().assign(b.vec());
 
     // Solve
     Atemp.solve_in_place(x.vec());
   }
-  //-----------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   template <typename Mat>
   void uBLASMatrix<Mat>::solve_in_place(uBLASVector& x, const uBLASVector& b)
   {
-    const std::size_t M = _A.size1();
+    const std::size_t M = _matA.size1();
     dolfin_assert(M == b.size());
 
     // Initialise solution vector
@@ -401,12 +424,12 @@ namespace dolfin
     // Solve
     solve_in_place(x.vec());
   }
-  //-----------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   template <typename Mat>
   void uBLASMatrix<Mat>::invert()
   {
-    const std::size_t M = _A.size1();
-    dolfin_assert(M == _A.size2());
+    const std::size_t M = _matA.size1();
+    dolfin_assert(M == _matA.size2());
 
     // Create indentity matrix
     Mat X(M, M);
@@ -414,7 +437,7 @@ namespace dolfin
 
     // Solve
     solve_in_place(X);
-    _A.assign_temporary(X);
+    _matA.assign_temporary(X);
   }
   //---------------------------------------------------------------------------
   template <typename Mat>
@@ -423,18 +446,18 @@ namespace dolfin
     // Iterate through no-zero pattern and zero entries
     typename Mat::iterator1 row;    // Iterator over rows
     typename Mat::iterator2 entry;  // Iterator over entries
-    for (row = _A.begin1(); row != _A.end1(); ++row)
+    for (row = _matA.begin1(); row != _matA.end1(); ++row)
       for (entry = row.begin(); entry != row.end(); ++entry)
         *entry = 0;
   }
-  //-----------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   template <typename Mat>
   void uBLASMatrix<Mat>::zero(std::size_t m, const dolfin::la_index* rows)
   {
     for(std::size_t i = 0; i < m; ++i)
-      ublas::row(_A, rows[i]) *= 0.0;
+      ublas::row(_matA, rows[i]) *= 0.0;
   }
-  //-----------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   template <typename Mat>
   void uBLASMatrix<Mat>::ident(std::size_t m, const dolfin::la_index* rows)
   {
@@ -445,7 +468,7 @@ namespace dolfin
     bool diagonal_missing = true;
     typename Mat::iterator1 row;    // Iterator over rows
     typename Mat::iterator2 entry;  // Iterator over entries
-    for (row = _A.begin1(); row != _A.end1(); ++row)
+    for (row = _matA.begin1(); row != _matA.end1(); ++row)
     {
       entry = row.begin();
       if (std::find(_rows.begin(), _rows.end(), entry.index1()) != _rows.end())
@@ -488,10 +511,8 @@ namespace dolfin
     }
 
     // Resize RHS if empty
-    if (yy.size() == 0)
-    {
-      resize(yy, 0);
-    }
+    if (yy.empty())
+      init_vector(yy, 0);
 
     if (size(0) != yy.size())
     {
@@ -500,59 +521,88 @@ namespace dolfin
                    "Vector for matrix-vector result has wrong size");
     }
 
-    ublas::axpy_prod(_A, xx.vec(), yy.vec(), true);
+    ublas::axpy_prod(_matA, xx.vec(), yy.vec(), true);
   }
   //-----------------------------------------------------------------------------
+  template <class Mat>
+  void uBLASMatrix<Mat>::set_diagonal(const GenericVector& x)
+  {
+    if (size(1) != size(0) || size(0) != x.size())
+    {
+      dolfin_error("uBLASMatrix.h",
+                   "Set diagonal of a uBLAS Matrix",
+                   "Matrix and vector dimensions don't match");
+    }
+
+    const double* xx = x.down_cast<uBLASVector>().data();
+    typename Mat::iterator1 row;    // Iterator over rows
+    typename Mat::iterator2 entry;  // Iterator over entries
+
+    // FIXME: Cannot be a good way to do it for a dense matrices
+    for (row = _matA.begin1(); row != _matA.end1(); ++row)
+    {
+      for (entry = row.begin(); entry != row.end(); ++entry)
+      {
+	if (entry.index2() > entry.index1())
+	  break;
+	if (entry.index2() == entry.index1())
+	  *entry = xx[entry.index1()];
+      }
+    }
+  }
+  //----------------------------------------------------------------------------
   template <typename Mat>
-  void uBLASMatrix<Mat>::transpmult(const GenericVector& x, GenericVector& y) const
+  void uBLASMatrix<Mat>::transpmult(const GenericVector& x,
+                                    GenericVector& y) const
   {
     dolfin_error("uBLASMatrix.h",
                  "compute transpose matrix-vector product",
                  "Not supported by the uBLAS linear algebra backend");
   }
-  //-----------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   template <typename Mat>
   const uBLASMatrix<Mat>& uBLASMatrix<Mat>::operator*= (double a)
    {
-    _A *= a;
+    _matA *= a;
     return *this;
   }
-  //-----------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   template <typename Mat>
   const uBLASMatrix<Mat>& uBLASMatrix<Mat>::operator/= (double a)
   {
-    _A /= a;
+    _matA /= a;
     return *this;
   }
-  //-----------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   template <typename Mat>
   const GenericMatrix& uBLASMatrix<Mat>::operator= (const GenericMatrix& A)
   {
     *this = as_type<const uBLASMatrix<Mat> >(A);
     return *this;
   }
-  //-----------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   template <typename Mat>
-  inline const uBLASMatrix<Mat>& uBLASMatrix<Mat>::operator= (const uBLASMatrix<Mat>& A)
+  inline const
+    uBLASMatrix<Mat>& uBLASMatrix<Mat>::operator= (const uBLASMatrix<Mat>& A)
   {
     // Check for self-assignment
     if (this != &A)
     {
       // Assume uBLAS take care of deleting an existing Matrix
       // using its assignment operator
-      _A = A.mat();
+      _matA = A.mat();
     }
     return *this;
   }
-  //-----------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   template <typename Mat>
   inline void uBLASMatrix<Mat>::compress()
   {
     Mat A_temp(this->size(0), this->size(1));
-    A_temp.assign(_A);
-    _A.swap(A_temp);
+    A_temp.assign(_matA);
+    _matA.swap(A_temp);
   }
-  //-----------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   template <typename Mat>
   std::string uBLASMatrix<Mat>::str(bool verbose) const
   {
@@ -564,7 +614,7 @@ namespace dolfin
     if (verbose)
     {
       s << str(false) << std::endl << std::endl;
-      for (it1 = _A.begin1(); it1 != _A.end1(); ++it1)
+      for (it1 = _matA.begin1(); it1 != _matA.end1(); ++it1)
       {
         s << "|";
         for (it2 = it1.begin(); it2 != it1.end(); ++it2)
@@ -585,18 +635,20 @@ namespace dolfin
 
     return s.str();
   }
-  //-----------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   // Specialised member functions (must be inlined to avoid link errors)
-  //-----------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   template <>
-  inline void uBLASMatrix<ublas_sparse_matrix>::init(const TensorLayout& tensor_layout)
+  inline void
+    uBLASMatrix<ublas_sparse_matrix>::init(const TensorLayout& tensor_layout)
   {
     resize(tensor_layout.size(0), tensor_layout.size(1));
-    _A.clear();
+    _matA.clear();
 
     // Get sparsity pattern
     dolfin_assert(tensor_layout.sparsity_pattern());
-    const SparsityPattern* pattern_pointer = dynamic_cast<const SparsityPattern*>(tensor_layout.sparsity_pattern().get());
+    const SparsityPattern* pattern_pointer
+      = dynamic_cast<const SparsityPattern*>(tensor_layout.sparsity_pattern().get());
     if (!pattern_pointer)
     {
       dolfin_error("uBLASMatrix.h",
@@ -605,22 +657,23 @@ namespace dolfin
     }
 
     // Reserve space for non-zeroes and get non-zero pattern
-    _A.reserve(pattern_pointer->num_nonzeros());
-    const std::vector<std::vector<std::size_t> > pattern = pattern_pointer->diagonal_pattern(SparsityPattern::sorted);
+    _matA.reserve(pattern_pointer->num_nonzeros());
+    const std::vector<std::vector<std::size_t> > pattern
+      = pattern_pointer->diagonal_pattern(SparsityPattern::sorted);
 
     // Add entries
     std::vector<std::vector<std::size_t> >::const_iterator row;
     Set<std::size_t>::const_iterator element;
     for(row = pattern.begin(); row != pattern.end(); ++row)
       for(element = row->begin(); element != row->end(); ++element)
-        _A.push_back(row - pattern.begin(), *element, 0.0);
+        _matA.push_back(row - pattern.begin(), *element, 0.0);
   }
   //---------------------------------------------------------------------------
   template <typename Mat>
   inline void uBLASMatrix<Mat>::init(const TensorLayout& tensor_layout)
   {
     resize(tensor_layout.size(0), tensor_layout.size(1));
-    _A.clear();
+    _matA.clear();
   }
   //---------------------------------------------------------------------------
   template <>
@@ -629,7 +682,7 @@ namespace dolfin
     Timer timer("Apply (matrix)");
 
     // Make sure matrix assembly is complete
-    _A.complete_index1_data();
+    _matA.complete_index1_data();
   }
   //---------------------------------------------------------------------------
   template <typename Mat>
@@ -650,20 +703,23 @@ namespace dolfin
                    "Dimensions don't match");
     }
 
-    _A += (a)*(as_type<const uBLASMatrix>(A).mat());
+    _matA += (a)*(as_type<const uBLASMatrix>(A).mat());
   }
   //---------------------------------------------------------------------------
   template <>
   inline boost::tuples::tuple<const std::size_t*, const std::size_t*,
-                         const double*, int> uBLASMatrix<ublas_sparse_matrix>::data() const
+                         const double*, int>
+    uBLASMatrix<ublas_sparse_matrix>::data() const
   {
-    typedef boost::tuples::tuple<const std::size_t*, const std::size_t*, const double*, int> tuple;
-    return tuple(&_A.index1_data()[0], &_A.index2_data()[0], &_A.value_data()[0], _A.nnz());
+    typedef boost::tuples::tuple<const std::size_t*, const std::size_t*,
+      const double*, int> tuple;
+    return tuple(&_matA.index1_data()[0], &_matA.index2_data()[0],
+                 &_matA.value_data()[0], _matA.nnz());
   }
   //---------------------------------------------------------------------------
   template <typename Mat>
-  inline boost::tuples::tuple<const std::size_t*, const std::size_t*, const double*, int>
-  uBLASMatrix<Mat>::data() const
+  inline boost::tuples::tuple<const std::size_t*, const std::size_t*,
+    const double*, int> uBLASMatrix<Mat>::data() const
   {
     dolfin_error("uBLASMatrix.h",
                  "return pointers to underlying matrix data",
@@ -677,14 +733,14 @@ namespace dolfin
   template<typename Mat> template<typename B>
   void uBLASMatrix<Mat>::solve_in_place(B& X)
   {
-    const std::size_t M = _A.size1();
-    dolfin_assert(M == _A.size2());
+    const std::size_t M = _matA.size1();
+    dolfin_assert(M == _matA.size2());
 
     // Create permutation matrix
     ublas::permutation_matrix<std::size_t> pmatrix(M);
 
     // Factorise (with pivoting)
-    std::size_t singular = ublas::lu_factorize(_A, pmatrix);
+    std::size_t singular = ublas::lu_factorize(_matA, pmatrix);
     if (singular > 0)
     {
       dolfin_error("uBLASMatrix.h",
@@ -694,9 +750,9 @@ namespace dolfin
     }
 
     // Back substitute
-    ublas::lu_substitute(_A, pmatrix, X);
+    ublas::lu_substitute(_matA, pmatrix, X);
   }
-  //-----------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
 }
 
 #endif
