@@ -59,7 +59,18 @@ Mesh::Mesh() : Variable("mesh", "DOLFIN mesh"),
                Hierarchical<Mesh>(*this),
                _cell_type(0),
                _ordered(false),
-               _cell_orientations(0)
+               _cell_orientations(0),
+               _mpi_comm(MPI_COMM_WORLD)
+{
+  // Do nothing
+}
+//-----------------------------------------------------------------------------
+Mesh::Mesh(MPI_Comm comm) : Variable("mesh", "DOLFIN mesh"),
+               Hierarchical<Mesh>(*this),
+               _cell_type(0),
+               _ordered(false),
+               _cell_orientations(0),
+               _mpi_comm(comm)
 {
   // Do nothing
 }
@@ -68,7 +79,8 @@ Mesh::Mesh(const Mesh& mesh) : Variable("mesh", "DOLFIN mesh"),
                                Hierarchical<Mesh>(*this),
                                _cell_type(0),
                                _ordered(false),
-                               _cell_orientations(0)
+                               _cell_orientations(0),
+                               _mpi_comm(MPI_COMM_WORLD)
 {
   *this = mesh;
 }
@@ -77,19 +89,27 @@ Mesh::Mesh(std::string filename) : Variable("mesh", "DOLFIN mesh"),
                                    Hierarchical<Mesh>(*this),
                                    _cell_type(0),
                                    _ordered(false),
-                                   _cell_orientations(0)
+                                   _cell_orientations(0),
+                                   _mpi_comm(MPI_COMM_WORLD)
 {
-  File file(filename);
+  File file(_mpi_comm, filename);
   file >> *this;
   _cell_orientations.resize(this->num_cells(), -1);
 }
 //-----------------------------------------------------------------------------
-Mesh::Mesh(LocalMeshData& local_mesh_data)
-                                 : Variable("mesh", "DOLFIN mesh"),
-                                   Hierarchical<Mesh>(*this),
-                                   _cell_type(0),
-                                   _ordered(false),
-                                   _cell_orientations(0)
+Mesh::Mesh(MPI_Comm comm, std::string filename)
+  : Variable("mesh", "DOLFIN mesh"), Hierarchical<Mesh>(*this),
+    _cell_type(0), _ordered(false), _cell_orientations(0), _mpi_comm(comm)
+{
+  File file(_mpi_comm, filename);
+  file >> *this;
+  _cell_orientations.resize(this->num_cells(), -1);
+}
+//-----------------------------------------------------------------------------
+Mesh::Mesh(MPI_Comm comm, LocalMeshData& local_mesh_data)
+  : Variable("mesh", "DOLFIN mesh"), Hierarchical<Mesh>(*this),
+    _cell_type(0), _ordered(false), _cell_orientations(0),
+    _mpi_comm(comm)
 {
   MeshPartitioning::build_distributed_mesh(*this, local_mesh_data);
 }
@@ -99,34 +119,35 @@ Mesh::Mesh(const CSGGeometry& geometry, std::size_t resolution)
     Hierarchical<Mesh>(*this),
     _cell_type(0),
     _ordered(false),
-    _cell_orientations(0)
-
+    _cell_orientations(0),
+    _mpi_comm(MPI_COMM_WORLD)
 {
   // Build mesh on process 0
-  if (MPI::process_number() == 0)
+  if (MPI::rank(_mpi_comm) == 0)
     CSGMeshGenerator::generate(*this, geometry, resolution);
 
   // Build distributed mesh
-  if (MPI::num_processes() > 1)
+  if (MPI::size(_mpi_comm) > 1)
     MeshPartitioning::build_distributed_mesh(*this);
 }
 //-----------------------------------------------------------------------------
-Mesh::Mesh(boost::shared_ptr<const CSGGeometry> geometry,
+Mesh::Mesh(std::shared_ptr<const CSGGeometry> geometry,
            std::size_t resolution)
   : Variable("mesh", "DOLFIN mesh"),
     Hierarchical<Mesh>(*this),
     _cell_type(0),
     _ordered(false),
-    _cell_orientations(0)
+    _cell_orientations(0),
+    _mpi_comm(MPI_COMM_WORLD)
 {
   assert(geometry);
 
   // Build mesh on process 0
-  if (MPI::process_number() == 0)
+  if (MPI::rank(_mpi_comm) == 0)
     CSGMeshGenerator::generate(*this, *geometry, resolution);
 
   // Build distributed mesh
-  if (MPI::num_processes() > 1)
+  if (MPI::size(_mpi_comm) > 1)
     MeshPartitioning::build_distributed_mesh(*this);
 }
 //-----------------------------------------------------------------------------
@@ -329,12 +350,12 @@ void Mesh::rotate(double angle, std::size_t axis, const Point& point)
   MeshTransformation::rotate(*this, angle, axis, point);
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<MeshDisplacement> Mesh::move(BoundaryMesh& boundary)
+std::shared_ptr<MeshDisplacement> Mesh::move(BoundaryMesh& boundary)
 {
   return ALE::move(*this, boundary);
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<MeshDisplacement> Mesh::move(Mesh& mesh)
+std::shared_ptr<MeshDisplacement> Mesh::move(Mesh& mesh)
 {
   return ALE::move(*this, mesh);
 }
@@ -392,7 +413,7 @@ Mesh::color(std::vector<std::size_t> coloring_type) const
   return MeshColoring::color(*_mesh, coloring_type);
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<BoundingBoxTree> Mesh::bounding_box_tree() const
+std::shared_ptr<BoundingBoxTree> Mesh::bounding_box_tree() const
 {
   // Allocate and build tree if necessary
   if (!_tree)
@@ -446,10 +467,16 @@ double Mesh::rmax() const
 //-----------------------------------------------------------------------------
 std::size_t Mesh::hash() const
 {
+  // Get local hashes
+  const std::size_t kt_local = _topology.hash();
+  const std::size_t kg_local = _geometry.hash();
+
+  // Compute global hash
+  const std::size_t kt = hash_global(_mpi_comm, kt_local);
+  const std::size_t kg = hash_global(_mpi_comm, kg_local);
+
   // Compute hash based on the Cantor pairing function
-  const std::size_t k1 = _topology.hash();
-  const std::size_t k2 = _geometry.hash();
-  return (k1 + k2)*(k1 + k2 + 1)/2 + k2;
+  return (kt + kg)*(kt + kg + 1)/2 + kg;
 }
 //-----------------------------------------------------------------------------
 std::string Mesh::str(bool verbose) const
