@@ -21,9 +21,9 @@
 // First added:  2006-05-19
 // Last changed: 2010-04-29
 
+#include <sstream>
 #include <boost/functional/hash.hpp>
 
-#include <dolfin/common/MPI.h>
 #include <dolfin/log/log.h>
 #include "MeshGeometry.h"
 
@@ -108,7 +108,6 @@ void MeshGeometry::set(std::size_t local_index,
 
   dolfin_assert(local_index < local_index_to_position.size());
   local_index_to_position[local_index] = local_index;
-
 }
 //-----------------------------------------------------------------------------
 std::size_t MeshGeometry::hash() const
@@ -116,19 +115,7 @@ std::size_t MeshGeometry::hash() const
   // Compute local hash
   boost::hash<std::vector<double> > dhash;
   const std::size_t local_hash = dhash(coordinates);
-
-  // Gather hash keys from all processes
-  std::vector<std::size_t> all_hashes;
-  MPI::gather(local_hash, all_hashes);
-
-  // Hash the received hash keys
-  boost::hash<std::vector<std::size_t> > sizet_hash;
-  std::size_t global_hash = sizet_hash(all_hashes);
-
-  // Broadcast hash key
-  MPI::broadcast(global_hash);
-
-  return global_hash;
+  return local_hash;
 }
 //-----------------------------------------------------------------------------
 std::string MeshGeometry::str(bool verbose) const
