@@ -26,8 +26,9 @@
 #include "Mesh.h"
 #include "BoundaryMesh.h"
 #include "Vertex.h"
-#include "Cell.h"
+#include "Edge.h"
 #include "Facet.h"
+#include "Cell.h"
 #include "MeshData.h"
 #include "SubDomain.h"
 #include "MeshSmoothing.h"
@@ -35,19 +36,22 @@
 using namespace dolfin;
 
 //-----------------------------------------------------------------------------
-void MeshSmoothing::smooth(Mesh& mesh, uint num_iterations)
+void MeshSmoothing::smooth(Mesh& mesh, std::size_t num_iterations)
 {
   log(PROGRESS, "Smoothing mesh");
 
   // Make sure we have cell-facet connectivity
   mesh.init(mesh.topology().dim(), mesh.topology().dim() - 1);
 
+  // Make sure we have vertex-edge connectivity
+  mesh.init(0, 1);
+
   // Make sure the mesh is ordered
   mesh.order();
 
   // Mark vertices on the boundary so we may skip them
-  BoundaryMesh boundary(mesh);
-  const MeshFunction<unsigned int> vertex_map = boundary.vertex_map();
+  BoundaryMesh boundary(mesh, "exterior");
+  const MeshFunction<std::size_t> vertex_map = boundary.entity_map(0);
   MeshFunction<bool> on_boundary(mesh, 0);
   on_boundary = false;
   if (boundary.num_vertices() > 0)
@@ -57,9 +61,9 @@ void MeshSmoothing::smooth(Mesh& mesh, uint num_iterations)
   }
 
   // Iterate over all vertices
-  const uint d = mesh.geometry().dim();
+  const std::size_t d = mesh.geometry().dim();
   std::vector<double> xx(d);
-  for (uint iteration = 0; iteration < num_iterations; iteration++)
+  for (std::size_t iteration = 0; iteration < num_iterations; iteration++)
   {
     for (VertexIterator v(mesh); !v.end(); ++v)
     {
@@ -72,21 +76,30 @@ void MeshSmoothing::smooth(Mesh& mesh, uint num_iterations)
       const Point p = v->point();
 
       // Compute center of mass of neighboring vertices
-      for (uint i = 0; i < d; i++) xx[i] = 0.0;
-      uint num_neighbors = 0;
-      for (VertexIterator vn(*v); !vn.end(); ++vn)
+      for (std::size_t i = 0; i < d; i++) xx[i] = 0.0;
+      std::size_t num_neighbors = 0;
+      for (EdgeIterator e(*v); !e.end(); ++e)
       {
+        // Get the other vertex
+        dolfin_assert(e->num_entities(0) == 2);
+        std::size_t other_index = e->entities(0)[0];
+        if (other_index == v->index())
+          other_index = e->entities(0)[1];
+
+        // Create the vertex
+        Vertex vn(mesh, other_index);
+
         // Skip the vertex itself
-        if (v->index() == vn->index())
+        if (v->index() == vn.index())
           continue;
         num_neighbors += 1;
 
         // Compute center of mass
-        const double* xn = vn->x();
-        for (uint i = 0; i < d; i++)
+        const double* xn = vn.x();
+        for (std::size_t i = 0; i < d; i++)
           xx[i] += xn[i];
       }
-      for (uint i = 0; i < d; i++)
+      for (std::size_t i = 0; i < d; i++)
         xx[i] /= static_cast<double>(num_neighbors);
 
       // Compute closest distance to boundary of star
@@ -94,7 +107,7 @@ void MeshSmoothing::smooth(Mesh& mesh, uint num_iterations)
       for (CellIterator c(*v); !c.end(); ++c)
       {
         // Get local number of vertex relative to facet
-        const uint local_vertex = c->index(*v);
+        const std::size_t local_vertex = c->index(*v);
 
         // Get normal of corresponding facet
         Point n = c->normal(local_vertex);
@@ -113,7 +126,7 @@ void MeshSmoothing::smooth(Mesh& mesh, uint num_iterations)
 
       // Move vertex at most a distance rmin / 2
       double r = 0.0;
-      for (uint i = 0; i < d; i++)
+      for (std::size_t i = 0; i < d; i++)
       {
         const double dx = xx[i] - x[i];
         r += dx*dx;
@@ -122,7 +135,7 @@ void MeshSmoothing::smooth(Mesh& mesh, uint num_iterations)
       if (r < DOLFIN_EPS)
         continue;
       rmin = std::min(0.5*rmin, r);
-      for (uint i = 0; i < d; i++)
+      for (std::size_t i = 0; i < d; i++)
         x[i] += rmin*(xx[i] - x[i])/r;
     }
   }
@@ -132,13 +145,13 @@ void MeshSmoothing::smooth(Mesh& mesh, uint num_iterations)
 }
 //-----------------------------------------------------------------------------
 void MeshSmoothing::smooth_boundary(Mesh& mesh,
-                                    uint num_iterations,
+                                    std::size_t num_iterations,
                                     bool harmonic_smoothing)
 {
   cout << "Smoothing boundary of mesh: " << mesh << endl;
 
   // Extract boundary of mesh
-  BoundaryMesh boundary(mesh);
+  BoundaryMesh boundary(mesh, "exterior");
 
   // Smooth boundary
   smooth(boundary, num_iterations);
@@ -154,16 +167,15 @@ void MeshSmoothing::snap_boundary(Mesh& mesh,
   cout << "Snapping boundary of mesh: " << mesh << endl;
 
   // Extract boundary of mesh
-  BoundaryMesh boundary(mesh);
+  BoundaryMesh boundary(mesh, "exterior");
 
-  const uint dim = mesh.geometry().dim();
-  Array<double> x;
+  const std::size_t dim = mesh.geometry().dim();
 
   // Smooth boundary
   MeshGeometry& geometry = boundary.geometry();
-  for (uint i = 0; i < boundary.num_vertices(); i++)
+  for (std::size_t i = 0; i < boundary.num_vertices(); i++)
   {
-    x.update(dim, geometry.x(i));
+    Array<double> x(dim, geometry.x(i));
     sub_domain.snap(x);
   }
 
@@ -181,13 +193,13 @@ void MeshSmoothing::move_interior_vertices(Mesh& mesh,
   else
   {
     // Use vertex map to update boundary coordinates of original mesh
-    const MeshFunction<unsigned int>& vertex_map = boundary.vertex_map();
-    const uint d = mesh.geometry().dim();
+    const MeshFunction<std::size_t>& vertex_map = boundary.entity_map(0);
+    const std::size_t d = mesh.geometry().dim();
     for (VertexIterator v(boundary); !v.end(); ++v)
     {
       const double* xb = v->x();
       double* xm = mesh.geometry().x(vertex_map[*v]);
-      for (uint i = 0; i < d; i++)
+      for (std::size_t i = 0; i < d; i++)
         xm[i] = xb[i];
     }
   }
