@@ -19,7 +19,7 @@
 // Modified by Kristoffer Selim 2008
 //
 // First added:  2007-12-12
-// Last changed: 2013-08-26
+// Last changed: 2014-01-06
 
 #ifndef __POINT_CELL_H
 #define __POINT_CELL_H
@@ -51,15 +51,17 @@ namespace dolfin
     std::size_t orientation(const Cell& cell) const;
 
     /// Create entities e of given topological dimension from vertices v
-    void create_entities(std::vector<std::vector<std::size_t> >& e, std::size_t dim,
-                         const unsigned int* v) const;
+    void create_entities(std::vector<std::vector<unsigned int> >& e,
+                         std::size_t dim, const unsigned int* v) const;
 
     /// Order entities locally (connectivity 1-0, 2-0, 2-1)
-    void order(Cell& cell,
-               const std::vector<std::size_t>& local_to_global_vertex_indices) const;
+    void
+      order(Cell& cell,
+            const std::vector<std::size_t>& local_to_global_vertex_indices) const;
 
     /// Refine cell uniformly
-    void refine_cell(Cell& cell, MeshEditor& editor, std::size_t& current_cell) const;
+    void refine_cell(Cell& cell, MeshEditor& editor,
+                     std::size_t& current_cell) const;
 
     /// Compute (generalized) volume (area) of triangle
     double volume(const MeshEntity& triangle) const;
@@ -90,6 +92,10 @@ namespace dolfin
 
     /// Check whether given entity collides with cell
     bool collides(const Cell& cell, const MeshEntity& entity) const;
+    
+    /// Compute triangulation of intersection of two cells
+    virtual std::vector<double>
+    triangulate_intersection(const Cell& c0, const Cell& c1) const;
 
     /// Return description of cell type
     std::string description(bool plural) const;
