@@ -45,16 +45,16 @@ using namespace dolfin;
 // in this file.  Used to search for plotter objects in get_plotter()
 // and to ensure that plotter objects are correctly destroyed when the
 // program terminates.
-static std::list<boost::shared_ptr<VTKPlotter> > stored_plotters;
+static std::list<std::shared_ptr<VTKPlotter> > stored_plotters;
 
 //-----------------------------------------------------------------------------
 // Function for getting already instantiated VTKPlotter for
 // the given object. If none is found, a new one is created.
-boost::shared_ptr<VTKPlotter> get_plotter(boost::shared_ptr<const Variable> obj, std::string key)
+std::shared_ptr<VTKPlotter> get_plotter(std::shared_ptr<const Variable> obj, std::string key)
 {
   log(TRACE, "Looking for cached VTKPlotter [%s].", key.c_str());
 
-  for (std::list<boost::shared_ptr<VTKPlotter> >::iterator it = stored_plotters.begin(); it != stored_plotters.end(); it++)
+  for (std::list<std::shared_ptr<VTKPlotter> >::iterator it = stored_plotters.begin(); it != stored_plotters.end(); it++)
   {
     if ( (*it)->key() == key && (*it)->is_compatible(obj) )
     {
@@ -65,7 +65,7 @@ boost::shared_ptr<VTKPlotter> get_plotter(boost::shared_ptr<const Variable> obj,
 
   // No previous plotter found, so create a new one
   log(TRACE, "No VTKPlotter found in cache, creating new plotter.");
-  boost::shared_ptr<VTKPlotter> plotter(new VTKPlotter(obj));
+  std::shared_ptr<VTKPlotter> plotter(new VTKPlotter(obj));
   plotter->set_key(key);
   stored_plotters.push_back(plotter);
 
@@ -73,8 +73,8 @@ boost::shared_ptr<VTKPlotter> get_plotter(boost::shared_ptr<const Variable> obj,
 }
 //-----------------------------------------------------------------------------
 // Function for plotting objects
-boost::shared_ptr<VTKPlotter> plot_object(boost::shared_ptr<const Variable> obj,
-                                          boost::shared_ptr<const Parameters> p,
+std::shared_ptr<VTKPlotter> plot_object(std::shared_ptr<const Variable> obj,
+                                          std::shared_ptr<const Parameters> p,
                                           std::string key)
 {
   // Get plotter from cache. Key given as parameter takes precedence.
@@ -84,7 +84,7 @@ boost::shared_ptr<VTKPlotter> plot_object(boost::shared_ptr<const Variable> obj,
     key = (std::string)*param_key;
   }
 
-  boost::shared_ptr<VTKPlotter> plotter = get_plotter(obj, key);
+  std::shared_ptr<VTKPlotter> plotter = get_plotter(obj, key);
 
   // Set plotter parameters
   plotter->parameters.update(*p);
@@ -95,9 +95,9 @@ boost::shared_ptr<VTKPlotter> plot_object(boost::shared_ptr<const Variable> obj,
   return plotter;
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<Parameters> new_parameters(std::string title, std::string mode)
+std::shared_ptr<Parameters> new_parameters(std::string title, std::string mode)
 {
-  boost::shared_ptr<Parameters> p(new Parameters());
+  std::shared_ptr<Parameters> p(new Parameters());
   if (!title.empty())
   {
     p->add("title", title);
@@ -111,33 +111,33 @@ void dolfin::interactive(bool really)
   VTKPlotter::all_interactive(really);
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<VTKPlotter> dolfin::plot(const Variable& var,
+std::shared_ptr<VTKPlotter> dolfin::plot(const Variable& var,
 					   std::string title,
 					   std::string mode)
 {
   return plot(reference_to_no_delete_pointer(var), title, mode);
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<VTKPlotter> dolfin::plot(boost::shared_ptr<const Variable> var,
+std::shared_ptr<VTKPlotter> dolfin::plot(std::shared_ptr<const Variable> var,
 					   std::string title, std::string mode)
 {
   return plot(var, new_parameters(title, mode));
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<VTKPlotter> dolfin::plot(const Variable& var,
+std::shared_ptr<VTKPlotter> dolfin::plot(const Variable& var,
 					   const Parameters& p)
 {
   return plot(reference_to_no_delete_pointer(var),
 	      reference_to_no_delete_pointer(p));
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<VTKPlotter> dolfin::plot(boost::shared_ptr<const Variable> var,
-					   boost::shared_ptr<const Parameters> p)
+std::shared_ptr<VTKPlotter> dolfin::plot(std::shared_ptr<const Variable> var,
+					   std::shared_ptr<const Parameters> p)
 {
   return plot_object(var, p, VTKPlotter::to_key(*var));
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<VTKPlotter> dolfin::plot(const Expression& expression,
+std::shared_ptr<VTKPlotter> dolfin::plot(const Expression& expression,
 					   const Mesh& mesh,
 					   std::string title, std::string mode)
 {
@@ -145,14 +145,14 @@ boost::shared_ptr<VTKPlotter> dolfin::plot(const Expression& expression,
 	      reference_to_no_delete_pointer(mesh), title, mode);
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<VTKPlotter> dolfin::plot(boost::shared_ptr<const Expression> expression,
-					   boost::shared_ptr<const Mesh> mesh,
+std::shared_ptr<VTKPlotter> dolfin::plot(std::shared_ptr<const Expression> expression,
+					   std::shared_ptr<const Mesh> mesh,
 					   std::string title, std::string mode)
 {
   return plot(expression, mesh, new_parameters(title, mode));
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<VTKPlotter> dolfin::plot(const Expression& expression,
+std::shared_ptr<VTKPlotter> dolfin::plot(const Expression& expression,
                                            const Mesh& mesh,
                                             const Parameters& p)
 {
@@ -161,11 +161,11 @@ boost::shared_ptr<VTKPlotter> dolfin::plot(const Expression& expression,
 	      reference_to_no_delete_pointer(p));
 }
 //-----------------------------------------------------------------------------
-boost::shared_ptr<VTKPlotter> dolfin::plot(boost::shared_ptr<const Expression> expression,
-					   boost::shared_ptr<const Mesh> mesh,
-					   boost::shared_ptr<const Parameters> p)
+std::shared_ptr<VTKPlotter> dolfin::plot(std::shared_ptr<const Expression> expression,
+					   std::shared_ptr<const Mesh> mesh,
+					   std::shared_ptr<const Parameters> p)
 {
-  boost::shared_ptr<const ExpressionWrapper>
+  std::shared_ptr<const ExpressionWrapper>
     wrapper(new ExpressionWrapper(expression, mesh));
   return plot_object(wrapper, p, VTKPlotter::to_key(*expression));
 }
