@@ -19,9 +19,10 @@ extension modules."""
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Johannes Ring, 2011
+# Modifie by Mikael Mortensen, 2014
 #
 # First added:  2009-08-15
-# Last changed: 2011-08-31
+# Last changed: 2014-02-14
 
 import sys
 import os
@@ -46,7 +47,7 @@ __all__ = ["compile_extension_module",
 # Bump the interface version if anything changes that invalidates cached
 # modules (not required for change in generated code, swig version or dolfin
 # version)
-_interface_version = 1
+_interface_version = 3
 
 # A list of supported math builtins
 _cpp_math_builtins = [
@@ -100,11 +101,7 @@ import_array();
 
 // Include global SWIG interface files:
 // Typemaps, shared_ptr declarations, exceptions, version
-%%include <boost_shared_ptr.i>
-
-%%{
-#define SWIG_SHARED_PTR_QNAMESPACE boost
-%%}
+%%include <std_shared_ptr.i>
 
 // Global typemaps and forward declarations
 %%include "dolfin/swig/typemaps/includes.i"
@@ -114,7 +111,7 @@ import_array();
 %%include <exception.i>
 %%include "dolfin/swig/exceptions.i"
 
-// Do not expand default arguments in C++ by generating two an extra 
+// Do not expand default arguments in C++ by generating two an extra
 // function in the SWIG layer. This reduces code bloat.
 %%feature("compactdefaultargs");
 
@@ -122,13 +119,10 @@ import_array();
 %%include <std_string.i>
 
 // Manually import ufc:
-%%shared_ptr(ufc::cell_integral)
-%%shared_ptr(ufc::dofmap)
-%%shared_ptr(ufc::finite_element)
-%%shared_ptr(ufc::function)
-%%shared_ptr(ufc::form)
-%%shared_ptr(ufc::exterior_facet_integral)
-%%shared_ptr(ufc::interior_facet_integral)
+%%{
+#include <ufc.h>
+%%}
+%%include "swig/ufc_shared_ptr_classes.i"
 %%import(module="ufc") "ufc.h"
 
 // Local shared_ptr declarations
@@ -201,7 +195,7 @@ def expression_to_code_fragments(expr, arguments, generic_function_members=None)
     # Generate code for member variables
     members_code = ["  double %s;" % name for name in variables \
                     if name not in generic_function_members]
-    members_code.extend("  boost::shared_ptr<dolfin::GenericFunction> shared_%s;" % name \
+    members_code.extend("  std::shared_ptr<dolfin::GenericFunction> shared_%s;" % name \
                         for name in generic_function_members)
     members_code = "\n".join(members_code)
 
@@ -220,11 +214,11 @@ def expression_to_code_fragments(expr, arguments, generic_function_members=None)
 def expect_list_of(argtype, arg, argname):
     if arg is None:
         return []
-    
+
     if isinstance(arg, (types.NoneType, list, tuple)):
         if all(isinstance(s, argtype) for s in arg):
             return arg
-    
+
     cpp.dolfin_error("compilemodule.py",
                      "ensure correct argument for compile_extension_module",
                      "Provide a 'tuple' or 'list' with '%s', for the "\
@@ -233,15 +227,15 @@ def expect_list_of(argtype, arg, argname):
 def expect_arg(argtype, arg, argname):
     # Check the type of the argument
     if isinstance(arg, argtype):
-        return 
-    
+        return
+
     cpp.dolfin_error("compilemodule.py",
                      "ensure correct argument for compile_extension_module",
                      "Provide a '%s', for the '%s' argument" % \
                      (argtype.__name__, argname))
-    
+
 def check_swig_version(compiled_module):
-    
+
     # Check swig version of compiled module
     if compiled_module and compiled_module.swigversion != cpp.__swigversion__:
         cpp.dolfin_error("compilemodule.py",
@@ -267,7 +261,7 @@ def compile_extension_module(code, module_name="",
             available, making it possible to interface with for example NumPy
             for Array<double/int> arguments. Source which is not wrapped in
             a dolfin namespace will be automatically wrapped.
-  
+
         module_name
             Force a name of the module. If not set a name based on the hex
             representation of the code will be used.
@@ -278,7 +272,7 @@ def compile_extension_module(code, module_name="",
         additional_system_headers :
             System headers needed to compile the generated can be included
             using this argument. The headers are passed using a list of 'str'
-        
+
     *Returns*
 
         The JIT compiled extension module
@@ -287,24 +281,25 @@ def compile_extension_module(code, module_name="",
 
         The following toy example shows how one can use compiled extension
         modules to access low level PETSc routines:
-        
+
         .. code-block:: python
 
             from numpy import arange
             code = '''
             namespace dolfin {
-            
-              void PETSc_exp(boost::shared_ptr<dolfin::PETScVector> vec) 
+
+              void PETSc_exp(std::shared_ptr<dolfin::PETScVector> vec)
               {
-                boost::shared_ptr<Vec> x = vec->vec();
+                Vec x = vec->vec();
                 assert(x);
-                VecExp(*x);
+                VecExp(x);
               }
             }
             '''
-            ext_module = compile_extension_module(code, 
+            ext_module = compile_extension_module(code,
                          additional_system_headers=["petscvec.h"])
-            vec = PETScVector(10)
+            comm = mpi_comm_world()
+            vec = PETScVector(comm, 10)
             vec[:] = arange(10)
             print vec[-1]
             ext_module.PETSc_exp(vec)
@@ -354,22 +349,22 @@ def compile_extension_module(code, module_name="",
     for dolfin_type in dolfin_type_def:
         for used_type in used_types:
             if dolfin_type in used_type:
-                
+
                 # Add bases and derived types
                 used_dolfin_types.extend(\
                     dolfin_type_def[dolfin_type]["bases"])
-                
+
                 # Add dolfin type
                 used_dolfin_types.append(dolfin_type)
 
                 break
-    
+
     # Generate dependency info
     dependencies = {}
     for dolfin_type in used_dolfin_types:
         if dolfin_type_def[dolfin_type]["submodule"] not in dependencies:
             dependencies[dolfin_type_def[dolfin_type]["submodule"]] = []
-        
+
         dependencies[dolfin_type_def[dolfin_type]["submodule"]].append(\
             dolfin_type_def[dolfin_type]["header"])
 
@@ -383,13 +378,13 @@ def compile_extension_module(code, module_name="",
         dependencies["common"].append("dolfin/common/types.h")
     else:
         dependencies["common"] = ["dolfin/common/types.h"]
-    
+
     # Sort the dependencies
     dependencies = sort_submodule_dependencies(dependencies, submodule_info)
 
     import_lines, headers_includes, file_dependencies = \
                   build_swig_import_info(dependencies, submodule_info, "dolfin.cpp.")
-    
+
     # Extract header info
     dolfin_system_headers = [header for header in file_dependencies \
                              if not "pre.i" in header]
@@ -408,13 +403,13 @@ def compile_extension_module(code, module_name="",
 
     sys.stdout.flush()
     dolfin.info("Calling DOLFIN just-in-time (JIT) compiler, this may take some time.")
-    
+
     # Configure instant and add additional system headers
-    
+
     # Add dolfin system headers
     instant_kwargs["system_headers"] = ["cmath", "iostream","complex",
                                         "stdexcept","numpy/arrayobject.h",
-                                        "boost/shared_ptr.hpp",
+                                        "memory",
                                         "dolfin/common/types.h",
                                         "dolfin/math/basic.h"] + \
                                         instant_kwargs.get("system_headers", [])
@@ -431,7 +426,7 @@ def compile_extension_module(code, module_name="",
     declaration_strs = {"additional_declarations":""}
     declaration_strs["dolfin_import_statement"] = \
                     "\n".join(import_lines)
-    
+
     # Add any provided additional declarations
     if additional_declarations is not None:
         declaration_strs["additional_declarations"] += additional_declarations
@@ -452,7 +447,7 @@ def compile_extension_module(code, module_name="",
     # Check that the swig version of the compiled module is the same as
     # dolfin was compiled with
     check_swig_version(compiled_module)
-    
+
     return compiled_module
 
 def extract_shared_ptr_declaration(declared_types, used_dolfin_types, shared_ptr_classes):
@@ -468,9 +463,9 @@ def extract_shared_ptr_declaration(declared_types, used_dolfin_types, shared_ptr
         if dolfin_type in shared_ptr_classes:
             used_shared_ptr_types.append(dolfin_type)
 
-    used_shared_ptr_types.extend(derived for derived, bases in declared_types.items() 
+    used_shared_ptr_types.extend(derived for derived, bases in declared_types.items()
                                  if any(base in shared_ptr_classes for base in bases))
-            
+
     shared_ptr_declarations = "\n".join([shared_ptr_format % derived \
                                          for derived in used_shared_ptr_types])
 
