@@ -141,7 +141,6 @@ def generate_error_control(problem, goal):
         :py:class:`ErrorControl <dolfin.cpp.ErrorControl>`
 
     """
-
     # Generate UFL forms to be used for error control
     (ufl_forms, is_linear) = generate_error_control_forms(problem, goal)
 
@@ -179,13 +178,30 @@ def generate_error_control_forms(problem, goal):
     msg = "Generating forms required for error control, this may take some time..."
     cpp.info(msg)
 
+    # Paranoid checks added after introduction of multidomain features in ufl:
+    for form in [goal]:
+        assert len(form.domains()) > 0, "Error control got as input a form with no domain!"
+        assert len(form.domains()) == 1, "Error control got as input a form with more than one domain!"
+
     # Extract primal forms from problem
     is_linear = True
     if isinstance(problem, LinearVariationalProblem):
         primal = (problem.a_ufl, problem.L_ufl)
+
+        # Paranoid checks added after introduction of multidomain features in ufl:
+        for form in primal:
+            assert len(form.domains()) > 0, "Error control got as input a form with no domain!"
+            assert len(form.domains()) == 1, "Error control got as input a form with more than one domain!"
+
     elif isinstance(problem, NonlinearVariationalProblem):
         is_linear = False
         primal = problem.F_ufl
+
+        # Paranoid checks added after introduction of multidomain features in ufl:
+        for form in [primal]:
+            assert len(form.domains()) > 0, "Error control got as input a form with no domain!"
+            assert len(form.domains()) == 1, "Error control got as input a form with more than one domain!"
+
     else:
         cpp.dolfin_error("adaptivesolving.py",
                          "generate forms required for error control",
@@ -198,4 +214,9 @@ def generate_error_control_forms(problem, goal):
     generator = DOLFINErrorControlGenerator(primal, goal, u)
     forms = generator.generate_all_error_control_forms()
 
+    # Paranoid checks added after introduction of multidomain features in ufl:
+    for form in forms:
+        assert len(form.domains()) > 0, "Error control produced a form with no domain!"
+        assert len(form.domains()) == 1, "Error control produced a form with more than one domain!"
+
     return (forms, is_linear)
