@@ -30,9 +30,6 @@ rely on the dolfin::Form class which is not used on the Python side."""
 # Modified by Johan Hake, 2008-2009.
 # Modified by Garth N. Wells, 2008-2013.
 # Modified by Joachim B. Haga, 2012.
-#
-# First added:  2007-08-15
-# Last changed: 2013-02-18
 
 __all__ = ["assemble", "assemble_system", "SystemAssembler"]
 
@@ -47,22 +44,82 @@ import dolfin.cpp as cpp
 # Local imports
 from dolfin.fem.form import *
 
+def _create_dolfin_form(form,
+                        form_compiler_parameters=None,
+                        # TODO: Remove these after 1.4 release:
+                        mesh=None,
+                        coefficients=None,
+                        function_spaces=None,
+                        # TODO: Remove these after 1.5 release:
+                        cell_domains=None,
+                        exterior_facet_domains=None,
+                        interior_facet_domains=None):
+
+    # First check if we got a cpp.Form which originates from cpp layer
+    if isinstance(form, cpp.Form) and not hasattr(form, "_compiled_form"):
+        # Then we just try to use that one
+        dolfin_form = form
+        # Set mesh if specified (important for functionals without a function spaces)
+        if mesh is not None:
+            cpp.warning("Passing mesh modifies the input dolfin Form!")
+            dolfin_form.set_mesh(mesh)
+        if function_spaces is not None:
+            cpp.warning("Ignoring function_spaces with a dolfin Form!")
+        if coefficients is not None:
+            cpp.warning("Ignoring coefficients with a dolfin Form!")
+        if form_compiler_parameters is not None:
+            cpp.warning("Ignoring form_compiler_parameters with a dolfin Form!")
+
+    else:
+        # Wrap form
+        dolfin_form = Form(form,
+                           form_compiler_parameters=form_compiler_parameters,
+                           # TODO: Remove these after 1.4 release:
+                           function_spaces=function_spaces,
+                           coefficients=coefficients,
+                           mesh=mesh)
+
+    # Attach domains to form
+    if cell_domains is not None:
+        if form is dolfin_form:
+            cpp.warning("Passing cell_domains modifies the input dolfin Form!")
+        if not isinstance(cell_domains, cpp.MeshFunctionSizet):
+            raise TypeError("expected a 'MeshFunction' of 'Sizet' or 'None', for argument 'cell_domains'")
+        dolfin_form.set_cell_domains(cell_domains);
+    if interior_facet_domains is not None:
+        if form is dolfin_form:
+            cpp.warning("Passing interior_facet_domains modifies the input dolfin Form!")
+        if not isinstance(interior_facet_domains, cpp.MeshFunctionSizet):
+            raise TypeError("expected a 'MeshFunction' of 'Sizet' or 'None', for argument 'interior_facet_domains'")
+        dolfin_form.set_interior_facet_domains(interior_facet_domains);
+    if exterior_facet_domains is not None:
+        if form is dolfin_form:
+            cpp.warning("Passing exterior_facet_domains modifies the input dolfin Form!")
+        if not isinstance(exterior_facet_domains, cpp.MeshFunctionSizet):
+            raise TypeError("expected a 'MeshFunction' of 'Sizet' or 'None', for argument 'exterior_facet_domains'")
+        dolfin_form.set_exterior_facet_domains(exterior_facet_domains);
+
+    return dolfin_form
+
 # JIT assembler
 def assemble(form,
              tensor=None,
-             mesh=None,
-             coefficients=None,
-             function_spaces=None,
-             cell_domains=None,
-             exterior_facet_domains=None,
-             interior_facet_domains=None,
-             reset_sparsity=True,
+             bcs=None,
+             form_compiler_parameters=None,
              add_values=False,
              finalize_tensor=True,
              keep_diagonal=False,
              backend=None,
-             form_compiler_parameters=None,
-             bcs=None):
+             # TODO: Remove after 1.5 release:
+             cell_domains=None,
+             exterior_facet_domains=None,
+             interior_facet_domains=None,
+             # TODO: Remove after 1.4 release:
+             mesh=None,
+             coefficients=None,
+             function_spaces=None,
+             reset_sparsity=None,
+             ):
     """
     Assemble the given form and return the corresponding tensor.
 
@@ -88,17 +145,16 @@ def assemble(form,
     If the form defines integrals over different subdomains,
     :py:class:`MeshFunctions <dolfin.cpp.MeshFunction>` over the
     corresponding topological entities defining the subdomains can be
-    provided. An instance of a :py:class:`SubDomain
-    <dolfin.cpp.SubDomain>` can also be passed for each subdomain.
+    provided.
 
     The implementation of the returned tensor is determined by the
     default linear algebra backend. This can be overridden by
     specifying a different backend.
 
     Each call to assemble() will create a new tensor. If the
-    ``tensor`` argument is provided, this will be used instead. If
-    ``reset_sparsity`` is set to False, the provided tensor will not be
-    reset to zero before assembling (adding) values to the tensor.
+    ``tensor`` argument is provided, this will be used instead.
+    Parameter ``reset_sparsity`` is deprecated. Sparsity pattern of
+    ``tensor`` is reset iff ``tensor.empty()`` holds.
 
     If the ``keep_diagonal`` is set to True, assembler ensures that
     potential zeros on a matrix diagonal are kept in sparsity pattern
@@ -148,55 +204,40 @@ def assemble(form,
         The form ``a`` is now assembled into the PETScMatrix ``A``.
 
     """
+    if reset_sparsity is not None:
+        cpp.deprecation("Parameter reset_sparsity of assembler",
+                        "1.4", "1.5",
+                        "Parameter reset_sparsity of assembler"
+                        " is no longer used. Tensor is reset iff empty().")
+    if (mesh is not None
+        or function_spaces is not None
+        or coefficients is not None):
+        cpp.deprecation("Parameters mesh, function_spaces and coefficients of assemble",
+                        "1.4.0", "1.5.0",
+                        "Parameter mesh, function_spaces and coefficients of assemble"
+                        " will be removed. Include this in the ufl form instead.")
+    if (cell_domains is not None
+        or exterior_facet_domains is not None
+        or interior_facet_domains is not None):
+        cpp.deprecation("Parameters *_domains of assemble",
+                        "1.4.0", "1.6.0",
+                        "Parameter *_domains of assemble"
+                        " will be removed. Include this in the ufl form instead.")
 
-    # Extract common cell from mesh (may be missing in form definition)
-    common_cell = None if mesh is None else mesh.ufl_cell()
-
-    # First check if we got a cpp.Form which originates from cpp layer
-    if isinstance(form, cpp.Form) and not hasattr(form, "_compiled_form"):
-
-        # Then we just try to use that one
-        dolfin_form = form
-
-    else:
-
-        # Wrap form
-        dolfin_form = Form(form,
-                           function_spaces=function_spaces,
-                           coefficients=coefficients,
-                           form_compiler_parameters=form_compiler_parameters,
-                           common_cell=common_cell)
-
-    # Set mesh if specified (important for functionals without a function spaces)
-    if mesh is not None:
-        dolfin_form.set_mesh(mesh)
+    # Create dolfin Form object referencing all data needed by assembler
+    dolfin_form = _create_dolfin_form(form, form_compiler_parameters,
+                                      mesh, coefficients, function_spaces,
+                                      cell_domains, exterior_facet_domains, interior_facet_domains)
 
     # Create tensor
     tensor = _create_tensor(form, dolfin_form.rank(), backend, tensor)
 
-    # Extract domains
-    cell_domains, exterior_facet_domains, interior_facet_domains = \
-        _extract_domains(dolfin_form.mesh(),
-                         cell_domains,
-                         exterior_facet_domains,
-                         interior_facet_domains)
-
-    # Attach domains to form 
-    if cell_domains is not None:
-        dolfin_form.set_cell_domains(cell_domains);
-    if interior_facet_domains is not None:
-        dolfin_form.set_interior_facet_domains(interior_facet_domains);
-    if exterior_facet_domains is not None:
-        dolfin_form.set_exterior_facet_domains(exterior_facet_domains);
-
     # Call C++ assemble function
     assembler = cpp.Assembler()
-    assembler.reset_sparsity = reset_sparsity
     assembler.add_values = add_values
     assembler.finalize_tensor = finalize_tensor
     assembler.keep_diagonal = keep_diagonal
-    assembler.assemble(tensor,
-                 dolfin_form)
+    assembler.assemble(tensor, dolfin_form)
 
     # Convert to float for scalars
     if dolfin_form.rank() == 0:
@@ -205,7 +246,7 @@ def assemble(form,
     # Apply (possibly list of) boundary conditions
     bcs = _wrap_in_list(bcs, 'bcs', cpp.DirichletBC)
     if bcs:
-        cpp.deprecation("Passing \"bcs\" to assemble", "1.3.0",
+        cpp.deprecation("Passing \"bcs\" to assemble", "1.3.0", "1.4.0",
                         "Apply DirichletBC manually after an assemble.")
     for bc in bcs:
         bc.apply(tensor)
@@ -218,21 +259,24 @@ def assemble_system(A_form,
                     b_form,
                     bcs=None,
                     x0=None,
-                    A_coefficients=None,
-                    b_coefficients=None,
-                    A_function_spaces=None,
-                    b_function_spaces=None,
-                    cell_domains=None,
-                    exterior_facet_domains=None,
-                    interior_facet_domains=None,
-                    reset_sparsity=True,
+                    form_compiler_parameters=None,
                     add_values=False,
                     finalize_tensor=True,
                     keep_diagonal=False,
                     A_tensor=None,
                     b_tensor=None,
                     backend=None,
-                    form_compiler_parameters=None):
+                    # TODO: Remove after 1.4 release:
+                    mesh=None,
+                    A_coefficients=None,
+                    b_coefficients=None,
+                    A_function_spaces=None,
+                    b_function_spaces=None,
+                    reset_sparsity=None,
+                    # TODO: Remove after 1.5 release:
+                    cell_domains=None,
+                    exterior_facet_domains=None,
+                    interior_facet_domains=None):
     """
     Assemble form(s) and apply any given boundary conditions in a
     symmetric fashion and return tensor(s).
@@ -266,47 +310,45 @@ def assemble_system(A_form,
        <dolfin.fem.assembling.assemble>`.
 
     """
+    if reset_sparsity is not None:
+        cpp.deprecation("Parameter reset_sparsity of assemble_system",
+                        "1.4", "1.5",
+                        "Parameter reset_sparsity of assemble_system"
+                        " is no longer used. Tensor is reset iff empty().")
+    if (mesh is not None
+        or A_function_spaces is not None
+        or b_function_spaces is not None
+        or A_coefficients is not None
+        or b_coefficients is not None):
+        cpp.deprecation("Parameters mesh, *_function_spaces and *_coefficients of assemble_system",
+                        "1.4.0", "1.5.0",
+                        "Parameters mesh, *_function_spaces and *_coefficients of assemble_system"
+                        " will be removed. Include this in the ufl form instead.")
+    if (cell_domains is not None
+        or exterior_facet_domains is not None
+        or interior_facet_domains is not None):
+        cpp.deprecation("Parameters *_domains of assemble_system",
+                        "1.4.0", "1.6.0",
+                        "Parameters *_domains of assemble_system"
+                        " will be removed. Include this in the ufl form instead.")
 
-    # Extract subdomains
-    subdomains = { "cell": cell_domains,
-                   "exterior_facet": exterior_facet_domains,
-                   "interior_facet": interior_facet_domains}
-
-    # Wrap forms
-    A_dolfin_form = Form(A_form, A_function_spaces, A_coefficients,
-                         subdomains, form_compiler_parameters)
-    b_dolfin_form = Form(b_form, b_function_spaces, b_coefficients,
-                         subdomains, form_compiler_parameters)
+    # Create dolfin Form objects referencing all data needed by assembler
+    A_dolfin_form = _create_dolfin_form(A_form, form_compiler_parameters,
+                                        mesh, A_coefficients, A_function_spaces,
+                                        cell_domains, exterior_facet_domains, interior_facet_domains)
+    b_dolfin_form = _create_dolfin_form(b_form, form_compiler_parameters,
+                                        mesh, b_coefficients, b_function_spaces,
+                                        cell_domains, exterior_facet_domains, interior_facet_domains)
 
     # Create tensors
     A_tensor = _create_tensor(A_form, A_dolfin_form.rank(), backend, A_tensor)
     b_tensor = _create_tensor(b_form, b_dolfin_form.rank(), backend, b_tensor)
 
-    # Extract domains
-    cell_domains, exterior_facet_domains, interior_facet_domains = \
-        _extract_domains(A_dolfin_form.mesh(),
-                         cell_domains,
-                         exterior_facet_domains,
-                         interior_facet_domains)
-
-    # Attach domains to form (from Mesh, in other cases form already
-    # has domain data attached)
-    if cell_domains is not None:
-        A_dolfin_form.set_cell_domains(cell_domains);
-        b_dolfin_form.set_cell_domains(cell_domains);
-    if interior_facet_domains is not None:
-        A_dolfin_form.set_interior_facet_domains(interior_facet_domains);
-        b_dolfin_form.set_interior_facet_domains(interior_facet_domains);
-    if exterior_facet_domains is not None:
-        A_dolfin_form.set_exterior_facet_domains(exterior_facet_domains);
-        b_dolfin_form.set_exterior_facet_domains(exterior_facet_domains);
-
     # Check bcs
     bcs = _wrap_in_list(bcs, 'bcs', cpp.DirichletBC)
 
     # Call C++ assemble function
     assembler = cpp.SystemAssembler(A_dolfin_form, b_dolfin_form, bcs)
-    assembler.reset_sparsity = reset_sparsity
     assembler.add_values = add_values
     assembler.finalize_tensor = finalize_tensor
     assembler.keep_diagonal = keep_diagonal
@@ -338,7 +380,7 @@ def _create_tensor(form, rank, backend, tensor):
 
     # Check backend argument
     if (not backend is None) and (not isinstance(backend, cpp.GenericLinearAlgebraFactory)):
-        raise TypeError, "Provide a GenericLinearAlgebraFactory as 'backend'"
+        raise TypeError("Provide a GenericLinearAlgebraFactory as 'backend'")
 
     # Create tensor
     if rank == 0:
@@ -350,60 +392,25 @@ def _create_tensor(form, rank, backend, tensor):
         if backend: tensor = backend.create_matrix()
         else:       tensor = cpp.Matrix()
     else:
-        raise RuntimeError, "Unable to create tensors of rank %d." % rank
+        raise RuntimeError("Unable to create tensors of rank %d." % rank)
 
     return tensor
 
-def _extract_domains(mesh,
-                     cell_domains,
-                     exterior_facet_domains,
-                     interior_facet_domains):
-
-    def check_domain_type(domain, domain_type):
-        if not isinstance(domain, (cpp.SubDomain,cpp.MeshFunctionSizet, types.NoneType)):
-            raise TypeError, "expected a 'SubDomain', 'MeshFunction' of 'Sizet' or 'None', for the '%s'"%domain_type
-
-    def build_mf(subdomain, dim):
-        " Builds a MeshFunction from a SubDomain"
-        mf = cpp.MeshFunction("size_t", mesh, dim)
-        mf.set_all(1)
-        subdomain.mark(mf,0)
-        return mf
-
-    # Type check of input
-    check_domain_type(cell_domains, "cell_domains")
-    check_domain_type(exterior_facet_domains, "exterior_facet_domains")
-    check_domain_type(interior_facet_domains, "interior_facet_domains")
-
-    # The cell dimension
-    cell_dim = mesh.topology().dim()
-
-    # Get cell_domains
-    if isinstance(cell_domains, cpp.SubDomain):
-        cell_domains = build_mf(cell_domains, cell_dim)
-
-    # Get exterior_facet_domains (may be stored as part of the mesh)
-    if isinstance(exterior_facet_domains, cpp.SubDomain):
-        exterior_facet_domains = build_mf(exterior_facet_domains, cell_dim-1)
-
-    # Get interior_facet_domains
-    if isinstance(interior_facet_domains, cpp.SubDomain):
-        interior_facet_domains = build_mf(interior_facet_domains, cell_dim-1)
-
-    return cell_domains, exterior_facet_domains, interior_facet_domains
-
 class SystemAssembler(cpp.SystemAssembler):
     __doc__ = cpp.SystemAssembler.__doc__
     def __init__(self, A_form, b_form,
                  bcs=None,
+                 form_compiler_parameters=None,
+                 # TODO: Remove after 1.4 release:
+                 mesh=None,
                  A_coefficients=None,
                  b_coefficients=None,
                  A_function_spaces=None,
                  b_function_spaces=None,
+                 # TODO: Remove after 1.5 release:
                  cell_domains=None,
                  exterior_facet_domains=None,
-                 interior_facet_domains=None,
-                 form_compiler_parameters=None):
+                 interior_facet_domains=None):
         """
         Create a SystemAssembler
 
@@ -415,47 +422,29 @@ class SystemAssembler(cpp.SystemAssembler):
            bcs (_DirichletBC_)
               A list or a single DirichletBC (optional)
         """
-        # Extract subdomains
-        subdomains = { "cell": cell_domains,
-                       "exterior_facet": exterior_facet_domains,
-                       "interior_facet": interior_facet_domains}
-
-        # Wrap forms
-        # First check if we got a cpp.Form which originates from cpp layer
-        if isinstance(A_form, cpp.Form) and not hasattr(A_form, "_compiled_form"):
-
-            # Then we just try to use that one
-            A_dolfin_form = A_form
-        else:
-            A_dolfin_form = Form(A_form, A_function_spaces, A_coefficients,
-                                 subdomains, form_compiler_parameters)
-            
-        if isinstance(b_form, cpp.Form) and not hasattr(b_form, "_compiled_form"):
-
-            # Then we just try to use that one
-            b_dolfin_form = b_form
-        else:
-            b_dolfin_form = Form(b_form, b_function_spaces, b_coefficients,
-                                 subdomains, form_compiler_parameters)
-        
-        # Extract domains
-        cell_domains, exterior_facet_domains, interior_facet_domains = \
-                      _extract_domains(A_dolfin_form.mesh(),
-                                       cell_domains,
-                                       exterior_facet_domains,
-                                       interior_facet_domains)
-
-        # Attach domains to form (from Mesh, in other cases form already
-        # has domain data attached)
-        if cell_domains is not None:
-            A_dolfin_form.set_cell_domains(cell_domains);
-            b_dolfin_form.set_cell_domains(cell_domains);
-        if interior_facet_domains is not None:
-            A_dolfin_form.set_interior_facet_domains(interior_facet_domains);
-            b_dolfin_form.set_interior_facet_domains(interior_facet_domains);
-        if exterior_facet_domains is not None:
-            A_dolfin_form.set_exterior_facet_domains(exterior_facet_domains);
-            b_dolfin_form.set_exterior_facet_domains(exterior_facet_domains);
+        if (mesh is not None
+            or A_function_spaces is not None
+            or b_function_spaces is not None
+            or A_coefficients is not None
+            or b_coefficients is not None):
+            cpp.deprecation("Parameters mesh, *_function_spaces and *_coefficients of SystemAssembler",
+                            "1.4.0", "1.5.0",
+                            "Parameters mesh, *_function_spaces and *_coefficients of SystemAssembler"
+                            " will be removed. Include this in the ufl form instead.")
+        if (cell_domains is not None
+            or exterior_facet_domains is not None
+            or interior_facet_domains is not None):
+            cpp.deprecation("Parameters *_domains of SystemAssembler",
+                            "1.4.0", "1.6.0",
+                            "Parameters *_domains of SystemAssembler"
+                            " will be removed. Include this in the ufl form instead.")
+        # Create dolfin Form objects referencing all data needed by assembler
+        A_dolfin_form = _create_dolfin_form(A_form, form_compiler_parameters,
+                                            mesh, A_coefficients, A_function_spaces,
+                                            cell_domains, exterior_facet_domains, interior_facet_domains)
+        b_dolfin_form = _create_dolfin_form(b_form, form_compiler_parameters,
+                                            mesh, b_coefficients, b_function_spaces,
+                                            cell_domains, exterior_facet_domains, interior_facet_domains)
 
         # Check bcs
         bcs = _wrap_in_list(bcs, 'bcs', cpp.DirichletBC)
