@@ -38,35 +38,50 @@ def adjoint(form, reordered_arguments=None):
 
     # Extract form arguments
     arguments = extract_arguments(form)
+    if any(arg.part() != None for arg in arguments):
+        cpp.dolfin_error("formmanipulation.py",
+                         "compute adjoint of form",
+                         "parts not supported")
     if not (len(arguments) == 2):
         cpp.dolfin_error("formmanipulation.py",
                          "compute adjoint of form",
                          "Form is not bilinear")
 
-    # Define new Argument(s) in the same spaces (NB: Order matters
-    # here!)
-    v_1 = Argument(arguments[1].function_space())
-    v_0 = Argument(arguments[0].function_space())
+    # Define new Argument(s) in the same spaces
+    # (NB: Order does not matter anymore here because number is absolute)
+    v_1 = Argument(arguments[1].function_space(), arguments[0].number(), arguments[0].part())
+    v_0 = Argument(arguments[0].function_space(), arguments[1].number(), arguments[1].part())
 
     # Call ufl.adjoint with swapped arguments as new arguments
     return ufl.adjoint(form, reordered_arguments=(v_1, v_0))
 
 adjoint.__doc__ = ufl.adjoint.__doc__
 
-def derivative(form, u, du=None):
+def derivative(form, u, du=None, coefficient_derivatives=None):
     if du is None:
+        # Get existing arguments from form and position the new one with the next argument number
+        from ufl.algorithms import extract_arguments
+        form_arguments = extract_arguments(form)
+
+        number = max([-1] + [arg.number() for arg in form_arguments]) + 1
+
+        if any(arg.part() is not None for arg in form_arguments):
+            cpp.dolfin_error("formmanipulation.py",
+                             "compute derivative of form",
+                             "Cannot automatically create third argument using parts, please supply one")
+        part = None
+
         if isinstance(u, Function):
             V = u.function_space()
-            du = Argument(V)
+            du = Argument(V, number, part)
         elif isinstance(u, (list,tuple)) and all(isinstance(w, Function) for w in u):
             V = MixedFunctionSpace([w.function_space() for w in u])
-            du = ufl.split(Argument(V))
+            du = ufl.split(Argument(V, number, part))
         else:
             cpp.dolfin_error("formmanipulation.py",
                              "compute derivative of form",
                              "Cannot automatically create third argument, please supply one")
-
-    return ufl.derivative(form, u, du)
+    return ufl.derivative(form, u, du, coefficient_derivatives)
 
 derivative.__doc__ = ufl.derivative.__doc__
 
