@@ -3,7 +3,7 @@
 <dolfin.functions.function.Function>`, including the standard
 :math:`L^2`-norm and other norms."""
 
-# Copyright (C) 2008 Anders Logg
+# Copyright (C) 2008-2014 Anders Logg
 #
 # This file is part of DOLFIN.
 #
@@ -19,12 +19,10 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
-#
-# First added:  2008-03-19
-# Last changed: 2009-12-11
 
 __all__ = ["norm", "errornorm"]
 
+import ufl
 from ufl import inner, grad, div, curl, dx, FiniteElement, VectorElement, Coefficient
 from math import sqrt
 
@@ -34,7 +32,7 @@ from dolfin.cpp import GenericVector, GenericFunction, Function, Mesh, error, Ve
 from dolfin.fem.assembling import assemble
 from dolfin.fem.interpolation import interpolate
 from dolfin.functions.function import Function
-from dolfin.functions.functionspace import FunctionSpace, VectorFunctionSpace
+from dolfin.functions.functionspace import FunctionSpace, VectorFunctionSpace, TensorFunctionSpace
 
 dim_to_shape = {1: "interval", 2: "triangle", 3: "tetrahedron"}
 
@@ -107,25 +105,32 @@ def norm(v, norm_type="L2", mesh=None):
                          "compute norm",
                          "Expecting a Mesh, not " + str(type(mesh)))
 
+    # Get mesh from function
+    if isinstance(v, Function) and mesh is None:
+        mesh = v.function_space().mesh()
+
+    # Define integration measure and domain
+    dx = ufl.dx(mesh)
+
     # Select norm type
     if isinstance(v, GenericVector):
         return v.norm(norm_type.lower())
 
     elif (isinstance(v, Coefficient) and isinstance(v, GenericFunction)):
         if norm_type.lower() == "l2":
-            M = inner(v, v)*dx()
+            M = v**2*dx
         elif norm_type.lower() == "h1":
-            M = inner(v, v)*dx() + inner(grad(v), grad(v))*dx()
+            M = (v**2 + grad(v)**2)*dx
         elif norm_type.lower() == "h10":
-            M = inner(grad(v), grad(v))*dx()
+            M = grad(v)**2*dx
         elif norm_type.lower() == "hdiv":
-            M = inner(v, v)*dx() + div(v)*div(v)*dx()
+            M = (v**2 + div(v)**2)*dx
         elif norm_type.lower() == "hdiv0":
-            M = div(v)*div(v)*dx()
+            M = div(v)**2*dx
         elif norm_type.lower() == "hcurl":
-            M = inner(v, v)*dx() + inner(curl(v), curl(v))*dx()
+            M = (v**2 + curl(v)**2)*dx
         elif norm_type.lower() == "hcurl0":
-            M = inner(curl(v), curl(v))*dx()
+            M = curl(v)**2*dx
         else:
             cpp.dolfin_error("norms.py",
                              "compute norm",
@@ -135,12 +140,8 @@ def norm(v, norm_type="L2", mesh=None):
                          "compute norm",
                          "Unknown object type. Must be a vector or a function")
 
-    # Get mesh
-    if isinstance(v, Function) and mesh is None:
-        mesh = v.function_space().mesh()
-
     # Assemble value
-    r = assemble(M, mesh=mesh, form_compiler_parameters={"representation": "quadrature"})
+    r = assemble(M, form_compiler_parameters={"representation": "quadrature"})
 
     # Check value
     if r < 0.0:
@@ -181,12 +182,12 @@ def errornorm(u, uh, norm_type="l2", degree_rise=3, mesh=None):
 
     .. code-block:: python
 
-        assemble(e*e*dx(), mesh)
+        assemble(e**2*dx(mesh))
 
     However, this is not stable w.r.t. round-off errors considering that
-    the form compiler may expand the expression above to::
+    the form compiler may expand(#) the expression above to::
 
-        e*e*dx() = u*u*dx() - 2*u*uh*dx() + uh*uh*dx()
+        e**2*dx = u**2*dx - 2*u*uh*dx + uh**2*dx
 
     and this might get further expanded into thousands of terms for
     higher order elements. Thus, the error will be evaluated by adding
@@ -198,6 +199,8 @@ def errornorm(u, uh, norm_type="l2", degree_rise=3, mesh=None):
     then subtracting the two fields (which is easy since they are
     expressed in the same basis) and then evaluating the integral.
 
+    (#) If using the tensor representation optimizations.
+    The quadrature represenation does not suffer from this problem.
     """
 
     # Check argument
@@ -221,11 +224,12 @@ def errornorm(u, uh, norm_type="l2", degree_rise=3, mesh=None):
                          "Missing mesh")
 
     # Get rank
-    if not u.value_rank() == uh.value_rank():
+    if not u.shape() == uh.shape():
         cpp.dolfin_error("norms.py",
                          "compute error norm",
-                         "Value ranks don't match")
-    rank = u.value_rank()
+                         "Value shapes don't match")
+    shape = u.shape()
+    rank = len(shape)
 
     # Check that uh is associated with a finite element
     if uh.ufl_element().degree() is None:
@@ -245,11 +249,9 @@ def errornorm(u, uh, norm_type="l2", degree_rise=3, mesh=None):
     if rank == 0:
         V = FunctionSpace(mesh, "Discontinuous Lagrange", degree)
     elif rank == 1:
-        V = VectorFunctionSpace(mesh, "Discontinuous Lagrange", degree)
-    else:
-        cpp.dolfin_error("norms.py",
-                         "compute error norm",
-                         "Can't handle elements of rank %d" % rank)
+        V = VectorFunctionSpace(mesh, "Discontinuous Lagrange", degree, dim=shape[0])
+    elif rank > 1:
+        V = TensorFunctionSpace(mesh, "Discontinuous Lagrange", degree, shape=shape)
 
     # Interpolate functions into finite element space
     pi_u = interpolate(u, V)
