@@ -35,7 +35,7 @@ create_compiled_expression_class.
 
 """
 
-# Copyright (C) 2008-2011 Johan Hake
+# Copyright (C) 2008-2014 Johan Hake
 #
 # This file is part of DOLFIN.
 #
@@ -53,9 +53,7 @@ create_compiled_expression_class.
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2008-2009.
-#
-# First added:  2008-11-03
-# Last changed: 2009-12-16
+# Modified by Martin S. Alnaes 2013-2014
 
 __all__ = ["Expression"]
 
@@ -86,7 +84,7 @@ class UserDefinedParameters(dict):
         assert(isinstance(expr, cpp.Expression))
         self._expr = expr
         dict.__init__(self, **kwargs)
-        
+
     def __setitem__(self, name, value):
         __doc__ = dict.__setitem__.__doc__
         if name not in self:
@@ -102,25 +100,25 @@ class UserDefinedParameters(dict):
         else:
             for name, value in other:
                 self[name] = value
-            
+
 def create_compiled_expression_class(cpp_base):
     # Check the cpp_base
     assert(isinstance(cpp_base, (types.ClassType, type)))
 
-    def __init__(self, cppcode, element=None, cell=None, \
+    def __init__(self, cppcode, element=None, cell=None, domain=None, \
                  degree=None, name=None, label=None, **kwargs):
         """Initialize the Expression """
         # Initialize the cpp base class first and extract value_shape
         cpp_base.__init__(self)
         value_shape = tuple(self.value_dimension(i) \
-                                  for i in range(self.value_rank()))
+                            for i in range(self.value_rank()))
 
         # Store the value_shape
         self._value_shape = value_shape
 
         # Select an appropriate element if not specified
         if element is None:
-            element = _auto_select_element_from_shape(value_shape, degree, cell)
+            element = _auto_select_element_from_shape(value_shape, degree, cell, domain)
         else:
             # Check that we have an element
             if not isinstance(element, ufl.FiniteElementBase):
@@ -135,7 +133,7 @@ def create_compiled_expression_class(cpp_base):
 
         # Initialize UFL base class
         self._ufl_element = element
-        ufl.Coefficient.__init__(self, self._ufl_element)
+        ufl.Coefficient.__init__(self, self._ufl_element, count=self.id())
 
         # Set default variables
         for member, value in kwargs.items():
@@ -188,10 +186,11 @@ def create_python_derived_expression_class(class_name, user_bases, user_dict):
 
     def __init__(self, *args, **kwargs):
         """Initialize the Expression"""
-        # Get element, cell and degree
+        # Get element, domain and degree
         element = kwargs.get("element", None)
         degree = kwargs.get("degree", None)
         cell = kwargs.get("cell", None)
+        domain = kwargs.get("domain", None)
         name = kwargs.get("name", None)
         label = kwargs.get("label", None)
 
@@ -201,30 +200,32 @@ def create_python_derived_expression_class(class_name, user_bases, user_dict):
             from operator import add
             # First count how many valid kwargs is passed
             num_valid_kwargs = reduce(add, [kwarg is not None \
-                                      for kwarg in [element, degree, cell, name, label]])
+                                      for kwarg in [element, degree, cell, domain, name, label]])
             if len(kwargs) != num_valid_kwargs:
                 raise TypeError, "expected only 'kwargs' from one of "\
-                      "'element', 'degree', 'cell', 'name' or 'label'"
+                      "'element', 'degree', 'cell', 'domain', 'name' or 'label'"
             if len(args) != 0:
                 raise TypeError, "expected no 'args'"
 
         # Select an appropriate element if not specified
         if element is None:
             element = _auto_select_element_from_shape(self.value_shape(),
-                                                      degree, cell)
+                                                      degree, cell, domain)
         elif isinstance(element, ufl.FiniteElementBase):
             pass
         else:
             raise TypeError, "The 'element' argument must be a UFL finite"\
                   " element."
 
-        # Initialize UFL base class
-        self._ufl_element = element
-        ufl.Coefficient.__init__(self, element)
-
         # Initialize cpp_base class
         cpp.Expression.__init__(self, list(element.value_shape()))
 
+        # Initialize UFL base class
+        ufl.Coefficient.__init__(self, element, count=self.id())
+
+        # Why do we store the ufl element here in addition to in ufl.Coefficient? It's available as self.element()...
+        self._ufl_element = element
+
         # Calling the user defined_init
         if user_init is not None:
             user_init(self, *args, **kwargs)
@@ -406,7 +407,7 @@ class Expression(object):
             {
             public:
 
-              boost::shared_ptr<MeshFunction<unsigned int> > cell_data;
+              std::shared_ptr<MeshFunction<std::size_t> > cell_data;
 
               MyFunc() : Expression()
               {
@@ -416,7 +417,7 @@ class Expression(object):
                       const ufc::cell& c) const
               {
                 assert(cell_data);
-                const Cell cell(cell_data->mesh(), c.index);
+                const Cell cell(*cell_data->mesh(), c.index);
                 switch ((*cell_data)[cell.index()])
                 {
                 case 0:
@@ -450,34 +451,34 @@ class Expression(object):
               class Delta : public Expression
               {
               public:
-              
+
                 Delta() : Expression() {}
-              
+
                 void eval(Array<double>& values, const Array<double>& data,
                           const ufc::cell& cell) const
                 { }
-              
-                void update(const boost::shared_ptr<const Function> u, 
+
+                void update(const std::shared_ptr<const Function> u,
                             double nu, double dt, double C1,
                             double U_infty, double chord)
                 {
-                  const boost::shared_ptr<const Mesh> mesh = u->function_space()->mesh();
-                  const boost::shared_ptr<const GenericDofMap> dofmap = u->function_space()->dofmap();
+                  const std::shared_ptr<const Mesh> mesh = u->function_space()->mesh();
+                  const std::shared_ptr<const GenericDofMap> dofmap = u->function_space()->dofmap();
                   const uint ncells = mesh->num_cells();
                   uint ndofs_per_cell;
-                  if (ncells > 0) 
+                  if (ncells > 0)
                   {
                     CellIterator cell(*mesh);
                     ndofs_per_cell = dofmap->cell_dimension(cell->index());
-                  } 
-                  else 
+                  }
+                  else
                   {
                      return;
                   }
                 }
               };
             }'''
-            
+
             e = Expression(code)
 
     *3. User-defined expressions by subclassing*
@@ -561,7 +562,7 @@ class Expression(object):
     # Set the meta class
     __metaclass__ = ExpressionMetaClass
 
-    def __new__(cls, cppcode=None, element=None, cell=None, degree=None, \
+    def __new__(cls, cppcode=None, element=None, cell=None, domain=None, degree=None, \
                 name=None, label=None, **kwargs):
 
         # If the __new__ function is called because we are instantiating
@@ -581,8 +582,8 @@ class Expression(object):
         cpp_base, members = compile_expressions([cppcode],
                                                 [generic_function_members])
         cpp_base, members = cpp_base[0], members[0]
-        
-        # Check passed default arguments 
+
+        # Check passed default arguments
         not_allowed = [n for n in dir(cls) if n[0] !="_"]
         not_allowed += ["cppcode", "user_parameters"]
         _check_kwargs(members, kwargs, not_allowed)
@@ -595,7 +596,7 @@ class Expression(object):
 
     # This method is only included so a user can check what arguments
     # one should use in IPython using tab completion
-    def __init__(self, cppcode=None, element=None, cell=None, degree=None, \
+    def __init__(self, cppcode=None, element=None, cell=None, domain=None, degree=None, \
                  name=None, label=None, **kwargs): pass
 
     # Reuse the docstring from __new__
@@ -725,12 +726,9 @@ class Expression(object):
             values_provided = False
             values = numpy.zeros(value_size, dtype='d')
 
-        # Check if a cell is defined
-        cell = self.ufl_element().cell()
-        cell_defined = cell is not None
-
-        if cell_defined:
-            dim = cell.geometric_dimension()
+        # Get dim if element has any domains
+        domains = self.ufl_element().domains()
+        dim = domains[0].geometric_dimension() if domains else None
 
         # Assume all args are x argument
         x = args
@@ -738,7 +736,7 @@ class Expression(object):
         # If only one x argument has been provided, unpack it if it's an iterable
         if len(x) == 1:
             if isinstance(x[0], cpp.Point):
-                if cell_defined:
+                if dim is not None:
                     x = [x[0][i] for i in xrange(dim)]
                 else:
                     x = [x[0][i] for i in xrange(3)]
@@ -755,10 +753,12 @@ class Expression(object):
         if len(x) == 0:
             raise TypeError, "coordinate argument too short"
 
-        if cell_defined:
+        if dim is None:
+            warning("Evaluating an Expression without knowing the right geometric dimension, assuming %d is correct." % len(x))
+        else:
             if len(x) != dim:
-                raise TypeError, "expected the geometry argument to be of "\
-                      "length %d"%dim
+                raise TypeError("expected the geometry argument to be of "\
+                                "length %d" % dim)
 
         # The actual evaluation
         self.eval(values, x)
@@ -779,19 +779,31 @@ def _check_cppcode(cppcode):
         raise TypeError, "Please provide a 'str', 'tuple' or 'list' for "\
               "the 'cppcode' argument."
 
-def _auto_select_element_from_shape(shape, degree=None, cell=None):
+def _auto_select_element_from_shape(shape, degree=None, cell=None, domain=None):
     "Automatically select an appropriate element from cppcode."
 
+    # Cell to domain conversion TODO: deprecate cell argument
+    if domain is None:
+        if cell is None:
+            pass # No domain will result in undefined element
+        else:
+            domain = ufl.as_domain(cell)
+    else:
+        if cell is None:
+            domain = ufl.as_domain(domain) # Got only domain, good!
+        else:
+            raise ValueError("Got both domain and cell, use only domain.")
+
     # Default element, change to quadrature when working
     Family = "Lagrange"
 
     # Check if scalar, vector or tensor valued
     if len(shape) == 0:
-        element = ufl.FiniteElement(Family, cell, degree)
+        element = ufl.FiniteElement(Family, domain, degree)
     elif len(shape) == 1:
-        element = ufl.VectorElement(Family, cell, degree, dim=shape[0])
+        element = ufl.VectorElement(Family, domain, degree, dim=shape[0])
     else:
-        element = ufl.TensorElement(Family, cell, degree, shape=shape)
+        element = ufl.TensorElement(Family, domain, degree, shape=shape)
 
     cpp.debug("Automatic selection of expression element: " + str(element))
 
@@ -802,7 +814,7 @@ def _check_kwargs(members, kwargs, not_allowed):
     Check that all kwargs passed is either scalars or scalar Constants,
     and that the name is allowed
     """
-    
+
     # Check passed default values
     if not all(member in kwargs for member in members):
         missing = []
@@ -817,10 +829,9 @@ def _check_kwargs(members, kwargs, not_allowed):
         if name in not_allowed:
             raise RuntimeError("Parameter name: '%s' is not allowed. It is "\
                                "part of the interface of Expression" % name)
-            
+
     if not (all(isinstance(value, (int, float)) or \
                 (isinstance(value, cpp.GenericFunction) and value.value_size()==1) \
                 for value in kwargs.values())):
         raise TypeError, "expected default arguments for member variables "\
               "to be scalars or a scalar GenericFunctions."
-
