@@ -1,6 +1,6 @@
 """This module handles the Function class in Python.
 """
-# Copyright (C) 2009 Johan Hake
+# Copyright (C) 2009-2014 Johan Hake
 #
 # This file is part of DOLFIN.
 #
@@ -17,8 +17,7 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 #
-# First added:  2009-10-06
-# Last changed: 2011-04-18
+# Modified by Martin S. Alnaes 2013-2014
 
 __all__ = ["Function", "TestFunction", "TrialFunction", "Argument",
            "TestFunctions", "TrialFunctions"]
@@ -247,89 +246,99 @@ class Function(ufl.Coefficient, cpp.Function):
 
     def __init__(self, *args, **kwargs):
         """Initialize Function."""
-        # Check arguments
+        # Initial quick check for valid arguments (other checks sprinkled below)
         if len(args) == 0:
             raise TypeError("expected 1 or more arguments")
 
-        if not isinstance(args[0], (FunctionSpaceBase, cpp.Function)):
-            raise TypeError("expected a FunctionSpace or a Function as argument 1")
-
-        # If using the copy constuctor
+        # Type switch on argument types
         if isinstance(args[0], Function):
             other = args[0]
-            # If using the copy constuctor
             if len(args) == 1:
-                # Instantiate base classes
-                cpp.Function.__init__(self, other)
-                ufl.Coefficient.__init__(self, other._element)
-                return
-
-            # If using sub-function constructor
-            elif len(args) == 2 and isinstance(args[1], int):
+                return self.__init_copy_constructor(other)
+            elif len(args) == 2:
                 i = args[1]
-                num_sub_spaces = other.function_space().num_sub_spaces()
-
-                if num_sub_spaces == 1:
-                    raise RuntimeError("No subfunctions to extract")
-                if not i < num_sub_spaces:
-                    raise RuntimeError("Can only extract subfunctions "
-                                       "with i = 0..%d"% num_sub_spaces)
-                cpp.Function.__init__(self, other, i)
-                ufl.Coefficient.__init__(self, self.function_space().ufl_element())
-                return
+                if not isinstance(i, int):
+                    raise TypeError("Invalid subfunction number %s" % (i,))
+                return self.__init_subfunction_constructor(other, i)
             else:
                 raise TypeError("expected one or two arguments when "
                                 "instantiating from another Function")
-
-        # If creating a dolfin.Function from a cpp.Function
         elif isinstance(args[0], cpp.Function):
-            if  len(args) == 1:
-
-                # Lets be agressive in abusing dynamic typing shall we...
-                self.__class__ = Function
-                self.__dict__ = args[0].__dict__
-
-                # Instantiate base classes
-                V = args[0].function_space()
-                ufl.Coefficient.__init__(self, V.ufl_element())
-                return
+            other = args[0]
+            if len(args) == 1:
+                # If creating a dolfin.Function from a cpp.Function
+                self.__init_cpp_copy_constructor(other)
             else:
                 raise TypeError("expected only one argument when passing cpp.Function"
                                 "to dolfin.Function constructor")
-
-        V = args[0]
-
-        # Instantiate ufl base class
-        ufl.Coefficient.__init__(self, V.ufl_element())
-
-        # Passing only the FunctionSpace
-        if len(args) == 1:
-            # Instantiate cpp base classes
-            cpp.Function.__init__(self, V)
-        elif len(args) == 2:
-            # If passing FunctionSpace together with cpp.Function
-            # Attached passed FunctionSpace and initialize the cpp.Function
-            # using the passed Function
-            if isinstance(args[1], cpp.Function):
-                if args[1].function_space().dim() != V.dim():
-                    raise ValueError("non matching dimensions on passed FunctionSpaces")
-
-                cpp.Function.__init__(self, args[1])
+        elif isinstance(args[0], FunctionSpaceBase):
+            V = args[0]
+            # If initialising from a FunctionSpace
+            if len(args) == 1:
+                # If passing only the FunctionSpace
+                self.__init_from_function_space(V)
+            elif len(args) == 2:
+                # If passing FunctionSpace together with cpp.Function
+                # Attached passed FunctionSpace and initialize the cpp.Function
+                # using the passed Function
+                other = args[1]
+                if isinstance(other, cpp.Function):
+                    self.__init_from_function_space_and_cpp_function(V, other)
+                else:
+                    self.__init_from_function_space_and_function(V, other)
             else:
-                cpp.Function.__init__(self, *args)
+                raise TypeError("too many arguments")
         else:
-            raise TypeError("too many arguments")
+            raise TypeError("expected a FunctionSpace or a Function as argument 1")
 
         # Set name as given or automatic
         name = kwargs.get("name") or "f_%d" % self.count()
         self.rename(name, "a Function")
 
+    def __init_copy_constructor(self, other):
+        cpp.Function.__init__(self, other)
+        ufl.Coefficient.__init__(self, other._element, count=self.id())
+
+    def __init_cpp_copy_constructor(self, other):
+        # Lets be agressive in abusing dynamic typing shall we...
+        self.__class__ = Function
+        self.__dict__ = other.__dict__
+
+        # (Not calling cpp.Function.__init__)
+        ufl.Coefficient.__init__(self, other.function_space().ufl_element(), count=self.id())
+
+    def __init_subfunction_constructor(self, other, i):
+        num_sub_spaces = other.function_space().num_sub_spaces()
+        if num_sub_spaces == 1:
+            raise RuntimeError("No subfunctions to extract")
+        if not i < num_sub_spaces:
+            raise RuntimeError("Can only extract subfunctions "
+                               "with i = 0..%d"% num_sub_spaces)
+
+        cpp.Function.__init__(self, other, i)
+        ufl.Coefficient.__init__(self, self.function_space().ufl_element(), count=self.id())
+
+    def __init_from_function_space(self, V):
+        cpp.Function.__init__(self, V)
+        ufl.Coefficient.__init__(self, V.ufl_element(), count=self.id())
+
+    def __init_from_function_space_and_cpp_function(self, V, other):
+        # Simple consistency checks on function spaces
+        if other.function_space().dim() != V.dim():
+            raise ValueError("non matching dimensions on passed FunctionSpaces")
+
+        cpp.Function.__init__(self, other)
+        ufl.Coefficient.__init__(self, V.ufl_element(), count=self.id())
+
+    def __init_from_function_space_and_function(self, V, other):
+        cpp.Function.__init__(self, V, other)
+        ufl.Coefficient.__init__(self, V.ufl_element(), count=self.id())
+
     def _sub(self, i, deepcopy = False):
         cpp.deprecation("Using Function._sub", "1.3.0",
                         "Use Function.sub instead")
         self.sub(i, deepcopy)
 
-
     def sub(self, i, deepcopy = False):
         """
         Return a sub function.
@@ -564,8 +573,8 @@ class Function(ufl.Coefficient, cpp.Function):
             values_provided = False
             values = numpy.zeros(value_size, dtype='d')
 
-        # Get the dimension of the cell
-        dim = self.ufl_element().cell().geometric_dimension()
+        # Get the geometric dimension we live in
+        dim = self.geometric_dimension() # (UFL function)
 
         # Assume all args are x argument
         x = args
@@ -616,14 +625,14 @@ class Argument(ufl.Argument):
 
     This is the overloaded PyDOLFIN variant.
     """
-    def __init__(self, V, index=None):
+    def __init__(self, V, number, part=None):
         if not isinstance(V, FunctionSpaceBase):
             if isinstance(V, ufl.FiniteElementBase):
                 raise TypeError(_ufl_dolfin_difference_message)
             else:
                 raise TypeError("Illegal argument for creation of Argument, not a FunctionSpace: " + str(V))
             raise TypeError("Illegal argument for creation of Argument, not a FunctionSpace: " + str(V))
-        ufl.Argument.__init__(self, V.ufl_element(), index)
+        ufl.Argument.__init__(self, V.ufl_element(), number, part)
         self._V = V
 
     def function_space(self):
@@ -634,32 +643,33 @@ class Argument(ufl.Argument):
         """Extending UFL __eq__ here to distinguish test and trial
         functions in different function spaces with same ufl element."""
         return (isinstance(other, Argument) and
-                self._count == other._count and
+                self.number() == other.number() and
+                self.part() == other.part() and
                 self._V == other._V)
 
-def TestFunction(V):
+def TestFunction(V, part=None):
     """UFL value: Create a test function argument to a form.
 
     This is the overloaded PyDOLFIN variant.
     """
-    return Argument(V, -2)
+    return Argument(V, 0, part)
 
-def TrialFunction(V):
+def TrialFunction(V, part=None):
     """UFL value: Create a trial function argument to a form.
 
     This is the overloaded PyDOLFIN variant.
     """
-    return Argument(V, -1)
+    return Argument(V, 1, part)
 
 #--- TestFunctions and TrialFunctions ---
 
-def Arguments(V):
+def Arguments(V, number):
     """UFL value: Create an Argument in a mixed space, and return a
     tuple with the function components corresponding to the subelements.
 
     This is the overloaded PyDOLFIN variant.
     """
-    return ufl.split(Argument(V))
+    return ufl.split(Argument(V, number))
 
 def TestFunctions(V):
     """UFL value: Create a TestFunction in a mixed space, and return a
