@@ -30,7 +30,7 @@
 # Modified by Evan Lezar (add support for reading gmsh physical regions on facets)
 # Modified by Jan Blechta (add triangle support for marker on edges and attributes on triangles)
 #
-# Last changed: 2012-11-22
+# Last changed: 2014-02-06
 
 # NOTE: This module does not depend on (py)dolfin beeing installed.
 # NOTE: If future additions need that please import dolfin in a try: except:
@@ -383,7 +383,7 @@ def gmsh2xml(ifilename, handler):
             num_vertices = len(vertex_dict)
             handler.start_vertices(num_vertices)
             if process_facets:
-                mesh_editor.init_vertices ( num_vertices )
+                mesh_editor.init_vertices_global(num_vertices, num_vertices)
             state = 5
         elif state == 5:
             (node_no, x, y, z) = line.split()
@@ -418,7 +418,7 @@ def gmsh2xml(ifilename, handler):
         elif state == 8:
             handler.start_cells(num_cells_counted)
             if process_facets:
-                mesh_editor.init_cells( num_cells_counted )
+                mesh_editor.init_cells_global(num_cells_counted, num_cells_counted)
 
             state = 9
         elif state == 9:
@@ -817,7 +817,7 @@ def scotch_graph2graph_xml(ifilename, ofilename):
 
 
 def diffpack2xml(ifilename, ofilename):
-    "Convert from Diffpack tetrahedral grid format to DOLFIN XML."
+    "Convert from Diffpack tetrahedral/triangle grid format to DOLFIN XML."
 
     print diffpack2xml.__doc__
 
@@ -844,8 +844,18 @@ def diffpack2xml(ifilename, ofilename):
 	    num_cells = int(re.match(r".*\s(\d+).*", line).group(1))
         if re.search(r"Number of nodes", line):
 	    num_vertices = int(re.match(r".*\s(\d+).*", line).group(1))
+        if re.search(r"Number of space dim.", line):
+            num_dims = int(re.match(r".*\s(\d+).*", line).group(1))
+
+    if num_dims == 3:
+        xml_writer.write_header_mesh(ofile, "tetrahedron", 3)
+        elem_type = "ElmT4n3D"
+        write_cell_func = xml_writer.write_cell_tetrahedron
+    else:
+        xml_writer.write_header_mesh(ofile, "triangle", 2)
+        elem_type = "ElmT3n2D"
+        write_cell_func = xml_writer.write_cell_triangle
 
-    xml_writer.write_header_mesh(ofile, "tetrahedron", 3)
     xml_writer.write_header_vertices(ofile, num_vertices)
 
     # Read & write vertices and collect markers for vertices
@@ -897,15 +907,19 @@ def diffpack2xml(ifilename, ofilename):
         if not v:
             continue
 
-        if v[1] != "ElmT4n3D":
-           _error("Only tetrahedral elements (ElmT4n3D) are implemented.")
+        if v[1] != elem_type:
+           _error("Only tetrahedral (ElmT4n3D) and triangular (ElmT3n2D) elements are implemented.")
 
         # Store Cell markers
         cell_markers.append(int(v[2]))
 
         # Sort vertex indices
         cell_indices = sorted(map(lambda x: int(x)-1, v[3:]))
-        xml_writer.write_cell_tetrahedron(ofile, cell_ind, *cell_indices)
+        write_cell_func(ofile, cell_ind, *cell_indices)
+
+        if num_dims == 2:
+            cell_ind += 1
+            continue
 
         # Check Facet info
         process_facet = set(range(4))
@@ -950,10 +964,10 @@ def diffpack2xml(ifilename, ofilename):
                                                         marker, local_facet)
         xml_writer.write_footer_meshvaluecollection(ofile)
 
-    xml_writer.write_header_meshvaluecollection(ofile, "m", 3, \
+    xml_writer.write_header_meshvaluecollection(ofile, "m", num_dims, \
                                                 len(cell_markers), "uint")
     for cell, marker in enumerate(cell_markers):
-        xml_writer.write_entity_meshvaluecollection(ofile, 2, cell, \
+        xml_writer.write_entity_meshvaluecollection(ofile, num_dims, cell, \
                                                     marker)
     xml_writer.write_footer_meshvaluecollection(ofile)
     xml_writer.write_footer_domains(ofile)
