@@ -73,10 +73,11 @@ def write_footer_edges(ofile):
     ofile.write("    </edges>\n")
     print "Found all edges"
 
-def write_vertex(ofile, vertex, x, y, z):
+def write_vertex(ofile, vertex, *args):
     "Write vertex"
-    ofile.write("      <vertex index=\"%d\" x=\"%.16e\" y=\"%.16e\" z=\"%.16e\"/>\n" % \
-        (vertex, x, y, z))
+    coords = " ".join(['%s="%.16e"' % (comp, num) for (comp, num) in zip(["x","y","z"], args)])
+    ofile.write('      <vertex index="%d" %s/>\n' % \
+                (vertex, coords))
 
 def write_graph_vertex(ofile, vertex, num_edges, weight = 1):
     "Write graph vertex"
