@@ -18,11 +18,12 @@
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2013-03-02
-# Last changed: 2013-03-04
+# Last changed: 2014-05-28
 
 import unittest
 from dolfin import UnitSquareMesh, BoundaryMesh, Expression, \
-                   CellFunction, SubMesh, Constant, MPI, MeshQuality
+                   CellFunction, SubMesh, Constant, MPI, MeshQuality,\
+                   mpi_comm_world
 
 class HarmonicSmoothingTest(unittest.TestCase):
 
@@ -59,52 +60,50 @@ class HarmonicSmoothingTest(unittest.TestCase):
         rmin = MeshQuality.radius_ratio_min_max(mesh)[0]
         self.assertTrue(rmin > magic_number)
 
-if MPI.num_processes() == 1:
+@unittest.skipIf(MPI.size(mpi_comm_world()) > 1, "Skipping unit test(s) not working in parallel")
+class ALETest(unittest.TestCase):
 
-    class ALETest(unittest.TestCase):
+    def test_ale(self):
 
-        def test_ale(self):
+        print ""
+        print "Testing ALE::move(Mesh& mesh0, const Mesh& mesh1)"
 
-            print ""
-            print "Testing ALE::move(Mesh& mesh0, const Mesh& mesh1)"
+        # Create some mesh
+        mesh = UnitSquareMesh(4, 5)
 
-            # Create some mesh
-            mesh = UnitSquareMesh(4, 5)
+        # Make some cell function
+        # FIXME: Initialization by array indexing is probably
+        #        not a good way for parallel test
+        cellfunc = CellFunction('size_t', mesh)
+        cellfunc.array()[0:4] = 0
+        cellfunc.array()[4:]  = 1
 
-            # Make some cell function
-            # FIXME: Initialization by array indexing is probably
-            #        not a good way for parallel test
-            cellfunc = CellFunction('size_t', mesh)
-            cellfunc.array()[0:4] = 0
-            cellfunc.array()[4:]  = 1
+        # Create submeshes - this does not work in parallel
+        submesh0 = SubMesh(mesh, cellfunc, 0)
+        submesh1 = SubMesh(mesh, cellfunc, 1)
 
-            # Create submeshes - this does not work in parallel
-            submesh0 = SubMesh(mesh, cellfunc, 0)
-            submesh1 = SubMesh(mesh, cellfunc, 1)
+        # Move submesh0
+        disp = Constant(("0.1", "-0.1"))
+        submesh0.move(disp)
 
-            # Move submesh0
-            disp = Constant(("0.1", "-0.1"))
-            submesh0.move(disp)
+        # Move and smooth submesh1 accordignly
+        submesh1.move(submesh0)
 
-            # Move and smooth submesh1 accordignly
-            submesh1.move(submesh0)
-
-            # Move mesh accordingly
-            parent_vertex_indices_0 = \
-                     submesh0.data().array('parent_vertex_indices', 0)
-            parent_vertex_indices_1 = \
-                     submesh1.data().array('parent_vertex_indices', 0)
-            mesh.coordinates()[parent_vertex_indices_0[:]] = \
-                     submesh0.coordinates()[:]
-            mesh.coordinates()[parent_vertex_indices_1[:]] = \
-                     submesh1.coordinates()[:]
-
-            # If test passes here then it is probably working
-            # Check for cell quality for sure
-            magic_number = 0.28
-            rmin = MeshQuality.radius_ratio_min_max(mesh)[0]
-            self.assertTrue(rmin > magic_number)
+        # Move mesh accordingly
+        parent_vertex_indices_0 = \
+            submesh0.data().array('parent_vertex_indices', 0)
+        parent_vertex_indices_1 = \
+            submesh1.data().array('parent_vertex_indices', 0)
+        mesh.coordinates()[parent_vertex_indices_0[:]] = \
+            submesh0.coordinates()[:]
+        mesh.coordinates()[parent_vertex_indices_1[:]] = \
+            submesh1.coordinates()[:]
 
+        # If test passes here then it is probably working
+        # Check for cell quality for sure
+        magic_number = 0.28
+        rmin = MeshQuality.radius_ratio_min_max(mesh)[0]
+        self.assertTrue(rmin > magic_number)
 
 if __name__ == "__main__":
     print ""
