@@ -14,7 +14,7 @@ import numpy
 #-------------- Preprocessing step -----------------
 
 # Create mesh and define function space
-mesh = UnitSquare(3, 2)
+mesh = UnitSquareMesh(3, 2)
 V = FunctionSpace(mesh, 'Lagrange', 1)
 
 # Define boundary segments for Neumann, Robin and Dirichlet conditions
