@@ -22,7 +22,7 @@
 # Modified by Martin Alnaes 2012
 #
 # First added:  2011-09-19
-# Last changed: 2012-10-16
+# Last changed: 2014-05-28
 
 import unittest
 import numpy
@@ -63,7 +63,7 @@ class DirichletBCTest(unittest.TestCase):
         bc1.apply(A1)
 
         self.assertAlmostEqual(A1.norm("frobenius"), A0.norm("frobenius"))
-        
+
     def test_get_values(self):
         mesh = UnitSquareMesh(8, 8)
         dofs = numpy.zeros(3, dtype="I")
@@ -92,23 +92,17 @@ class DirichletBCTest(unittest.TestCase):
         bc1 = DirichletBC(V, u1, 1)
         bc2 = DirichletBC(V, u2, 2)
         bc3 = DirichletBC(V, u3, 3)
-
         bcs = [bc1, bc2, bc3]
 
         L = f*v*dx
-
         b = assemble(L)
         [bc.apply(b) for bc in bcs]
-
         self.assertAlmostEqual(norm(b), 16.55294535724685)
 
+    @unittest.skipIf(MPI.size(mpi_comm_world()) > 1, "Skipping unit test(s) not working in parallel")
     def test_bc_for_piola_on_manifolds(self):
         "Testing DirichletBC for piolas over standard domains vs manifolds."
 
-        if MPI.num_processes() > 1:
-            # SubMesh not working in parallel (the rest should)
-            return
-
         n = 4
         side = CompiledSubDomain("near(x[2], 0.0)")
 
@@ -117,13 +111,16 @@ class DirichletBCTest(unittest.TestCase):
         mesh.init_cell_orientations(Expression(("0.0", "0.0", "1.0")))
 
         RT1 = lambda mesh: FunctionSpace(mesh, "RT", 1)
+        RT2 = lambda mesh: FunctionSpace(mesh, "RT", 2)
+        DRT1 = lambda mesh: FunctionSpace(mesh, "DRT", 1)
+        DRT2 = lambda mesh: FunctionSpace(mesh, "DRT", 2)
         BDM1 = lambda mesh: FunctionSpace(mesh, "BDM", 1)
         BDM2 = lambda mesh: FunctionSpace(mesh, "BDM", 2)
         N1curl1 = lambda mesh: FunctionSpace(mesh, "N1curl", 1)
         N2curl1 = lambda mesh: FunctionSpace(mesh, "N2curl", 1)
         N1curl2 = lambda mesh:FunctionSpace(mesh, "N1curl", 2)
         N2curl2 = lambda mesh: FunctionSpace(mesh, "N2curl", 2)
-        elements = [N1curl1, N2curl1,  N1curl2, N2curl2, RT1, BDM1, BDM2]
+        elements = [N1curl1, N2curl1,  N1curl2, N2curl2, RT1, RT2, BDM1, BDM2, DRT1, DRT2]
 
         for element in elements:
             V = element(mesh)
