@@ -1,6 +1,6 @@
 """Unit tests for the fem interface"""
 
-# Copyright (C) 2011 Johan Hake
+# Copyright (C) 2011-2014 Johan Hake
 #
 # This file is part of DOLFIN.
 #
@@ -17,67 +17,34 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 #
-# First added:  2011-12-02
-# Last changed: 2012-12-11
-#
 # Modified by Marie E. Rognes (meg@simula.no), 2012
+# Modified by Martin S. Alnaes (martinal@simula.no), 2014
 
 import unittest
 import numpy
 import ufl
 from dolfin import *
 
-class FormTest(unittest.TestCase):
-
-    def setUp(self):
-        self.mesh = UnitSquareMesh(10, 10)
-        self.V = FunctionSpace(self.mesh, "CG", 1)
-        self.f = Expression("sin(pi*x[0]*x[1])")
-        self.v = TestFunction(self.V)
-        self.u = TrialFunction(self.V)
-
-    def test_assemble(self):
-        ufl_form = self.f*self.u*self.v*dx
-
-        dolfin_form = Form(ufl_form)
-        ufc_form = dolfin_form._compiled_form
-
-        A_ufl_norm = assemble(ufl_form).norm("frobenius")
-        A_dolfin_norm = assemble(dolfin_form).norm("frobenius")
-        A_ufc_norm = assemble(ufc_form, coefficients=[self.f],
-                              function_spaces=[self.V, self.V]).norm("frobenius")
-
-        self.assertAlmostEqual(A_ufl_norm, A_dolfin_norm)
-        self.assertAlmostEqual(A_ufl_norm, A_ufc_norm)
-
 class FormTestsOverManifolds(unittest.TestCase):
 
     def setUp(self):
 
-        # Boundary mesh not running in parallel
-        if MPI.num_processes() > 1:
-            return
-
         # 1D in 2D spaces
         self.square = UnitSquareMesh(2, 2)
-        self.mesh1 = BoundaryMesh(self.square, "exterior")
-        self.V1 = FunctionSpace(self.mesh1, "CG", 1)
-        self.VV1 = VectorFunctionSpace(self.mesh1, "CG", 1)
-        self.Q1 = FunctionSpace(self.mesh1, "DG", 0)
+        self.square_boundary = BoundaryMesh(self.square, "exterior")
+        self.V1 = FunctionSpace(self.square_boundary, "CG", 1)
+        self.VV1 = VectorFunctionSpace(self.square_boundary, "CG", 1)
+        self.Q1 = FunctionSpace(self.square_boundary, "DG", 0)
 
         # 2D in 3D spaces
         self.cube = UnitCubeMesh(2, 2, 2)
-        self.mesh2 = BoundaryMesh(self.cube, "exterior")
-        self.V2 = FunctionSpace(self.mesh2, "CG", 1)
-        self.VV2 = VectorFunctionSpace(self.mesh2, "CG", 1)
-        self.Q2 = FunctionSpace(self.mesh2, "DG", 0)
+        self.cube_boundary = BoundaryMesh(self.cube, "exterior")
+        self.V2 = FunctionSpace(self.cube_boundary, "CG", 1)
+        self.VV2 = VectorFunctionSpace(self.cube_boundary, "CG", 1)
+        self.Q2 = FunctionSpace(self.cube_boundary, "DG", 0)
 
     def test_assemble_functional(self):
 
-        # Boundary mesh not running in parallel
-        if MPI.num_processes() > 1:
-            return
-
         u = Function(self.V1)
         u.vector()[:] = 1.0
         surfacearea = assemble(u*dx)
@@ -100,14 +67,11 @@ class FormTestsOverManifolds(unittest.TestCase):
 
     def test_assemble_linear(self):
 
-        # Boundary mesh not running in parallel
-        if MPI.num_processes() > 1:
-            return
-
         u = Function(self.V1)
         w = TestFunction(self.Q1)
         u.vector()[:] = 0.5
-        facetareas = assemble(u*w*dx).array().sum()
+        facetareas = MPI.sum(self.square_boundary.mpi_comm(),
+                             assemble(u*w*dx).array().sum())
         self.assertAlmostEqual(facetareas, 2.0)
 
         u = Function(self.V2)
@@ -115,7 +79,8 @@ class FormTestsOverManifolds(unittest.TestCase):
         u.vector()[:] = 0.5
         a = u*w*dx
         b = assemble(a)
-        facetareas = assemble(u*w*dx).array().sum()
+        facetareas = MPI.sum(self.cube_boundary.mpi_comm(),
+                             assemble(u*w*dx).array().sum())
         self.assertAlmostEqual(facetareas, 3.0)
 
         mesh = UnitSquareMesh(8, 8)
@@ -128,16 +93,15 @@ class FormTestsOverManifolds(unittest.TestCase):
         bu = TrialFunction(BV)
         bv = TestFunction(BV)
 
-        a = assemble(inner(u, v)*ds).array().sum()
-        b = assemble(inner(bu, bv)*dx).array().sum()
+        a = MPI.sum(mesh.mpi_comm(),
+                    assemble(inner(u, v)*ds).array().sum())
+        b = MPI.sum(bdry.mpi_comm(),
+                    assemble(inner(bu, bv)*dx).array().sum())
         self.assertAlmostEqual(a, b)
 
+    @unittest.skipIf(MPI.size(mpi_comm_world()) > 1, "Skipping unit test(s) not working in parallel")
     def test_assemble_bilinear_1D_2D(self):
 
-        # Boundary mesh not running in parallel
-        if MPI.num_processes() > 1:
-            return
-
         V = FunctionSpace(self.square, 'CG', 1)
         u = TrialFunction(V)
         v = TestFunction(V)
@@ -145,57 +109,82 @@ class FormTestsOverManifolds(unittest.TestCase):
         bu = TrialFunction(self.V1)
         bv = TestFunction(self.V1)
 
-        a = assemble(inner(u, v)*ds).array().sum()
-        b = assemble(inner(bu, bv)*dx).array().sum()
+        a = MPI.sum(self.square.mpi_comm(),
+                    assemble(inner(u, v)*ds).array().sum())
+        b = MPI.sum(self.square_boundary.mpi_comm(),
+                    assemble(inner(bu, bv)*dx).array().sum())
         self.assertAlmostEqual(a, b)
 
-        bottom = CompiledSubDomain("near(x[1], 0.0)")
-        foo = abs(assemble(inner(grad(u)[0], grad(v)[0])*ds(0),
-                           exterior_facet_domains=bottom).array()).sum()
-        BV = FunctionSpace(SubMesh(self.mesh1, bottom), "CG", 1)
+        # Assemble over subset of mesh facets
+        subdomain = CompiledSubDomain("near(x[1], 0.0)")
+        bottom = FacetFunctionSizet(self.square)
+        bottom.set_all(0)
+        subdomain.mark(bottom, 1)
+        dss = ds[bottom]
+        foo = MPI.sum(self.square.mpi_comm(),
+                      abs(assemble(inner(grad(u)[0], grad(v)[0])*dss(1)).array()).sum())
+        # Assemble over all cells of submesh created from subset of boundary mesh
+        bottom2 = CellFunctionSizet(self.square_boundary)
+        bottom2.set_all(0)
+        subdomain.mark(bottom2, 1)
+        BV = FunctionSpace(SubMesh(self.square_boundary, bottom2, 1), "CG", 1)
         bu = TrialFunction(BV)
         bv = TestFunction(BV)
-        bar = abs(assemble(inner(grad(bu), grad(bv))*dx).array()).sum()
+        bar = MPI.sum(self.square_boundary.mpi_comm(),
+                      abs(assemble(inner(grad(bu)[0], grad(bv)[0])*dx).array()).sum())
+        # Should give same result
         self.assertAlmostEqual(bar, foo)
 
+    @unittest.skipIf(MPI.size(mpi_comm_world()) > 1, "Skipping unit test(s) not working in parallel")
     def test_assemble_bilinear_2D_3D(self):
 
-        # Boundary mesh not running in parallel
-        if MPI.num_processes() > 1:
-            return
-
         V = FunctionSpace(self.cube, 'CG', 1)
         u = TrialFunction(V)
         v = TestFunction(V)
 
+        # V2 is a FunctionSpace over self.cube_boundary
         bu = TrialFunction(self.V2)
         bv = TestFunction(self.V2)
 
-        a = assemble(inner(u, v)*ds).array().sum()
-        b = assemble(inner(bu, bv)*dx).array().sum()
+        a = MPI.sum(self.cube.mpi_comm(),
+                    assemble(inner(u, v)*ds).array().sum())
+        b = MPI.sum(self.cube_boundary.mpi_comm(),
+                    assemble(inner(bu, bv)*dx).array().sum())
         self.assertAlmostEqual(a, b)
 
-        bottom = CompiledSubDomain("near(x[1], 0.0)")
-        foo = abs(assemble(inner(grad(u)[0], grad(v)[0])*ds(0),
-                           exterior_facet_domains=bottom).array()).sum()
-        BV = FunctionSpace(SubMesh(self.mesh1, bottom), "CG", 1)
+        # Assemble over subset of mesh facets
+        subdomain = CompiledSubDomain("near(x[1], 0.0)")
+        bottom = FacetFunctionSizet(self.cube)
+        bottom.set_all(0)
+        subdomain.mark(bottom, 1)
+        dss = ds[bottom]
+        foo = MPI.sum(self.cube.mpi_comm(),
+                      abs(assemble(inner(grad(u)[0], grad(v)[0])*dss(1)).array()).sum())
+        #foo = MPI.sum(self.cube.mpi_comm(),
+        #              abs(assemble(inner(grad(u)[0], grad(v)[0])*ds(1),
+        #                           exterior_facet_domains=bottom).array()).sum())
+        # Assemble over all cells of submesh created from subset of boundary mesh
+        bottom2 = CellFunctionSizet(self.cube_boundary)
+        bottom2.set_all(0)
+        subdomain.mark(bottom2, 1)
+        BV = FunctionSpace(SubMesh(self.cube_boundary, bottom2, 1), "CG", 1)
         bu = TrialFunction(BV)
         bv = TestFunction(BV)
-        bar = abs(assemble(inner(grad(bu), grad(bv))*dx).array()).sum()
+        bar = MPI.sum(self.cube_boundary.mpi_comm(),
+                      abs(assemble(inner(grad(bu)[0], grad(bv)[0])*dx).array()).sum())
+        # Should give same result
         self.assertAlmostEqual(bar, foo)
 
+@unittest.skipIf(MPI.size(mpi_comm_world()) > 1, "Skipping unit test(s) not working in parallel")
 class FormTestsOverFunnySpaces(unittest.TestCase):
 
     def setUp(self):
 
-        # Boundary mesh not running in parallel
-        if MPI.num_processes() > 1:
-            return
-
         # Set-up meshes
         n = 16
         plane = CompiledSubDomain("near(x[1], 1.0)")
         self.square = UnitSquareMesh(n, n)
+
         self.square3d = SubMesh(BoundaryMesh(UnitCubeMesh(n, n, n), "exterior"), plane)
 
         # Define global normal and create orientation map
@@ -213,10 +202,6 @@ class FormTestsOverFunnySpaces(unittest.TestCase):
 
     def test_basic_rt(self):
 
-        # Boundary mesh not running in parallel
-        if MPI.num_processes() > 1:
-            return
-
         f2 = Expression(("2.0", "1.0"))
         f3 = Expression(("1.0", "0.0", "2.0"))
 
@@ -248,8 +233,8 @@ class FormTestsOverFunnySpaces(unittest.TestCase):
         a3 = assemble(inner(w3, w3)*dx)
 
         # Compare various results
-        self.assertAlmostEqual((w2.vector() - pw2.vector()).norm("l2"), 0.0,
-                               places=6)
+        self.assertAlmostEqual((w2.vector() - pw2.vector()).norm("l2"), 0.0, \
+                               places=5)
         self.assertAlmostEqual(a3, 5.0)
         self.assertAlmostEqual(a2, a3)
         self.assertAlmostEqual(pa2, a2)
@@ -257,10 +242,6 @@ class FormTestsOverFunnySpaces(unittest.TestCase):
 
     def test_mixed_poisson_solve(self):
 
-        # Boundary mesh not running in parallel
-        if MPI.num_processes() > 1:
-            return
-
         f = Constant(1.0)
 
         # Solve mixed Poisson on standard unit square
@@ -283,152 +264,130 @@ class FormTestsOverFunnySpaces(unittest.TestCase):
         self.assertAlmostEqual(assemble(inner(w2, w2)*dx),
                                assemble(inner(w3, w3)*dx))
 
+@unittest.skipIf(MPI.size(mpi_comm_world()) > 1, "Skipping unit test(s) not working in parallel")
 class TestGeometricQuantitiesOverManifolds(unittest.TestCase):
 
     def setUp(self):
 
-        # Boundary mesh not running in parallel
-        if MPI.num_processes() > 1:
-            return
-
         m = 3
         self.m = m
+        self.cube = UnitCubeMesh(m, m, m)
+        self.cube_boundary = BoundaryMesh(self.cube, "exterior")
+
         plane = CompiledSubDomain("near(x[1], 0.0)")
-        self.mesh1 = BoundaryMesh(UnitSquareMesh(m, m), "exterior")
-        self.bottom1 = SubMesh(self.mesh1, plane)
-
-        self.mesh2 = BoundaryMesh(UnitCubeMesh(m, m, m), "exterior")
-        self.bottom2 = SubMesh(self.mesh2, plane)
+        self.square = UnitSquareMesh(m, m)
+        self.square_boundary = BoundaryMesh(self.square, "exterior")
+        self.bottom1 = SubMesh(self.square_boundary, plane)
 
+        self.bottom2 = SubMesh(self.cube_boundary, plane)
         line = CompiledSubDomain("near(x[0], 0.0)")
-        self.mesh3 = BoundaryMesh(SubMesh(self.mesh2, plane), "exterior")
+        self.mesh3 = BoundaryMesh(SubMesh(self.cube_boundary, plane), "exterior")
         self.bottom3 = SubMesh(self.mesh3, line)
 
     def test_normals_2D_1D(self):
-
-        # Boundary mesh not running in parallel
-        if MPI.num_processes() > 1:
-            return
-
         "Testing assembly of normals for 1D meshes embedded in 2D"
-        n = ufl.Cell("interval", geometric_dimension=2).n
+
+        n = FacetNormal(self.bottom1)
         a = inner(n, n)*ds
-        value_bottom1 = assemble(a, mesh=self.bottom1)
+        value_bottom1 = assemble(a)
         self.assertAlmostEqual(value_bottom1, 2.0)
         b = inner(n('+'), n('+'))*dS
-        b1 = assemble(b, mesh=self.bottom1)
+        b1 = assemble(b)
         c = inner(n('+'), n('-'))*dS
-        c1 = assemble(c, mesh=self.bottom1)
+        c1 = assemble(c)
         self.assertAlmostEqual(b1, self.m-1)
         self.assertAlmostEqual(c1, - b1)
 
     def test_normals_3D_1D(self):
         "Testing assembly of normals for 1D meshes embedded in 3D"
 
-        # Boundary mesh not running in parallel
-        if MPI.num_processes() > 1:
-            return
-
-        n = ufl.Cell("interval", geometric_dimension=3).n
+        n = FacetNormal(self.bottom3)
         a = inner(n, n)*ds
-        v1 = assemble(a, mesh=self.bottom3)
+        v1 = assemble(a)
         self.assertAlmostEqual(v1, 2.0)
         b = inner(n('+'), n('+'))*dS
-        b1 = assemble(b, mesh=self.bottom3)
+        b1 = assemble(b)
         c = inner(n('+'), n('-'))*dS
-        c1 = assemble(c, mesh=self.bottom3)
+        c1 = assemble(c)
         self.assertAlmostEqual(b1, self.m-1)
         self.assertAlmostEqual(c1, - b1)
 
     def test_normals_3D_2D(self):
         "Testing assembly of normals for 2D meshes embedded in 3D"
 
-        # Boundary mesh not running in parallel
-        if MPI.num_processes() > 1:
-            return
-
-        n = ufl.Cell("triangle", geometric_dimension=3).n
+        n = FacetNormal(self.bottom2)
         a = inner(n, n)*ds
-        v1 = assemble(a, mesh=self.bottom2)
+        v1 = assemble(a)
         self.assertAlmostEqual(v1, 4.0)
 
         b = inner(n('+'), n('+'))*dS
-        b1 = assemble(b, mesh=self.bottom2)
+        b1 = assemble(b)
         c = inner(n('+'), n('-'))*dS
-        c1 = assemble(c, mesh=self.bottom2)
+        c1 = assemble(c)
         self.assertAlmostEqual(c1, - b1)
 
     def test_cell_volume(self):
         "Testing assembly of volume for embedded meshes"
 
-        # Boundary mesh not running in parallel
-        if MPI.num_processes() > 1:
-            return
-
-        volume = ufl.Cell("interval", geometric_dimension=2).volume
+        volume = CellVolume(self.bottom1)
         a = volume*dx
-        b = assemble(a, mesh=self.bottom1)
+        b = assemble(a)
         self.assertAlmostEqual(b, 1.0/self.m)
 
-        volume = ufl.Cell("interval", geometric_dimension=3).volume
+        volume = CellVolume(self.bottom3)
         a = volume*dx
-        b = assemble(a, mesh=self.bottom3)
+        b = assemble(a)
         self.assertAlmostEqual(b, 1.0/self.m)
 
-        volume = ufl.Cell("triangle", geometric_dimension=3).volume
+        volume = CellVolume(self.bottom2)
         a = volume*dx
-        b = assemble(a, mesh=self.bottom2)
+        b = assemble(a)
         self.assertAlmostEqual(b, 1.0/(2*self.m*self.m))
 
     def test_circumradius(self):
         "Testing assembly of circumradius for embedded meshes"
 
-        # Boundary mesh not running in parallel
-        if MPI.num_processes() > 1:
-            return
-
-        r = ufl.Cell("interval", geometric_dimension=2).circumradius
+        r = Circumradius(self.bottom1)
         a = r*dx
-        b = assemble(a, mesh=self.bottom1)
+        b = assemble(a)
         self.assertAlmostEqual(b, 0.5*(1.0/self.m))
 
-        r = ufl.Cell("interval", geometric_dimension=3).circumradius
+        r = Circumradius(self.bottom3)
         a = r*dx
-        b = assemble(a, mesh=self.bottom3)
+        b = assemble(a)
         self.assertAlmostEqual(b, 0.5*(1.0/self.m))
 
-        r = ufl.Cell("triangle", geometric_dimension=2).circumradius
+        square = UnitSquareMesh(self.m, self.m)
+        r = Circumradius(square)
         a = r*dx
-        b0 = assemble(a, mesh=UnitSquareMesh(self.m, self.m))
-        r = ufl.Cell("triangle", geometric_dimension=3).circumradius
+        b0 = assemble(a)
+
+        r = Circumradius(self.bottom2)
         a = r*dx
-        b1 = assemble(a, mesh=self.bottom2)
+        b1 = assemble(a)
         self.assertAlmostEqual(b0, b1)
 
     def test_facetarea(self):
         "Testing assembly of facet area for embedded meshes"
 
-        # Boundary mesh not running in parallel
-        if MPI.num_processes() > 1:
-            return
-
-        area = ufl.Cell("interval", geometric_dimension=2).facet_area
+        area = FacetArea(self.bottom1)
         a = area*ds
-        b = assemble(a, mesh=self.bottom1)
+        b = assemble(a)
         self.assertAlmostEqual(b, 2.0)
 
-        area = ufl.Cell("interval", geometric_dimension=3).facet_area
+        area = FacetArea(self.bottom3)
         a = area*ds
-        b = assemble(a, mesh=self.bottom3)
+        b = assemble(a)
         self.assertAlmostEqual(b, 2.0)
 
-        area = ufl.Cell("triangle", geometric_dimension=2).facet_area
+        square = UnitSquareMesh(self.m, self.m)
+        area = FacetArea(square)
         a = area*ds
-        b0 = assemble(a, mesh=UnitSquareMesh(self.m, self.m))
+        b0 = assemble(a)
 
-        area = ufl.Cell("triangle", geometric_dimension=3).facet_area
+        area = FacetArea(self.bottom2)
         a = area*ds
-        b1 = assemble(a, mesh=self.bottom2)
+        b1 = assemble(a)
         self.assertAlmostEqual(b0, b1)
 
 if __name__ == "__main__":
