@@ -41,7 +41,7 @@ public:
       markers(mesh, "xml_value_collection_ref.xml");
 
     // Check size
-    CPPUNIT_ASSERT(dolfin::MPI::sum(markers.size()) == 6);
+    CPPUNIT_ASSERT(dolfin::MPI::sum(mesh.mpi_comm(), markers.size()) == 6);
 
     // Check sum of values
     const std::map<std::pair<std::size_t, std::size_t>, std::size_t>&
@@ -51,7 +51,7 @@ public:
     std::size_t sum = 0;
     for (it = values.begin(); it != values.end(); ++it)
       sum += it->second;
-    CPPUNIT_ASSERT(dolfin::MPI::sum(sum) == 48);
+    CPPUNIT_ASSERT(dolfin::MPI::sum(mesh.mpi_comm(), sum) == 48);
   }
 
 };
