@@ -29,7 +29,7 @@ class LocalMeshDataXML_IO(unittest.TestCase):
 
     def testRead(self):
         file = File("../snake.xml.gz")
-        localdata = cpp.LocalMeshData()
+        localdata = cpp.LocalMeshData(mpi_comm_world())
         file >> localdata
 
 if __name__ == "__main__":
